#ifndef FEPS_2011_H
#define FEPS_2011_H

//#define MAX_HOTSPOTS 100
#define MAX_LEVELS 50
#define MAX_SPECIES 20
//#define MAX_EMODELS 10
#define MAX_EMODELS 5
#define SPECIES_SIZE 25
#define MAX_HOURS 72
#define MAX_CWFIS 30
#define MAX_ALLOCATIONS 10
//#define NB_CHAR_PATH 240
#define NB_CHAR_PATH 120

// MAX_TIMESTEPS only used by feps.Qs[MAX_TIMESTEPS+1] below
#define MAX_TIMESTEPS 1440 // 24 hours of 1 minute timesteps will be the maximum array size
//#define MAX_TIMESTEPS 288 // 72 hours of 15 minute timesteps will be the maximum array size
//#define MAX_TIMESTEPS 72 // 72 hours of 1 hour timesteps will be the maximum array size

// ... defined in pfas.h
#define MAX_STNS 20
//#define MAX_STNS 200
#define MAX_VALUES 100
#define ID_SIZE 24
//#define NAME_SIZE 256
#define NAME_SIZE 120
#define MAX_DCS 450
#define FUEL_SIZE 6


/* FEPS structure
   much in common with FBP */


typedef struct                        /* FEPS */
{
/* inputs */
   char method[32],	   /* method of employment: AB= Alberta Plume Study; ICAO=Standard Atmosphere test; anything else=cmc calculations */
		profilename[NB_CHAR_PATH],/* name of the WRF->Hysplit profile filename (profile0_.txt) */
		csvfilename[NB_CHAR_PATH],/* name of Peter's hotspot file name */
		emissionsfilename[NB_CHAR_PATH],/* name of emissions factor matrix file name */
		emissionsmodelsfilename[NB_CHAR_PATH],/* name of emissions models file name */
		allocationfilename[NB_CHAR_PATH],/* name of emissions factor matrix file name */
		diurnalfilename[NB_CHAR_PATH],/* name of diurnal fire-growth weights */
        outfilename[NB_CHAR_PATH],/* name of the output file name */
        previous[NB_CHAR_PATH],   /* name of the previous time step's ini file (required to ascertain Qo) */
        diurnal[4],         /* Choice of diurnal approaches: CVW, BDL, EMC, OFF -- 2020-06-04 */
        shape[32],          /* shape of entrainment cloud: line/wedge, crescent, ellipse */
        type[32],		    /* type of upper air profile method used (average lapse rate, dry, wet) */
        ID[64],             /* fire ID */
        source[6],
        sensor[6],
        rep_date[24],      /* report date [YYYY-MM-DD HH:MM;SS] or [YYYY-MM-DD H:MM;SS] -- added 2020-07-10 */
        cmc_date[20],      /* report date [YYYY-MM-DD HH:MM;SS] or [YYYY-MM-DD H:MM;SS] -- added 2020-07-10 */
        glc_region[2],          /* GLC2000 region, used in GFFEPS -- 2021-04-16 */
        country[4],          /* country, used in GFFEPS -- 2021-04-16 */
        giglio[5];          /* region based on Giglio et al 2006, used in GFFEPS -- 2021-04-16 */

   int header;    /* print header info 1=Yes, 0=No */
   int ddate;     /* detection date of fire YYYYMMDD */
   int dtime;     /* detection time of fire HHMM (time all values are assumed to be collected) */
   int fdate;	  /* forecast (valid) date YYYYMMDD */
   int ftime;	  /* forecast (valid) time HHMM */
   int timezone;  /* time zone - hour offset from UTC [standard time] */
   int obs;       /* the time of the plume observation use in the Alberta study (HHMM) */
   int LDT;       /* local daylight time: 1=Yes, 0=No */
//   int diurnal;   /* Diurnal adjustment: 1=Yes, 0=No *** old type - now char[4]  -- 2020-06-04 */
   int sinks;     /* include the sinks terms: 1=Yes, 0=No */
   int radiation; /* radiation method: < 0 Byram's 1200/8600; =0 A_wall/A_top; > 0  value as % of Qfire */
   int print;     /* print: 1=Yes, 0=No */
   int newfire;   /* whether the hotpsot is a new fire (start fresh) or old (use existing energy and emissions data) 1=Yes, 0=No */
   int agfire;    /* whether the hotpsot is an agricultural fire 1=Yes, 0=No */

   int ihotspot;    /* Jack's new parameter */

   int glc, region;   /* GLC2000 land classification, used in GFFEPS -- 2021-04-16 */
   int mlc;           /* MODIS MCD12C12 land classification, used in GFFEPS -- 2022-02-22 */

   double cmcUTCo;	  /* hour of first fire report (0...72) */

   double temp,        /* surface temperature (oC) */
	      rh,              /* humidity (%) */
          ws,              /* wind speed (km/h) */
          wd,              /* wind direction (o) */
	      precip, 		   /* precipitation (mm) */
          DMC,
          DC,
          FWI,
          bfc,             /* BORFIRE fuel consumption (no longer used) */
          pressure,        /* MSL pressure (mb) */
          tmin,            /* minumum temprature (oC) -- 2022-02-22 */
          tavg,            /* minumum temprature (oC) -- 2022-02-22 */
          tdavg,            /* minumum temprature (oC) -- 2022-02-22 */

          growth,          /* change in fire size over last timestep (ha) */
          residencetime,   /* period used to calculate fire size of base of plume (hours) */
          residencegrowth, /* fire growth over residence time (hours) */
          area,            /* size of fire at dtime (ha) */
	      estarea,		   /* estimated fire size at time of detection (ha) */

          maxarea,         /* maximum growth size (used in Alberta study) (ha) */
          perimeter,       /* perimeter at dtime (m) */

	      reset,		   /* reset time of smoke plume calculations [decimal hours LST] (off if < 0) */
	      thstart,		   /* time to start (exclusive) top-hat fire growth [decimal hours LST] */
	      thend,		   /* time to end (inclusive) top-hat fire growth [decimal hours LST] */

          geometric,       /* daily geometric growth above persistence [%], default = 0 (off) */

          lapse,           /* lapse rate (oC/m) */

          ts,              /* temperature at surface (oC) */
          t850,            /* temperature at 850 mb */
          t700,            /* temperature at 700 mb */
          t500,            /* temperature at 500 mb */
          t250,            /* temperature at 250 mb */
          zs,              /* height at surface (m) */
          z850,            /* height at 850 mb */
          z700,            /* height at 700 mb */
          z500,            /* height at 500 mb */
          z250,            /* height at 250 mb */
          zt,              /* height of the tropopause (essentially the maximum height */

          alpha,           /* entrainment half-angle (o) */
          timestep,        /* timestep for modelling black body radiation loss (decimal hours) */
          elapsed,		   /* elapsed time (decimal hours) */

          Qo,              /* amount of energy previously injected into atmosphere (J) */
          Qfire,           /* energy of the fire: used when directly calculating the plume height from energy (in stand alone module) */
          Qplume,          /* energy in the plume: used when directly calculating the plume height from energy (in stand alone module) */
          Qf2Qp,           /* conversion factor from fire to plume: used when directly calculating the plume height from energy (in stand alone module) */

          Qflaming,		   /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */
          Qsmoldering,     /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */
          Qresidual,       /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */

	      Qs[MAX_TIMESTEPS+1],

          Mw_flaming,	   /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */
          Mw_smoldering,   /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */
          Mw_residual,     /* Qplume broken down into flaming, smoldering and residual components (used in emissions.c) */

          dz,              /* plume height (m) */
	      dzmin,		   /* minimum plume height (m)*/
          M,               /* mass of the plume volume *(kg) */
          Mw,              /* mass of water in the plume */

          Flaming,         /* consumption during the flaming stage [kg/m2] (used in emissions.c) */
          Smoldering,      /* consumption during the smoldering stage [kg/m2] (used in emissions.c) */
          Residual,        /* consumption during the residual stage [kg/m2] (used in emissions.c) */
	      totalemissions,  /* total emissions from the fire (tonnes) (used in emissions.c) */
	      r_smoke,		   /* mixing ratio of smoke to clear air (g/kg) */

          SCR,             /* Smouldering combustion rate [cm/hr] */

          GSI,             /* Growing Season Index -- Jolly et al. 2005 */
          GSI01,             /* Growing Season Index -- Jolly et al. 2005 */
          GSI07,             /* Growing Season Index -- Jolly et al. 2005 */
          GSI14,             /* Growing Season Index -- Jolly et al. 2005 */
          GSI21,             /* Growing Season Index -- Jolly et al. 2005 */

          DOB,             /* depth of burn (used in emissions.c) */
          DOB_F,           /* depth of burn - flaming (used in emissions.c) */
          DOB_S,           /* depth of burn - smouldering (used in emissions.c) */

          FRP,             /* Fire Radiative Power (used in ALberta Plume Rise Study) */
          Primary,         /* Primary plume height (used in ALberta Plume Rise Study) */
          Secondary,       /* Secondary plume height (used in ALberta Plume Rise Study) */

          north,           /* geographic extents of the model run -- 2022-11-14 */
          south,           /* geographic extents of the model run -- 2022-11-14 */
          east,            /* geographic extents of the model run -- 2022-11-14 */
          west,            /* geographic extents of the model run -- 2022-11-14 */

          SFB[15],         /* Small fire boost [per 14 regions as defined by Giglio (region 0 = globe)] */

          weight[24],      /* diurnal weighting scheme per hour (total = 1.0) */
          agweight[24];    /* diurnal crop burning weighting scheme per hour (total = 1.0) -  added 2023-05-04 */

}  FEPS;

typedef struct                        /* CSV field positions */
{
// cmc:
// lat,lon,rep_date,source,sensor,ffmc,dmc,dc,ws,fwi,fuel,ros,sfc,tfc,bfc,hfi,estarea,UTC,temp,rh,ZS,ws-met,Precip rate,TS,T850,T700,T500,T250,Z850,Z700,Z500,Z250

// cmcnew:
// lat,lon,rep_date,temp,rh,ws,wd,pcp,ffmc,dmc,dc,isi,fwi,estarea,pcuring,greenup,elev,fuel,slope,aspect,pconif,pdf,cbh,gfl,sfl,cfl,ffl,wfl,sh,sd,UTC,temp,rh,ZS,ws-met,Precip rate,TS,T850,T700,T500,T250,Z850,Z700,Z500,Z250

// global
// rep_date,latitude,longitude,glc_region,glc,country_code,region,estarea,UTC,P0,PR,PC,HU1015,HR1015,UUVV,TD0,T0,T850,T700,T500,T250,Z850,Z700,Z500,Z250,FFMC,DMC,DC,FWI,BUI,ISI
// rep_date,latitude,longitude,modis_lc,glc_region,glc,country_code,region,estarea,frp,UTC,FFMC,DMC,DC,FWI,BUI,ISI,TT,TD,PR,UV

    int columns,
        lat,
        lon,
        rep_date,
        spot,
        source,
        sensor,
        temp,
        td,
        rh,
        ws,
        wd,
        pcp,
        ffmc,
        dmc,
        dc,
        isi,
        fwi,
        gsi,      // KRA 2023-03-25
        gsi01,      // KRA 2023-09-15
        gsi07,      // KRA 2023-09-15
        gsi14,      // KRA 2023-09-15
        gsi21,      // KRA 2023-09-15
        estarea,
        pcuring,
        greenup,
        elev,
        fuel,
        ros,
        sfc,
        tfc,
        bfc,
        hfi,
        slope,
        aspect,
        pconif,
        pdf,
        cbh,
        gfl,
        sfl,
        cfl,
        ffl,
        wfl,
        lfl,
        dfl,
        hfl,
        sh,
        sd,
        UTC,
        temp_met,
        rh_met,
        zs_met,
        ws_met,
        wd_met,
        tmin_met,
        tavg_met,
        tdavg_met,
        Precip_rate,
        ts,
        t850,
        t700,
        t500,
        t250,
        zs,
        z850,
        z700,
        z500,
        z250,
        glc_region,
        glc,
        mlc,
        FRP,
        country,
        region,
        giglio,
        pressure;

} FIELDS;

typedef struct                        /* emissions data*/
{
   char	*species[MAX_SPECIES],
            emissionsfilename[NB_CHAR_PATH];/* name of emissions factor matrix file name */

   double a[MAX_EMODELS][MAX_SPECIES],	/* emissions factor [g/kg] of flaming combustion */
          b[MAX_EMODELS][MAX_SPECIES],	/* emissions factor [g/kg] of smoldering combustion */
          c[MAX_EMODELS][MAX_SPECIES],	/* emissions factor [g/kg] of residual combustion */

         f[MAX_SPECIES][MAX_TIMESTEPS+1],   /* cumulative emissions for flaming combustion */
         s[MAX_SPECIES][MAX_TIMESTEPS+1],   /* cumulative emissions for smoldering combustion */
         r[MAX_SPECIES][MAX_TIMESTEPS+1],   /* cumulative emissions for residual combustion */
/* we need to consider both end points UTC(t=0) and UTC(t=MAX_TIMESTEPS) */

         residence_flaming,    /* residence time for flaming combustion (hours) */
         residence_smoldering, /* residence time for smoldering combustion (hours)  - follows flaming */
         residence_residual;   /* residence time for residual combustion (hours)  - follows smoldering */

      int nModels, nSpecies,     /* number of models and species */
         emodel[MAX_EMODELS];  /* this allows for different emissions models based on CWFIS fuel types */

} EMISSIONS;

typedef struct                        /* emissions models (related to EMISSIONS.emodel[MAX_EMODELS]) */
{
   char	*FuelType[MAX_CWFIS];
   int     nModels,                        /* number of models  (-1 = generic model) */
           emodel[MAX_CWFIS];              /* this allows for different emissions models based on CWFIS fuel types */
   double rho2[MAX_CWFIS],
          rho4[MAX_CWFIS],
          rho6[MAX_CWFIS],
          rho8[MAX_CWFIS];

} EMODELS;

typedef struct                        /* allocations table data */
{
   double SFC_L[3],
          SFC_F[3],
          SFC_H[3],
          CFC_F[3],
          CFC_B[3],
          CFC_S[3],
          FFC[3],
          WFC[3],
          GFC[3];

} ALLOCATIONS;

typedef struct                        /* individual upper air sounding */
{
	double P[MAX_LEVELS],
		   Z[MAX_LEVELS],
		   T[MAX_LEVELS],
		   Td[MAX_LEVELS],
		   W[MAX_LEVELS],
		   D[MAX_LEVELS];
    int iLevels;
} UA;

typedef struct                        /* gridded surface weather with time used by fire-growth model */
{
    int     ndays,
            greenup,
            standing;
    double  cured;

    double  Ts[MAX_STNS][MAX_HOURS+1],
            rhs[MAX_STNS][MAX_HOURS+1],
            Ws[MAX_STNS][MAX_HOURS+1],
            wds[MAX_STNS][MAX_HOURS+1],
            rains[MAX_STNS][MAX_HOURS+1],
	        FFMCts[MAX_STNS][MAX_HOURS+1],

            DMCs[MAX_STNS][3],
            DCs[MAX_STNS][3],
            xs[MAX_STNS],
            ys[MAX_STNS],
            elevs[MAX_STNS],
            values[MAX_STNS];
} FGMWX;

typedef struct                        /* gridded upper air profiles with time */
{
    float   P[MAX_STNS][MAX_HOURS+1][MAX_LEVELS],
            Z[MAX_STNS][MAX_HOURS+1][MAX_LEVELS],
            T[MAX_STNS][MAX_HOURS+1][MAX_LEVELS],
            Td[MAX_STNS][MAX_HOURS+1][MAX_LEVELS];
} UAWX;

// global values taken from pfas.h
static char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

static char *dirs[] = {"N", "NE", "E", "SE", "S", "SW", "W", "NW"};

static int jdays[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

// ASCII Grid Info
int ncols, nrows, NODATA_value;
//int C1,C2,C3,C4,C5,C6,C7,D1,M1,M2,M3,M4,O1a,O1b,S1,S2,S3,WA,NF, maxFuels;
double xllcorner, yllcorner, cellsize, missing;
int wxdays;
int iwidths;


#endif
