typedef struct                        /* PFAS global data */
{
   double DMCex,
          alpha,
          beta,
          gamma,
          WxFactor,
          WnFactor,
          idw_pow,
		  friction;

   int rad,
       year,
	   timezone,
	   model;

   char diurnal[2];

}  PFAS;

typedef struct                        /* Climate data */
{
   double Tmax[12],
          Tmin[12],
          Tmean[12],
		  RH0600[12],
		  W[12],
		  cured[12];
   int	  greenup[12],
		  standing[12];
}  CLIM;

typedef struct                        /* Risk data */
{
   int year,
	   month,
	   day,
	   ndays,
	   userDMC,
	   simdays[12];

   double minprob,
	      friction; /* a universal friction value -- kinda obsolete now */

   char DMCsource[4];

}  RISK;

typedef struct                        /* Simulation data */
{
   int year,
	   month,
	   day,
	   ndays,
	   userDMC,
	   simdays[12];

   double lat,
	      lng,
		  minprob,
		  fsize,	/* final size: used for percentile calculations */
		  friction; /* a universal friction value -- kinda obsolete now */

   char DMCsource[4], igngrid[64];

}  SIM;

typedef struct
{
   int year,
	   month,
	   day,
	   time,
	   BE,
	   accel,
	   simdays[12];

   char diurnal[12],
	    file[64];
   double lat,
          lng,
          ndays,
	      FFMC,
	      DMC,
	      DC,
          BUI,
		  GFL,
		  CBH,
		  PC,
		  PDF,
		  DMCex;

}  FGM;

typedef struct
{
   char diurnal[2],
	    file[64];

   int year,
       month,
       day,
       time,
       ndays,
       simdays[12];

   double lat,
          lng,
	      T,
	      rh,
	      ws,
          wd,
		  Tn,
		  Tx,
		  Wn,
		  Wx;

}  WX;

typedef struct
{
   double temp,
	      rh,
	      ws,
	      wd,
	      precip,
	      FFMC,
		  DMC,
		  DC,
		  ISI,
		  BUI,
		  FWI;
}  FWX;

typedef struct                        /* FWAP record */
{
   unsigned short dry,
                  wet,
                  humid,
                  wind,
                  rain,
                  dir,
                  ffmc,
                  dmc,
                  dc,
                  isi,
                  bui,
                  fwi,
                  dsr;
}  FWAP;

typedef struct
{
   int DMCeqn[17];

   double depth[17],
          rhoB[17],
          Ri[17];

}  SMO;


int Date2Julian(int, int, int);
void Julian2Date(int, int *, int *, int *);

void Error(char *);

int ReadClimateData(CLIM *ptr);
int ReadGlobalData(PFAS *ptr);
int ReadSimData(SIM *ptr);
int ReadFgmData(FGM *ptr);
int ReadRiskData(RISK *ptr);
int ReadWxData(WX *ptr);
int ReadSmolderingData(SMO *ptr);
int PrintSimData(SIM *ptr);
int PrintFgmData(FGM *ptr);
int FBPType(char *, int, int, int);

/* System allows for 20 fuel types including non-fuels */
char *FBPs[20];
int iFuels[20];
double *Psp[8], *Psp0[8];

static char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

static char *dirs[] = {"N", "NE", "E", "SE", "S", "SW", "W", "NW"};

static int jdays[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

/* ASCII Grid Info */
int ncols, nrows, NODATA_value;
int C1,C2,C3,C4,C5,C6,C7,D1,M1,M2,M3,M4,O1a,O1b,S1,S2,S3,WA,NF, maxFuels;
double xllcorner, yllcorner, cellsize;
int wxdays;

/* filepath to data */
char filepath[64];
char errortext[64];
static double zero = 0.;
