      MODULE CFFEPS_module

!RThe following values must match those in feps_2011.h
      integer, parameter :: MAX_LEVELS = 100
      integer, parameter :: MAX_SPECIES = 20
      integer, parameter :: MAX_EMODELS = 10
      integer, parameter :: LABEL_SIZE = 32
      integer, parameter :: MAX_HOURS = 72
      integer, parameter :: MAX_CWFIS = 30
      integer, parameter :: NB_CHAR_PATH = 240
      integer, parameter :: MAX_TIMESTEPS = 24
      integer, parameter :: MAX_HOTSPOTS = 100
      integer, parameter :: FUEL_SIZE = 4
      integer, parameter :: MAX_FUELS = 20

      type FEPS_TYPE
         sequence
         character(len=LABEL_SIZE) :: method                    ! method of employment: AB= Alberta Plume Study ICAO=Standard Atmosphere test anything else=cmc calculations
         character(len=NB_CHAR_PATH) :: profilename                ! name of the WRF->Hysplit profile filename (profile0_.txt)
         character(len=NB_CHAR_PATH) :: csvfilename                ! name of Peter's hotspot file name
         character(len=NB_CHAR_PATH) :: emissionsfilename        ! name of emissions factor matrix file name
         character(len=NB_CHAR_PATH) :: emissionsmodelsfilename    ! name of emissions models file name
         character(len=NB_CHAR_PATH) :: outfilename                ! name of the output file name
         character(len=NB_CHAR_PATH) :: previous                    ! name of the previous time step's ini file (required to ascertain Qo)
         character(len=LABEL_SIZE) :: fshape                    ! shape of entrainment cloud: line/wedge, crescent, ellipse (was shape)
         character(len=LABEL_SIZE) :: ftype                        ! type of upper air profile method used (average lapse rate, dry, wet) (was type)
         character(len=LABEL_SIZE) :: fID                        ! fire ID

! from the GEM forecast data
!         character(len=24) :: rep_date    ! fire report date
!         character(len=20) :: cmc_date    ! the CMC forecast date and time

         integer :: header        ! print header info 1=Yes, 0=No
         integer :: ddate        ! detection date of fire YYYYMMDD
         integer :: dtime        ! detection time of fire HHMM (time all values are assumed to be collected)
         integer :: fdate        ! forecast (valid) date YYYYMMDD
         integer :: ftime        ! forecast (valid) time HHMM
         integer :: timezone    ! time zone - hour offset from UTC [standard time]
         integer :: obs            ! the time of the plume observation use in the Alberta study
         integer :: LDT            ! local daylight time: 1=Yes, 0=No
         integer :: diurnal        ! Diurnal adjustment: 1=Yes, 0=No
         integer :: sinks        ! include the sinks terms: 1=Yes, 0=No
         integer :: radiation    ! radiation method: < 0 Byram's 1200/8600 =0 A_wall/A_top > 0  value as % of Qfire
         integer :: fprint        ! print: 1=Yes, 0=No (was print)
         integer :: newfire        ! whether the hotpsot is a new fire (start fresh) or old (use existing energy and emissions data) 1=Yes, 0=No
         integer :: ihotspot      ! track the current hotspot with a numeric index
         real * 8 :: cmcUTCo        ! hour of first fire report (0...72)

         real * 8 :: temp        ! surface temperature
         real * 8 :: rh            ! humidity
         real * 8 :: ws            ! wind speed
         real * 8 :: precip     ! precipitation
         real * 8 :: DMC        ! Duff Moisture Code
         real * 8 :: DC            ! Drought Code
         real * 8 :: pressure    ! MSL pressure

         real * 8 :: growth            ! change in fire size over last timestep
         real * 8 :: residencetime        ! period used to calculate fire size of base of plume
         real * 8 :: residencegrowth    ! fire growth over residence time
         real * 8 :: area                ! size of fire at dtime
         real * 8 :: estarea            ! estimated fire size at time of detection

         real * 8 :: maxarea    ! maximum growth size (used in Alberta study)
         real * 8 :: perimeter    ! perimeter at dtime

         real * 8 :: reset        ! reset time of smoke plume calculations [decimal hours LST] (off if < 0)
         real * 8 :: thstart    ! time to start (exclusive) top-hat fire growth [decimal hours LST]
         real * 8 :: thend        ! time to end (inclusive) top-hat fire growth [decimal hours LST]

         real * 8 :: lapse        ! lapse rate oC/m

         real * 8 :: ts        ! temperature at surface
         real * 8 :: t850        ! temperature at 850 mb
         real * 8 :: t700        ! temperature at 700 mb
         real * 8 :: t500        ! temperature at 500 mb
         real * 8 :: t250        ! temperature at 250 mb
         real * 8 :: zs        ! height at surface
         real * 8 :: z850        ! height at 850 mb
         real * 8 :: z700        ! height at 700 mb
         real * 8 :: z500        ! height at 500 mb
         real * 8 :: z250        ! height at 250 mb
         real * 8 :: zt        ! height of the tropopause (essentially the maximum height

         real * 8 :: alpha        ! entrainment half-angle
         real * 8 :: timestep    ! timestep for modelling black body radiation loss
         real * 8 :: elapsed    ! elapsed time decimal hours

         real * 8 :: Qo        ! amount of energy previously injected into atmosphere
         real * 8 :: Qfire        ! energy of the fire: used when directly calculating the plume height from energy (in stand alone module)
         real * 8 :: Qplume    ! energy in the plume: used when directly calculating the plume height from energy (in stand alone module)
         real * 8 :: Qf2Qp        ! conversion factor from fire to plume: used when directly calculating the plume height from energy (in stand alone module)

         real * 8 :: Qflaming        ! Qplume broken down into flaming, smoldering and residual components (used in emissions.c)
         real * 8 :: Qsmoldering    ! Qplume broken down into flaming, smoldering and residual components (used in emissions.c)
         real * 8 :: Qresidual        ! Qplume broken down into flaming, smoldering and residual components (used in emissions.c)

         real * 8 :: Qs(MAX_TIMESTEPS, MAX_HOTSPOTS)

         real * 8 :: dz    ! plume height in metres
         real * 8 :: dzmin    ! minimum plume height
         real * 8 :: M        ! mass of the plume volume

         real * 8 :: Flaming            ! consumption during the flaming stage [kg/m2] (used in emissions.c)
         real * 8 :: Smoldering        ! consumption during the smoldering stage [kg/m2] (used in emissions.c)
         real * 8 :: Residual            ! consumption during the residual stage [kg/m2] (used in emissions.c)
         real * 8 :: totalemissions    ! total emissions from the fire (tonnes) (used in emissions.c)
         real * 8 :: r_smoke            ! mixing ratio of smoke to clear air

         real * 8 :: DOB    ! depth of burn (used in emissions.c)
         real * 8 :: DOB_F    ! depth of burn - flaming (used in emissions.c)
         real * 8 :: DOB_S    ! depth of burn - smouldering (used in emissions.c)

      end type FEPS_TYPE

      type FBP_TYPE
         sequence
! inputs
         character(len=FUEL_SIZE) :: FuelType
         integer :: Accel           ! 1 = point, 0 = line (no longer accepts 2 as a line source)
         integer :: Dj              ! Julian Day
         integer :: D0              ! Julian day of minimum FMC (was Do)
         integer :: ELV             ! Elevation [m ASL]
         integer :: BUIEff          ! BUI effect: 1 = yes, 0 = no
         integer :: FC_method       ! Fuel COnsumption method: 1 = default FBP SFL values

         real * 8 :: t               ! Hours since ignition
         real * 8 :: FFMC            ! FFMC
         real * 8 :: DMC             ! DMC  - added 2020-02-21
         real * 8 :: DC              ! DC   - added 2020-02-21
         real * 8 :: Depth           ! Fuel Depth (cm) - added 2020-02-21
         real * 8 :: BD              ! Bulk Density   - added 2020-02-21
         real * 8 :: ISI             ! ISI
         real * 8 :: BUI             ! BUI
         real * 8 :: WS              ! wind speed [kmh]
         real * 8 :: WD              ! wind direction [degrees]
         real * 8 :: GS              ! Slope [percent]
         real * 8 :: Aspect          ! Aspect [degrees]
         real * 8 :: PC              ! Percent Confier for M1/M2
         real * 8 :: PDF             ! Percent Dead Fir for M3/M4
         real * 8 :: C               ! Percent Cured for O1a/O1b (85% default)
         real * 8 :: GFL             ! Grass Fuel Load [kg/m^2] (0.3 default)
         real * 8 :: CBH             ! Crown to Base Height [m] (FBP defaults)
         real * 8 :: CFL             ! Crown Fuel Load [kg/m^2] (FBP defaults)
         real * 8 :: SFL             ! Surface Fuel Load [kg/m^2] (FBP defaults)  - added 2020-02-21
         real * 8 :: FFL             ! Fine Fuel Load [kg/m^2] (FBP defaults)  - added 2020-02-21
         real * 8 :: WFL             ! Woody Fuel Load [kg/m^2] (FBP defaults)  - added 2020-02-21
         real * 8 :: LFL             ! Litter Fuel Load [kg/m^2] -- 2019-11-20
         real * 8 :: DFL             ! Duff (Fermentation) Fuel Load [kg/m^2] -- 2019-11-20 */
         real * 8 :: HFL             ! Humus Fuel Load [kg/m^2] -- 2019-11-20
         real * 8 :: LAT             ! Latitude [decimal degrees]
         real * 8 :: LON             ! Longitude [decimal degrees]
         real * 8 :: FMC             ! FMC if known
         real * 8 :: SH              ! C6 Stand Height [m] - 2009
         real * 8 :: SD              ! C6 Stand Density [stems/ha] - 2009
         real * 8 :: theta           ! elliptical direction of calculation

! outputs
         real * 8 :: ROS             ! Rate of Spread [m/min]
         real * 8 :: FROS            ! Flank rate of Spread [m/min]
         real * 8 :: BROS            ! Back Rate of Spread [m/min]
         real * 8 :: TROS            ! Rate of Spread at angle theta [m/min]
         real * 8 :: HROSt           ! Head Rate of Spread at time t [m/min]
         real * 8 :: FROSt           ! Flank Rate of Spread at time t [m/min]
         real * 8 :: BROSt           ! Back Rate of Spread at time t [m/min]
         real * 8 :: TROSt           ! Rate of Spread at angle theta at time t [m/min]
         real * 8 :: CFB             ! Crown Fraction Burned
         real * 8 :: FCFB            ! Flank Crown Fraction Burned [%]
         real * 8 :: BCFB            ! Back Crown Fraction Burned [%]
         real * 8 :: TCFB            ! Crown Fraction Burned at angle thetea [%]
         real * 8 :: HFI             ! Head Fire Intensity [kW/m]
         real * 8 :: FFI             ! Head Fire Intensity [kW/m]
         real * 8 :: BFI             ! Head Fire Intensity [kW/m]
         real * 8 :: TFI             ! Head Fire Intensity [kW/m]
         real * 8 :: TFC             ! Total Fuel Consumption [kg/m^2]
         real * 8 :: FTFC            ! Flank Total Fuel Consumption [kg/m^2]
         real * 8 :: BTFC            ! Back Total Fuel Consumption [kg/m^2]
         real * 8 :: TTFC            ! Total Fuel Consumption at angle theta [kg/m^2]
         real * 8 :: SFC             ! Surface Fuel Consumption [kg/m^2]
         real * 8 :: TI              ! Time of Crown Fire initiation [hrs since ignition]
         real * 8 :: FTI             ! Time of Flank Crown Fire initiation [hrs since ignition]
         real * 8 :: BTI             ! Time of Back Crown Fire initiation [hrs since ignition]
         real * 8 :: TTI             ! Time of Crown Fire initiation at angle theta [hrs since ignition]
         real * 8 :: LB              ! Length to Breadth ratio
         real * 8 :: RAZ             ! Spread direction azimuth
         real * 8 :: WSV             ! Net vectored wind speed       end type FBP

      end type FBP_TYPE

      type EMISSIONS_TYPE
         sequence
         character(len=LABEL_SIZE) :: species(MAX_SPECIES)
         character(len=NB_CHAR_PATH) :: emissionsfilename! name of emissions factor matrix file name

! NOTE: FORTRAN and C have reverse orders of indexing
         real * 8 :: a(MAX_EMODELS, MAX_SPECIES)    ! emissions factor [g/kg] of flaming combustion
         real * 8 :: b(MAX_EMODELS, MAX_SPECIES)    ! emissions factor [g/kg] of smoldering combustion
         real * 8 :: c(MAX_EMODELS, MAX_SPECIES)    ! emissions factor [g/kg] of residual combustion

         real * 8 :: f(MAX_TIMESTEPS, MAX_HOTSPOTS, MAX_SPECIES)   ! cumulative emissions for flaming combustion
         real * 8 :: s(MAX_TIMESTEPS, MAX_HOTSPOTS, MAX_SPECIES)   ! cumulative emissions for smoldering combustion
         real * 8 :: r(MAX_TIMESTEPS, MAX_HOTSPOTS, MAX_SPECIES)   ! cumulative emissions for residual combustion
! we need to consider both end points UTC(t=0) and UTC(t=MAX_TIMESTEPS)

         real * 8 :: residence_flaming    ! residence time for flaming combustion (hours)
         real * 8 :: residence_smoldering ! residence time for smoldering combustion (hours)  - follows flaming
         real * 8 :: residence_residual   ! residence time for residual combustion (hours)  - follows smoldering

         integer :: nModels      ! number of models
         integer :: nSpecies     ! number of species
         integer :: emodel(MAX_EMODELS)  ! this allows for different emissions models based on CWFIS fuel types
      end type EMISSIONS_TYPE

      type EMODELS_TYPE
         sequence
         character(len=FUEL_SIZE) :: FuelType(MAX_CWFIS)
         integer :: nModels                        ! number of models  (-1 = generic model)
         integer :: emodel(MAX_CWFIS)              ! this allows for different emissions models based on CWFIS fuel types
      end type EMODELS_TYPE

      type UA_TYPE
         real * 8 P(MAX_LEVELS)
         real * 8 Z(MAX_LEVELS)
         real * 8 T(MAX_LEVELS)
         real * 8 Td(MAX_LEVELS)
         real * 8 W(MAX_LEVELS)
         real * 8 D(MAX_LEVELS)
         integer iLevels
      end type UA_TYPE

        END MODULE CFFEPS_module
