#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdbool.h>
#include <malloc.h>
#include "fbp_2021.h"
#include "fwi84.h"
#include "feps_2011.h"
#include "thermo.h"

FILE *infile, *outfile, *csvfile, *uafile, *profile, *Final_file, *Final_CO_file, *Final_Area_file, *glc_emissions_file, *glc_areas_file, *glc_count_file, *glc_region_emissions_file, *glc_region_count_file;

double PlumeCalc();
double PlumeCalcDry();
double PlumeCalcWet();

double EnergyCalc();
void EmissionsCalc();
int EmissionsOverTime();
double EmissionsOverDistance();
double eq_ffmc();
double hourly_ffmc();
double bdl_ffmc();
double BUIcalc();
double FMCcalc();
void FBP_C_test();
int mtime();
double overlap();
double Readprofile();
int Readupperair();
int ReadEmissionsData();
int ReadEmissionsModels();
int ReadHotspotHeader();
int ReadHotspotData();
int ReadAllocationTable();
int ReadDiurnal();
int ReadFEPSData();
int WriteFEPSData();
double T_d_Calc();
double e_w_Calc();

// grid.c for use with FGM
int *read_int_ASCIIGRID();
int *read_trim_int_ASCIIGRID();
double *read_double_ASCIIGRID();
double *read_trim_double_ASCIIGRID();
int write_int_ASCIIGRID();
int write_as_int_ASCIIGRID();
int write_double_ASCIIGRID();
int write_trim_int_ASCIIGRID();

int FBPType();
void get_xy();
//int diurnal();
int interp_wx();
void fire_propagate();
double burn_cell();

int main(int argc, char *argv[])
{
    char infilename[NB_CHAR_PATH], outfilename[NB_CHAR_PATH], profilename[NB_CHAR_PATH], profilepath[NB_CHAR_PATH], buffer[640], errortext[100];
    char buffer1[640], filename[32];
    char *regions[15] = {"GLOBAL", "BONA", "TENA", "CEAM", "NHSA", "SHSA", "EURO", "MIDE", "NHAF", "SHAF", "BOAS", "CEAS", "SEAS", "EQAS", "AUST"};

    double Le, Ld, Ts, Tm, Tt, Ths, Thm, q, m, surface, Ediff, Tplume, Tenv, lapse, dz, dH;
    int i, j, icol, jrow, nshort, nlong, status, firsttime, iyear, imonth, iday, hour, batchjob, iLevels, itimestep, ntimesteps, istn,
        pyear, pmonth, pday, phour, pmin, pDj, Do;
    double x, y, FFMCy, FFMCnoon, profile_date, timezone, detectionUTC, sum,
       elapsed, detectionLST, discovery, ignitionLST, discoverysize, discoverydist, timestep, current, observationLST,
       t1, t2, dt, dt_o, A, AA, AAA, P, D, dist, area, perim, nextdist, nextfdist, nextbdist, growth, a,b, diameter, hourlygrowth, dailygrowth,
       Alag, Plag, Dlag, tlag, precip,
       DC, DMC, FFMC, FFMCt, hum, mc, Q, Qo, Qfire;
    double Qs8, Qs7, Qs5, Q87, Q85, Les8, Les7, Les5, Le87, Le85, dzs8, dzs7, dzs5, dz87, dz85, dzb8, dzbs;
    double Qa, Qd, Qw, Qr;
    double dz8, dz7, dz5, l8, l7, l5, r_smoke, estarea, estareao, estarea2, dzdry, dzwet, dzwetter, Mdry, Mplume, Mw, C;
    double g = 9.80025, cp = 1005, sbc = 5.67e-8, H = 18000000.;
    double pi = 3.1415926;
    double zero = 0.;
    double sumROS, ROSo, ROSs[24];    // some ideas 2020-06-08

    char rep_date[24], rep_time[20], cmc_date[20], source[8], sensor[6], fuel[12];
    char *token;
    double lat, lon, lat0, lon0, fwi, ros, bfc;
//    double lats[500], lons[500], estareas[500];  // used for overlap calculations (500 hotspots maximum), can be removed
    double plat, plon, parea, ptime, COemissions, totalCOemissions, totalemissions, totalarea, totalgrowth, briggs, Final[23], FinalCO[23], FinalArea[23];    // used with the CMC data
    double *GridTotalEmissions, *GridTotalMws, *GridTotalFs, *GridTotalSs, *GridTotalRs;
    double *FCs, *Fs, *Ss, *Rs, *Qs, *Mws;
    int ii, iii, k, iUTC, iMax, iFuel, nFuels, iModel, jModel, iSpecies, nSpecies, SpeciesCO, nAllocations, cmcdate, cmcDATEo, cmcDj, cmcDjo, cmcUTC, cmcUTCo, cmcLST, newfire;
    double dcmcUTC, dcmcUTCo, dcmcLST, pcmcUTC;
    double ROSd, SFCd, CFCd, TFCd, HFId, CFBd;  // FBP conditions at detection time.

    double Ft, St, Rt, discoveryLST, discoveryLST1, detectionsize; // used in Alberta Plume Rise study

    double hours, concentration, width, distance;  // used for TROPOMI calculations

    double *areas, *areas0, *ts, *ts0;

    double isi, pcuring, greenup, cfl_tph, branch_tph, deadwood_tph, foliage_tph; // Dan's new data

    double F, M, Z, BIAS, HR, FAR, CSI;
    int file1, file2, region, pregion, glc, pglc;

    double offset;  // a test value 2023-03-08

    double T, WS, Vis, Td, Precip, RH, ISI, BUI, FWI, GSI;

int glc_count[24];			// values 1-23 (0 not used)
double glc_emissions[24];	// values 1-23 (0 not used)
double glc_areas[24];

int glc_region_count[24][23];			// values 1-23 (0 not used)
double glc_region_emissions[24][23];	// values 1-23 (0 not used)



double Mdd, dy, Cy[100];

int across, down, ignpt, fgmdirection, fgmmethod, ext_status;  // ASCII grids use in FGM
int *FBPg, *FBPg0,
    *slope, *slope0,
	*aspect, *aspect0;  // ASCII grids use in FGM
double *ign, *ign0, r2, r2min, //*hfi, *hfi0, *cfb, *cfb0, *sfc, *sfc0, *tfc, *tfc0, r2;
       *Pg, *Pg0,
       *Og, *Og0,
       *Zg, *Zg0,
       *cbh, *cbh0,
       *cfl, *cfl0,
       *dfl, *dfl0,
       *pc, *pc0,
       *sd, *sd0,
       *sfl, *sfl0,
       *sh, *sh0;

float hy[] = { 0.997197, 0.988796, 0.977063, 0.963677, 0.948656, 0.932560, 0.915401, 0.896631, 0.876275, 0.854893,
               0.831394, 0.805832, 0.779303, 0.750732, 0.720190, 0.688218, 0.655409, 0.622924, 0.591074, 0.560228,
               0.530349, 0.501327, 0.473018, 0.445392, 0.418798, 0.393278, 0.368783, 0.345211, 0.322204, 0.299929,
               0.278683, 0.258633, 0.239737, 0.221835, 0.204740, 0.188180, 0.172537, 0.158254, 0.145685, 0.134907};

    FBP fbp, pfbp;
    FEPS feps, pfeps;
    FIELDS fields;
    EMISSIONS emissions;
    ALLOCATIONS allocations;
    EMODELS emodels;
    UA ua;
    FGMWX fgmwx;
    UAWX uawx;
//  feps.print=-2;   // this turns on the printf statement for following the code

/* 1. Set up the run */
if (feps.print==-2) printf("1.\n");

/* 1.a. Read the argument line for the name of the input file.
    If this is missing, use the default infile.dat. */
if (feps.print==-2) printf("1.a.\n");
    strcpy(infilename, "infile.dat\0");

    if (argc > 1)
    {
        sprintf(infilename, "%s\0", argv[1]);
        strcpy(infilename, argv[1]);
    }

/* 1.b. setup FBP default values */
//if (feps.print==-2) printf("1.b.\n");
    FBPdefaults(&fbp);

//    FBP_C_test(&fbp);

/* 1.c. read in FEPS input data */
if (feps.print==-2) printf("1.c.\n");
    if (ReadFEPSData(infilename, &feps, &fbp)) printf("Error reading %s\n", infilename);
    feps.perimeter = diameter = 2*sqrt(10000.*feps.area/2/pi)/1000.;    // diameter of a circle [km] of estarea [ha]
    estarea = feps.area;

//  feps.print=-2;   // this turns on the printf statement for following the code

    feps.radiation = 0;

/* 1.d. Define time series arrays for emissions of each pollutant */
if (feps.print==-2) printf("1.d. '%s'\n", feps.method);
    nSpecies = ReadEmissionsData(feps.emissionsfilename, &emissions);
    nFuels = ReadEmissionsModels(feps.emissionsmodelsfilename, &emodels);
    nAllocations = ReadAllocationTable(feps.allocationfilename, &allocations);  /* KRA: 2019-09-25 */
    status = ReadDiurnal(feps.diurnalfilename, &feps);                          /* KRA: 2019-09-25 */
//    if (!strcasecmp(feps.method, "cmc") ) feps.timestep = 1.;
    lapse = feps.lapse; /* used to refresh the original lapse rate per time step */

    areas = (double *)calloc((int)(24./feps.timestep + 1), sizeof(double));  // used to track the areas
    areas0 = areas;

    ts = (double *)calloc((int)(24./feps.timestep + 1), sizeof(double));  // used to track the areas
    ts0 = ts;

    for (i=0; i<(int)(24./feps.timestep + 1); i++);
    {
        *areas = -999; areas++;
        *ts = -999; ts++;
    }
    areas = areas0; ts = ts0;

    for(i=0; i< 24; i++)
    {
        glc_count[i]=0;
        glc_emissions[i]=0.;
        glc_areas[i]=0.;
    }

    for(i=0; i< 24; i++)
		for (j=0;j<23;j++)
		{
			glc_region_count[i][j]=0;
			glc_region_emissions[i][j]=0.;
		}

    for (i=0;i<23;i++)
    {
        Final[i] = 0.; // used to calculate final total emissions
        FinalCO[i] = 0.;  // added 2022-04-22 to track total CO emissions
        FinalArea[i] = 0.;  // added 2022-04-22 to track total area
    }

    for (i=0;i<100;i++) Cy[i]=0;

    pregion=-999;
	pglc=-999;

/* number of short/long-term smoldering time steps */
    nshort = (int)(2./feps.timestep);
    nlong = (int)(MAX_HOURS/feps.timestep);
    ntimesteps = nlong+1;   // we need to consider both end points UTC(t=0) and UTC(t=MAX_HOURS)

    GridTotalEmissions = (double *)calloc((int)(ntimesteps), sizeof(double));
    GridTotalMws = (double *)calloc((int)(ntimesteps), sizeof(double));
    GridTotalFs = (double *)calloc((int)(ntimesteps), sizeof(double));
    GridTotalSs = (double *)calloc((int)(ntimesteps), sizeof(double));
    GridTotalRs = (double *)calloc((int)(ntimesteps), sizeof(double));
    Qs = (double *)calloc((int)(ntimesteps), sizeof(double));
    Mws = (double *)calloc((int)(ntimesteps), sizeof(double));
    FCs = (double *)calloc((int)(ntimesteps), sizeof(double));
    Fs = (double *)calloc((int)(ntimesteps), sizeof(double));
    Ss = (double *)calloc((int)(ntimesteps), sizeof(double));
    Rs = (double *)calloc((int)(ntimesteps), sizeof(double));

    newfire = false; // false
    profile_date = -999.;
    totalCOemissions = COemissions = 0.;   // added 2022-04-22 to track total CO emissions
    totalemissions = 0.;
    totalarea = 0.;
    totalgrowth = 0.;
    dzdry = dzwet = dzwetter = 0.; // seems this is turned off for now

// for "top-hat", reset weight[] values to top-hat approach
    if (!strncmp(feps.shape, "top-hat", 3))
        for (i=0;i<24;i++)
            if (i >= floor(feps.thstart) && i < ceil(feps.thend))
            {
                if ( (feps.thend - feps.thstart) < 1.)
                    dt = feps.thend - feps.thstart;
                else if ( i < feps.thstart && feps.thstart - (double)i < 1.)
                    dt = (double)i + 1. - feps.thstart;
                else if ( feps.thend - (double)i < 1.)
                    dt = feps.thend - (double)i;
                else
                    dt = 1.;
                feps.weight[i] = dt/(feps.thend - feps.thstart);
            }
            else
                feps.weight[i] = 0.;

    if (!strncmp(feps.diurnal,"OFF",3))  /* no diurnal effect -- 2020-06-04 */
        for (i=0;i<24;i++)
            feps.weight[i] = 1./24.;

    for (itimestep=0; itimestep<ntimesteps; itimestep++)
    {
//        printf("%d\n", itimestep);
        GridTotalEmissions[itimestep]=0.;
        GridTotalMws[itimestep]=0.;
        GridTotalFs[itimestep]=0.;
        GridTotalSs[itimestep]=0.;
        GridTotalRs[itimestep]=0.;
    }

/*******************************************************************************************/
    if (!strncasecmp(feps.method, "cmc", 3)   /* CMC Firework method */
     || !strncasecmp(feps.method, "global", 3)     /* Global GFFEPS */
     || !strncasecmp(feps.method, "tropomi", 3) )    /* TROPOMI */
    {


/* 1.e. Open CMC GEM forecast CSV file as input.  Read in header line */
if (feps.print==-2) printf("1.e.\n");
        if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL) {
            printf("Error: could not find %s\n\n", feps.csvfilename);

        } else if ( (outfile = fopen(feps.outfilename, "w")) == NULL) {
            printf("Error: could not write to %s\n\n", feps.outfilename);
        }
        else
        {

// p is shorthand for previous record
            plat=0.;
            plon=0.;
            parea=0.;
            Qo=0.;

            if (!strncasecmp(feps.method, "global", 3))  // experimenting with options
            {
                fbp.FC_method = 2;
                strcpy(feps.type, "dry");
            }

// read (and ignore) header info
            fgets(buffer, 640, csvfile);
// printf("%s: %s\n", feps.csvfilename, buffer);
// print header info
            if (!strncasecmp(feps.method, "cmc", 3) || !strncasecmp(feps.method, "global", 3) || !strncasecmp(feps.method, "tropomi", 3) ) // seems we're already in (!strcasecmp(feps.method, "cmc")
            {
//                feps.timestep=1.;

/* 1.f. Print (to the screen), new header line for the CFFEPS predictions CSV file. */
if (feps.print==-2) printf("1.f.\n");
                fprintf(outfile, "lat, lon, rep_date, source, sensor, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, cfc, tfc, bfc, hfi, estarea, ");

                if (!strncasecmp(feps.method, "global", 3) || !strncasecmp(feps.method, "tropomi", 3))
                    fprintf(outfile, "glc region, glc region number, glc, country, region, frp, GSI, FMC, ");

                fprintf(outfile, "UTC, temp, rh, ws, precip, TS, T850, T700, T500, T250, ZS, Z850, Z700, Z500, Z250, Area(t), Growth (t), TotalEmissions, r_smoke, Zplume, Mplume, Qo, Qplume");

                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                {
                    fprintf(outfile, ", %s F", emissions.species[iSpecies]);
                    fprintf(outfile, ", %s S", emissions.species[iSpecies]);
                    fprintf(outfile, ", %s R", emissions.species[iSpecies]);

                    if (!strcasecmp(emissions.species[iSpecies], "CO")) SpeciesCO =  iSpecies;  // added 2022-04-22 to track total CO emissions
                }

                if (!strncasecmp(feps.method, "tropomi", 3))
                    fprintf(outfile, ", 0-4, 4-8, 8-12, 12-16, 16-20, 20-24, 24-28, 28-32, 32-36, 36-40");

                fprintf(outfile, "\n");

            }
            status = ReadHotspotHeader(&buffer, &fields);


// read in first line of data
            fgets(buffer, 640, csvfile);
        }  // if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL)... else


// printf("%s\n", buffer);

/* 2. Begin reading CFFEPS predictions CSV file */
if (feps.print==-2) printf("2.\n");
//------------------------------------------------------------------------------------------------------//
        do  /* while !feof(csvfile); */
        {



/* 2.a. Reset arrays to zero. */
if (feps.print==-2) printf("2.a.\n");

            feps.header=-1;

/* 2.b. Read line of data into buffer.  Parse comma separated variables in buffer.
        Make adjustments for possible variation in units (e.g Kelvin instead of Celsius, etc.) */
if (feps.print==-2) printf("2.b. '%s'\n", feps.method);
// read in first line of data
            status = ReadHotspotData(&buffer, &fields, &feps, &fbp, &ua);

//   if (strncasecmp(feps.country, "CUB", 3)) strcpy(fbp.FuelType, "NF"); // exclude all but Cuba
//   if (feps.region != 9) strcpy(fbp.FuelType, "NF"); // limit to "SHAF"
//   if (feps.region != 12) strcpy(fbp.FuelType, "NF"); // limit to "SEAS"
//   if (feps.region != 13) strcpy(fbp.FuelType, "NF"); // limit to "EQAS"
//   if (feps.region != 14) strcpy(fbp.FuelType, "NF"); // limit to "AUST"
/*   else
   {
        offset = 4.;
//        printf ("%s %lf %lf %d... ", feps.rep_date, offset, fbp.LON, feps.dtime);
        fbp.LON = fbp.LON - 30. * offset;
        feps.dtime = feps.dtime - 300 * offset;
//        printf (" %lf %d\n", fbp.LON, feps.dtime);
   }*/

//   if (fbp.LAT < -20 || fbp.LAT > -10 ) strcpy(fbp.FuelType, "NF");  // limits Oliveira et al (2015)

//   if (feps.region == 12) glc_count[feps.glc]++; strcpy(fbp.FuelType, "NF"); // count the occurrences in SEAS
//   if (feps.region == 13) glc_count[feps.glc]++; strcpy(fbp.FuelType, "NF"); // count the occurrences in EQAS
//   if (feps.region == 14) glc_count[feps.glc]++; // strcpy(fbp.FuelType, "NF"); // count the occurrences in AUST

// temporarily limit hotspots to Lac_La_Loche -- needed to comment these lines out KRA - 2021-03-12
// if (fbp.LAT < 56.3 || fbp.LAT > 56.56 || fbp.LON < -109.95 || fbp.LON > -109.65) strcpy(fbp.FuelType, "NF"); // ignore these records

// Limit hotspots to six peat fire sites in Kalimantan, Indonesia, in 2015 (Graham et al. 2022)
// if (fbp.LAT < -2.75 || fbp.LAT > -1.75 || fbp.LON < 114.25 || fbp.LON > 115.0)  strcpy(fbp.FuelType, "NF"); // ignore these records
// if (fbp.LAT < -2.5226 || fbp.LAT > -2.2064 || fbp.LON < 114.39 || fbp.LON > 114.63175) strcpy(fbp.FuelType, "NF"); // ignore these records
// central Kalimantan
// if (fbp.LAT < -4. || fbp.LAT > 0. || fbp.LON < 112. || fbp.LON > 116.) strcpy(fbp.FuelType, "NF"); // ignore these records

			if ( fbp.LAT > feps.north || fbp.LAT < feps.south || fbp.LON > feps.east || fbp.LON < feps.west )
				strcpy(fbp.FuelType, "NF"); // ignore these records -- added 2022-11-14 (can be deleted)
//			else
//				glc_count[feps.glc]++;					// 2023-06-27 -- now moved to new fire

// set these back to internal variables
            estarea = feps.estarea;
            timezone = feps.timezone;
            feps.LDT = 0.;

// Small fire boost (only applied to global data) - 2023-02-13
//            if (!strncasecmp(feps.method, "global", 3))
//                estarea = feps.SFB[feps.region]*feps.estarea;

/* elapsed is the time since detectionLST (dhrs) */
            elapsed = 0.;firsttime = 1;
            timestep = feps.timestep;

// NOTE - cmcUTC, etc. are integer hours.  This is needed for the diurnal growth variable feps.weight[cmcUTC], which is hourly.
//        dcmcUTC, etc. are doubles
            cmcUTC = feps.ftime/100;
            cmcdate = feps.fdate;
            cmcUTC = feps.ftime/100;

            dcmcUTC = cmcUTC + (feps.ftime % 100)/60.;
            cmcLST = cmcUTC + (int)timezone;
            dcmcLST = dcmcUTC + (int)timezone;
            if (cmcLST < 0) cmcLST = cmcLST+24;
            if (cmcLST >= 24) cmcLST = cmcLST-24;   // 2020-06-10

// Section moved below -- KRA 2021-03-11

/* Determine CMC julian date */
            if (cmcdate> 0)
            {
                if (cmcdate > 9999)
                    iyear = (int)(cmcdate/10000);
                else
                    iyear = 0;
                imonth = (int)((cmcdate-10000*iyear)/100);
                iday = cmcdate-10000*iyear-100*imonth;
                cmcDj = jdays[imonth-1]+iday;
                if (iyear > 0 && iyear%4 == 0 && imonth >2) cmcDj+=1;
            }  // iyear, imonth and iday will be overwritten

            detectionUTC = (int)(feps.dtime / 100) + (feps.dtime % 100) /60.;
            detectionLST = (int)(feps.dtime / 100) + (feps.dtime % 100) /60. - feps.LDT + (int)timezone;

            if (detectionLST < 0.)
            {
                detectionLST = detectionLST + 24.;
                fbp.Dj = fbp.Dj;    // keep fbp.Dj as the UTC date
            }

// printf("%s %lf %d %d %lf %lf\n", feps.rep_date, feps.ftime, feps.fdate, feps.dtime, detectionUTC, detectionLST);

// the following section was moved from above (to allow for cmcDj and fbp.Dj calculation) -- KRA 2021-03-11

            if ( (int)(cmcdate/10000) > (int)(feps.ddate/10000) ) cmcDj = cmcDj+365; // KRA - 2022-03-21  -- moved from below


/* adjust cmcUTC to a time wrt detection time and date, so this is 0 at detection time, negative before and positive after (can exceed 24) */
            cmcUTC = cmcUTC + 24*(cmcDj - fbp.Dj);    // should use a new variable name like "elapsed"
            if (newfire == true) // this seems to be ahead of "newfire" assessment? -- 2023-06-29
            {
                cmcUTCo = cmcUTC; // true, cmcUTCo = the first fire record time (not the time of detection)
                dcmcUTCo = cmcUTCo;
            }

if (feps.print<0 && cmcUTC<cmcUTCo) printf("2.h. Warning: cmcUTC (%d) < cmcUTCo (%d)", cmcUTC, cmcUTCo);

// end of section moved

//            if ( (int)(cmcdate/10000) > (int)(feps.ddate/10000) ) cmcDj = cmcDj+365; // KRA - 2021-01-05


/* 2.d. Determine if the line read in is a new hotspot
        (if the latitude, longitude or date does not equal previous line’s values).
        If new, reset arrays to zero. */
if (feps.print==-2) printf("2.d. %d\n", ua.iLevels);

//            if (plat==fbp.LAT && plon==fbp.LON && cmcdate <= atoi(&cmc_date[0])) // same fire
            if (plat==fbp.LAT && plon==fbp.LON) // same fire
            {
                newfire = false; // false
                itimestep++;


				timestep = dcmcUTC - pcmcUTC;  					// 2023-06-21
				if (timestep < 0.) timestep = timestep + 24.;	// 2023-06-21
				pcmcUTC = dcmcUTC;								// 2023-06-21
				pglc = glc = feps.glc;


                // feps.area=parea;        // overwrite area with area from previous time step

// TESTING A RESET AT 6:00 AM LST //
/*                if (cmcLST==6)
                {
                    feps.M = feps.Mw = 0.;
                    totalemissions = 0.;  // zero everything including total emissions
                    feps.Qo = feps.Qplume= Qo = 0.;    // Qo is used to temporarily store the energy when comparing height predictions using different lapse rates
                }
*/
// Done //

// printf("%d %lf %lf %lf %lf %lf\n", cmcLST, feps.Qo, feps.Qplume, totalemissions, feps.M, feps.Mw);
            }
            else    // new fire
			{
				glc = feps.glc;
				if (strcmp(fbp.FuelType, "NF")) glc_count[glc]++;
				if (strcmp(fbp.FuelType, "NF")) glc_region_count[glc][feps.region]++;
				if (strcmp(fbp.FuelType, "NF")) glc_region_count[glc][0]++;


                newfire = true; // true
                feps.area=0.; //=estarea;
                region = 0;
				pcmcUTC = dcmcUTC;	// 2023-06-21

// This section was used to create 4 North American regions to compare GFFEPS with CFFEPS
/*              if (fbp.LAT>=49.                && fbp.LON>=-141 && fbp.LON<-100) region=1;
                if (fbp.LAT>=49.                && fbp.LON>=-100 && fbp.LON<-50)  region=2;
                if (fbp.LAT>=25. && fbp.LAT<49. && fbp.LON>=-141 && fbp.LON<-100) region=3;
                if (fbp.LAT>=25. && fbp.LAT<49. && fbp.LON>=-100 && fbp.LON<-50)  region=4;
                pregion = feps.region;  // used to calculate global emissions following GLC 2000 regions -- 2022-02-24 */

//                if (pregion<0) glc = feps.glc;  // first record (and areas outside of North America)

                if (pregion<0) pregion = region = feps.region;  // first record (and areas outside of North America)
                if (pglc<0) pglc = glc = feps.glc;  // first record (and areas outside of North America)

// This section records emissions from the previous fire before starting a new fire
// (the final fire is recorded after exiting the reading input loop [2.n], prior to writing the output files)	-- 2023-06-23
                Final[pregion] = Final[pregion] + totalemissions;
                Final[0] = Final[0] + totalemissions;		// global values
                FinalCO[pregion] = FinalCO[pregion] + totalCOemissions;
                FinalCO[0] = FinalCO[0] + totalCOemissions;	// global values
//				if(totalarea>0) FinalArea[pregion] = FinalArea[pregion] + totalarea;
//				if(totalarea>0) FinalArea[0] = FinalArea[0] + totalarea;
				if(totalgrowth>0) FinalArea[pregion] = FinalArea[pregion] + totalgrowth;	// 2023-10-25
				if(totalgrowth>0) FinalArea[0] = FinalArea[0] + totalgrowth;

				glc_emissions[pglc] = glc_emissions[pglc] + totalemissions;   // track emissions per glc type
//				if(totalarea>0) glc_areas[pglc] = glc_areas[pglc] + totalarea;   // track emissions per glc type
				if(totalgrowth>0) glc_areas[pglc] = glc_areas[pglc] + totalgrowth;   // 2023-10-25
				glc_region_emissions[pglc][pregion] = glc_region_emissions[pglc][pregion] + totalemissions;   // track emissions per glc type per region
				glc_region_emissions[pglc][0] = glc_region_emissions[pglc][0] + totalemissions;   // track emissions per glc type for the globe

// printf ("509: %lf %lf %d %lf %lf %lf %lf\n", plat, plon, pregion, Final[pregion], FinalCO[pregion], totalemissions, totalCOemissions);

                pregion = region = feps.region;
				pglc = glc = feps.glc;

				if (!strncasecmp(feps.method, "cmc", 3)) pregion = 1;	// turn this off for standard CFFEPS runs -- 2023-06-23

                totalemissions = 0.;  // zero everything including total emissions
                totalCOemissions = COemissions = 0.;  // added 2022-04-22 to track total CO emissions
				totalarea = 0.;
				totalgrowth = 0.;
                feps.Qo = Qo = 0.;    // Qo is used to temporarily store the energy when comparing height predictions using different lapse rates
                feps.Qplume = 0.;
                feps.Mw = 0.;   // Mass of water released from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/
                for (itimestep=0; itimestep<ntimesteps; itimestep++)
                {
                    Qs[itimestep]=0.;
                    Mws[itimestep]=0.;
                    Fs[itimestep]=0.;       // new fire, set concentrations back to zero
                    Ss[itimestep]=0.;       // new fire, set concentrations back to zero
                    Rs[itimestep]=0.;       // new fire, set concentrations back to zero
                    for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                    {
                        emissions.f[iSpecies][itimestep]=0.;       // new fire, set concetrations back to zero
                        emissions.s[iSpecies][itimestep]=0.;       // new fire, set concetrations back to zero
                        emissions.r[iSpecies][itimestep]=0.;       // new fire, set concetrations back to zero
                    }
                }
                plat=fbp.LAT;
                plon=fbp.LON;
                parea=0.;
                estareao = estarea; // for use with geometric growth 2019-10-15

                feps.growth=0.;
                feps.Flaming=0.;
                feps.Smoldering=0.;
                feps.Residual=0.;
                feps.dz = dz = -999.;
                dzdry = dzwet = dzwetter = 0.;
                itimestep=0;

                }

/* Set FMC == each hotspot */
            if (newfire == true)
            {
               Do = -1; /* converted so Do value is returned (but not used in CFFEPS) -- 2019-07-29 */
               fbp.FMC = FMCcalc(fbp.LAT, fbp.LON, feps.zs, fbp.Dj, &Do);
            };
/* 2.h. If new fire, determine the hours from the first hourly CMC forecast to the detection time:
           if less than 24 hours, an initial fire size is calculated (based on persistence routine);
           if greater than zero, then the fire starts at zero size. */
if (feps.print==-2) printf("2.h. '%s'\n", feps.shape);

            if (newfire == true) // determine the initial Area(t) for first CMC hour.
            {


                Mdd = -1.;  // Maximum downwind distance - used in EmissionsOverDistance()
                for (i=0;i<100;i++) Cy[i]=0;  // downwind concentrations

                sumROS = 0.;
                for (i=0; i<24;i++) ROSs[i]=0.;

                dt_o = 24.*(fbp.Dj-cmcDj) + detectionUTC - dcmcUTC; // hours from first CMC hour to detection time

                if (dt_o < 24.)
                {

                    if (!strncmp(feps.shape, "tophat", 3) ||
                        !strncmp(feps.shape, "weighted", 3) ||
                        !strncmp(feps.shape, "ratio", 3) ||
                        !strncmp(feps.shape, "hybrid", 3)  ||
                        !strncmp(feps.shape, "wma", 3) )
                    {

/* new approach, count backwards from detectionLST */
                        sum = feps.weight[(int)detectionLST] * (detectionLST - (int)detectionLST);

                        if (cmcLST>detectionLST) // cmcLST on previous day
                        {
                            i=(int)detectionLST+1; if (i> 23) i=i-23;
                            sum = (detectionLST-(int)detectionLST)*feps.weight[i];
                            for (i=(int)detectionLST+24; i>cmcLST;i--)
                            {
                                if (i>23)
                                    sum+=feps.weight[i-24]; // accumulate the hourly weight over the course of a day
                                else
                                    sum+=feps.weight[i]; // accumulate the hourly weight over the course of a day
                            }
                        }
                        else // (cmcLST<detectionLST)
                        {
                            i=(int)detectionLST+1; if (i> 23) i=i-23;
                            sum = (detectionLST-(int)detectionLST)*feps.weight[i];
                            for (i=(int)detectionLST; i>cmcLST;i--)
                            {
                                if (i>23)
                                    sum+=feps.weight[i-24]; // accumulate the hourly weight over the course of a day
                                else
                                    sum+=feps.weight[i]; // accumulate the hourly weight over the course of a day
                            }
                        }

                        if (dt_o<0)
                            estarea2 = estarea - estarea*sum + (1-(int)dt_o/24)*estarea;  // account for burning prior to forecast start
                        else
                            estarea2 = estarea - estarea*sum;  // new value for estarea, capturing the entire burn period

                    }

                }
                else        // detection 24 hours or more after first CMC hour
                    estarea2 = 0.;
                feps.area = parea = estarea2;
            }

/* 2.i. Calculate FFMC from yesterday’s value using the technique described in Lawson et al. (1996).
        This is based on the observed noon FFMC.
        If the time is before noon, yesterday’s FFMC is required.
        This is converged on so that yesterday’s FFMC will predict today’s observed FFMC
        (set to 11:59AM, 23:59 hours later). */
if (feps.print==-2) printf("2.i. '%s' %lf\n", feps.diurnal, fbp.FFMC);

            FFMCy = FFMC = fbp.FFMC;     /* we are assuming the value passed in FBP is the noon FFMC */

 /* Calculate yesterday's FFMC */
            if (!strncmp(feps.diurnal,"BDL",3))  /* BDL diurnal effect -- 2020-06-04 */
            {
                if (detectionLST < 12.)
                {
                    FFMCy = FFMC; FFMCt=0.;    /* first guess */
                    FFMCnoon = bdl_ffmc(FFMC, 1200, (int)feps.rh);
                    i=0;
                    while (fabs(FFMCnoon-FFMCt) > .1 && FFMCy > 50. && FFMCy < 100. && i < 100)
                    {
                        FFMCt = bdl_ffmc(FFMCy, 1159, (int)feps.rh);
                        FFMCy = FFMCy + (FFMCnoon-FFMCt);
                        i++;
                    }
                    if (i==100) FFMCy = (FFMCt+FFMCy)/2;
                    if (FFMCy>99.) FFMCy=99.;
                }

                if (FFMCy < 60. ) // && strcasecmp(feps.method, "AB") && (strncasecmp(feps.method, "cmc", 3)  || !strncasecmp(feps.method, "Global", 3) )    // we're already in !strcasecmp(feps.method, "cmc")
                {
//                    fprintf(outfile, "Low FFMC: %lf set to 75.\n", FFMCy);
                    FFMCy = 75.;
                }

                hour = mtime(detectionLST);

                if (hour < 1200)
                    fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
                else
                    fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);
            }
            else if (!strncmp(feps.diurnal,"CVW",3))  /* CVW diurnal effect -- 2020-06-05 */
            {
                fbp.FFMC = hourly_ffmc(feps.temp, feps.rh, feps.ws, feps.precip, fbp.FFMC);
            }
            else if (!strncmp(feps.diurnal,"EMC",3)  || /* EMC diurnal effect -- 2020-06-04 */
                     !strncmp(feps.diurnal,"CVW",3) && newfire == true)  // set the FFMC to equilibrium
            {
                fbp.FFMC = eq_ffmc(feps.temp, feps.rh, feps.ws, feps.precip, zero);
            }   // else fbp.FFMC will be constant
            fbp.t = detectionLST;

/* 2.j  Calculate FBP values at time of detection (and then at current time) */
if (feps.print==-2) printf("2.j.\n");

            status = FBPCalc (&fbp);    // FBP at detectionLST

            ROSd = fbp.ROS; SFCd = fbp.SFC; TFCd = fbp.TFC; CFCd = TFCd-SFCd; HFId = fbp.HFI; CFBd = fbp.CFB;

/* 2.k. Determine discovery time (the number of hours of growth required to reach the detection size).
        When persistence is being used, this time is assumed to be 24 hours. */
if (feps.print==-2) printf("2.k. '%s'\n", fbp.FuelType);
            t1=0.; t2=0.; A=estarea; P=0.; D=0.;
            PROCalc(fbp.FuelType, fbp.Accel,
                        fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,   // ROS at UTC time
                        &t1, &t2, &A, &P, &D);
            discovery = t2;    // use the detectionLST to determine discovery; afterwards use cmcLST for the hour

            if (!strncmp(feps.shape, "tophat", 3) ||
                !strncmp(feps.shape, "weighted", 3) ||
                !strncmp(feps.shape, "ratio", 3) ||
                !strncmp(feps.shape, "hybrid", 3) ||
                !strncmp(feps.shape, "wma", 3) )
                    discovery = 24.; // set the time of discovery equal to 24 hours


            hour = mtime(dcmcLST);
            feps.elapsed = dt = 24.*cmcDj+dcmcUTC - (24.*fbp.Dj + detectionUTC-discovery); // difference in elapsed time from discovery to current time (NOTE: dt has changed from previous "newfire" definition)

            dcmcUTC - detectionUTC;  // used in Brigg's model to determine the current time
            if (feps.elapsed < 0.) feps.elapsed = 0.;

            if (!strncmp(feps.diurnal,"BDL",3))  /* BDL diurnal effect -- 2020-06-04 */
                if (hour < 1200)
                    fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
                else
                    fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);
            else if (!strncmp(feps.diurnal,"CVW",3))  /* CVW diurnal effect -- 2020-06-05 */
            {
                fbp.FFMC = hourly_ffmc(feps.temp, feps.rh, feps.ws, feps.precip, fbp.FFMC);
            }
            else if (!strncmp(feps.diurnal,"EMC",3)  || /* EMC diurnal effect -- 2020-06-04 */
                     !strncmp(feps.diurnal,"CVW",3) && newfire == true)  // set the FFMC to equilibrium
            {
                fbp.FFMC = eq_ffmc(feps.temp, feps.rh, feps.ws, feps.precip, zero);
            }   // else fbp.FFMC will be constant

/* 2.j  Calculate FBP values at current time */
            status = FBPCalc (&fbp);    // FBP at UTC time
            ROSs[cmcLST] = fbp.ROS;   // used to calculate 24-hour moving average

            if (status==0  && dt >=0.)
            {
                if (dt < 24.) sumROS = sumROS + fbp.ROS;

/* 2.l. If current time is prior to discovery, the fire has not yet occurred and does not grow; else: */
if (feps.print==-2) printf("2.l.\n");
//                if ( ( (detectionUTC-discovery) >= (double)cmcUTC ) || discovery == 999.9 ||  FFMCy < 60.)  // no fire yet
                if ( ( (detectionUTC-discovery) >= dcmcUTC ) || discovery == 999.9)  // no fire yet
                {
                    A = 0.;
                    feps.area = 0.;
                    r_smoke = -999.;
                    r_smoke = 0.;
                    dz = -999;
                    dH = -999.;
                    discovery = 0;
                }
                else
                {


/* 2.l.i. If the current time corresponds to the reset time (if being used, not the default), set the  area, cumulative energy and total emissions to zero */
if (feps.print==-2) printf("2.l.i\n");
                    if ( (feps.reset >= 0.) && (dcmcLST >= feps.reset && dcmcLST < feps.reset+timestep) )
                    {
                        parea = 0.;
                        Qo=0.;
                        totalemissions = 0.;
                        totalCOemissions = COemissions = 0.;
						totalarea = 0.;
 						totalgrowth = 0.;
                   }
                    else
                    {
                        parea = feps.area;
                        Qo=feps.Qo;
                    }
/* 2.l.ii. Determine difference in time between current time and detection time.  If less than an hour, calculate partial hour growth; otherwise, calculate hourly growth according to top-hat or weighted approach.  Add to total area. */
if (feps.print==-2) printf("2.l.ii.\n");

                    if (feps.agfire==true)    // this is based on a weighting scheme of McCarty et al. 2009 and used by the EPA
                    {
                        if (cmcLST >= 24) cmcLST-=24;

// new agricultural residue burning weighting scheme
                        if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
                            feps.growth = dt * timestep * feps.agweight[cmcLST] * estarea;      // weighted fire growth of previous hour
                        else
                            feps.growth = timestep * feps.agweight[cmcLST] * estarea;      // weighted fire growth of previous hour
                    }

                    else
					{

						if (!strncmp(feps.shape, "elliptical", 3))    // this portion of the code seems to be in flux
						{
							if (parea ==  0.)
							{
								t1 = 0;
								t2 = 24.*( (fbp.Dj+dcmcUTC/24.) - (fbp.Dj+(detectionUTC-discovery)/24.) );    // these can be multiday lengths, when grown at noon ROS produce wrong values
								t2 = dcmcUTC - (detectionUTC-discovery) ;    // these can be multiday lengths, when grown at noon ROS produce wrong values
								t2 = dt;    // this is correct

								if (t2 > 0.)
								{
									A=0.; P=0.; D=0;
									PROCalc(fbp.FuelType, fbp.Accel,
											fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
											&t1, &t2, &A, &P, &D);
									feps.growth = A;   // area at cmcUTC
								}
								else
									feps.growth = 0.;

								if (cmcUTCo >=0) parea = estarea;

								feps.growth = estarea;  //temporary fix

								parea = estarea2; // new approach 2018/01/18
								feps.growth = 0;  //temporary fix

							}   // if (parea ==  0.)
							else
							{
								t1=0.; t2=0.; A=parea; P=0.; D=0.;
								PROCalc(fbp.FuelType, fbp.Accel,
										fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
										&t1, &t2, &A, &P, &D);

								t1=t2; t2=t2+timestep; A=0.;P=0.;D=0.;
								PROCalc(fbp.FuelType, fbp.Accel,
										fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
										&t1, &t2, &A, &P, &D);

								feps.growth = A;  // area growth in one time step: A(t2+timestep)-A(t2)
							}
						}  // if (!strncmp(feps.shape, "elliptical", 3))


// This statement allows for geometric daily growth
						if (feps.geometric > 0. && dt > 0.)
						{
								estarea = estarea + floor(dt/24.)*feps.geometric/100.*estareao; // stepwise increase in size based on percent of original estarea
								estarea = estareao * pow( 1. + feps.geometric/100., floor(dt/24.)); // geometric growth
//                            printf("... %lf %lf %lf %lf %lf\n", estarea, feps.geometric, dt, floor(dt/24.), pow( 1. + feps.geometric/100., floor(dt/24.)) );
						}

						if (!strncmp(feps.shape, "tophat", 3))
						{
							if (hour > mtime(feps.thstart) && hour <= mtime(feps.thend))
								if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
									feps.growth = dt * timestep * estarea/(feps.thend - feps.thstart);      // top-hat
								else
									feps.growth = timestep * estarea/(feps.thend - feps.thstart);      // top-hat
							else
								feps.growth = 0.;
						}

						if (!strncmp(feps.shape, "weighted", 3))    // this is the default based on readfeps()
						{
							if (cmcLST >= 24) cmcLST-=24;

// original scheme using predefined weighting scheme
							if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
								feps.growth = dt * timestep * feps.weight[cmcLST] * estarea;      // weighted fire growth of previous hour
							else
								feps.growth = timestep * feps.weight[cmcLST] * estarea;      // weighted fire growth of previous hour
						}

						if (!strncmp(feps.shape, "ratio", 3))
						{
							if (cmcLST >= 24) cmcLST-=24;

// new weighting shceme based on the ratio of the current ROS (as calculated by GEM) over the CWFIS ros (baseline)
// note: "fbp.ROS/ros/24." so that the total over 24 hours ~ 1
							if (ros > 0)
								if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
									feps.growth = dt * timestep * fbp.ROS/ros/24. * estarea;      // weighted fire growth of previous hour
								else
									feps.growth = timestep * fbp.ROS/ros/24. * estarea;      // weighted fire growth of previous hour
							else
									feps.growth = 0.;
						}

// hybrid using the predefined weighting scheme prior to detection and the ratio afterwards
// note: "fbp.ROS/sumROS" so that the total over 24 hours ~ 1 ("/24" is embedded with the use of sumROS in the denominator)
						if (!strncmp(feps.shape, "hybrid", 3))
						{
							if (cmcLST >= 24) cmcLST-=24;

							if (dt <= 24.)
// original scheme
								if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
									feps.growth = dt * timestep * feps.weight[cmcLST] * estarea;      // weighted fire growth of previous hour
								else
									feps.growth = timestep * feps.weight[cmcLST] * estarea;      // weighted fire growth of previous hour
							else

// testing new weighting shceme based on the ratio of the sum of the current ROS over the ROS prior to detection
								if (sumROS > 0.)
									if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
										feps.growth = dt * timestep * fbp.ROS/sumROS * estarea;      // weighted fire growth of previous hour
									else
										feps.growth = timestep * fbp.ROS/sumROS * estarea;      // weighted fire growth of previous hour
								else
									feps.growth = 0.;
						}

// wma uses a weighted moving average of the ROS over the past 24 hours
// note: "fbp.ROS/sumROS" so that the total over 24 hours ~ 1 ("/24" is embedded with the use of sumROS in the denominator)
						if (!strncmp(feps.shape, "wma", 3))
						{
							if (cmcLST >= 24) cmcLST-=24;

							sumROS = 0.;
							for (i=0; i<24; i++)
							{
								j=cmcLST+i;
								if (j> 23) j =j-24;
								sumROS = sumROS + ROSs[j];
								if (ROSs[j]==0.)
									sumROS = sumROS + fbp.ROS * feps.weight[j]/feps.weight[cmcLST];
							}

// testing new weighting shceme based on the ratio of the sum of the current ROS over the ROS prior to detection
							if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
								feps.growth = dt * timestep * fbp.ROS/sumROS * estarea;      // weighted fire growth of previous hour
							else
								feps.growth = timestep * fbp.ROS/sumROS * estarea;      // weighted fire growth of previous hour

						}


						if (!strncmp(feps.shape, "line", 3) )
						{
							if (parea == 0. && dt > 0.) parea = estarea;

							diameter = 2*sqrt(10000.*estarea/pi);    // diameter of a circle of estarea

							if (dt > 0.  &&  dt < timestep) // was dt < 1.-- changed 2021-04-12
								feps.growth = timestep * dt * 60.*fbp.ROS * diameter/10000.;
							else
								feps.growth = timestep * 60.*fbp.ROS * diameter/10000.;
						}
					}

                    if (newfire == true) feps.growth = 0.;

                    feps.area = feps.growth + parea;
                    parea= feps.area;

                    dz8 = dz7 = dz5 = dz = -999.;
                    l8 = l7 = l5 = -999.;
                    r_smoke = -999.;
                    r_smoke = 0.;


/* 2.l.iii. Calculate energy of the fire */
if (feps.print==-2) printf("2.l.iii.\n");
                    feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);

/* 2.l.iv. Calculate the energy released from flaming, smoldering and residual combustion.
            Calculate flaming, smoldering and residual emissions rates in tonnes/h  */
if (feps.print==-2) printf("2.l.iv.\n");
                    EmissionsCalc(&feps, &fbp, &emodels, &allocations);

                        emissions.residence_flaming=0.25;   /* residence time for flaming combustion (hours) -- remains the same */
                        emissions.residence_smoldering = feps.DOB_S/2.; /* this is new, based on the assumption that a fire smolders at 1 cm/hour (Dan Thompson) */
                        emissions.residence_residual = feps.DOB_S/2.;   /* split equally between smoldering and residual */

// printf("\n... %lf %lf %lf... %lf %lf %lf\n", feps.DOB, feps.DOB_F, feps.DOB_S, emissions.residence_flaming, emissions.residence_smoldering, emissions.residence_residual);

/* 2.l.v. Distribute the water released [kg/timestep] over time, required for the subroutine PlumeCalcWet -- 2019-07-17*/
if (feps.print==-2) printf("2.l.v.\n");
                    jModel = -2;    /* an admittedly awkward way of creating hourly water components */
                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

//                    itimestep = (int)((dcmcUTC-dcmcUTCo)/timestep);

                    for (i=0; i< iMax; i++)
                        if (i+itimestep <= ntimesteps)
                        {
                            Mws[i+itimestep] = Mws[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                        }
                        else
                            if (feps.print==-1) printf("2.l.v. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", itimestep, MAX_HOURS);


/* 2.l.v. Distribute the energy over time */
if (feps.print==-2) printf("2.l.v.\n");
                    jModel = -1;    /* an admittedly awkward way of creating hourly energy components */
                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                    for (i=0; i< iMax; i++)
                        if (itimestep < ntimesteps)
                        {
                            Qs[i+itimestep] = Qs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                        }
                        else
                            if (feps.print==-1) printf("2.l.v. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", itimestep, MAX_HOURS);
/* 2.l.vi. From the area growth and energy released, calculate the plume rise */
if (feps.print==-2) printf("2.l.vi.\n");
                    feps.Qplume = Qs[itimestep];  // need to check this
                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                    feps.Mw = feps.Mw + Mws[itimestep];       // Cumulative mass of water released [kg] from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/
//                    feps.Mw = feps.Mw + Mws[cmcUTC-cmcUTCo] * (1. - fbp.CFB/2.);    // incomplete combustion?

/* 2.l.vi. (1) if type=dry, wet (Mw=0) or wetter (Mw>0), use all upper air data following a piecewise method of calculating plume rise */
                    if (!strcmp(feps.type, "dry"))
                    {
                        feps.Qo = Qo;   // remember the energy prior to entering decision tree
                        dz = PlumeCalcDry(&feps, &ua); // new piece-wise approach
                    }

                    else if (!strcmp(feps.type, "wet"))
                    {
                        Mw = feps.Mw;
                        feps.Mw = 0.;   // setting feps->Mw to 0 turns off PlumeCalcWetter (returning to PlumeCalcWet)        /* KRA: 2019-09-25 */
                        feps.Qo = Qo;   // remember the energy prior to entering decision tree
                        dz = PlumeCalcWet(&feps, &ua); // new piece-wise approach
                        feps.Mw = Mw;   // rewrite the value to feps.Mw
                    }
                    else if (!strcmp(feps.type, "wetter") ||        /* KRA: 2019-09-25 */
                             !strcmp(feps.type, "lcl"))             /* KRA: 2020-11-09 */
                    {
                        feps.Qo = Qo;   // remember the energy prior to entering decision tree
                        dz = PlumeCalcWet(&feps, &ua); // new piece-wise approach
                    }
/* 2.l.vi. (2) follow the standard average lapse rate method outlined in Anderson et al., 2011 */
                    else    // follow the standard average lapse rate method outlined in Anderson et al., 2011
                    {

/*** old approach -- a cumbersome sequence of layered if/else conditions.  Rewritten more succinctly below (this section left in for now) ***/

/* 2.l.vi. (2) (a) calculate height based on surface to 850 mb lapse */
                        feps.Qo = Qo;   // remember the energy prior to entering decision tree
                        l8 = feps.lapse = (feps.t850 - feps.ts) / (feps.z850 - feps.zs);
                        if (feps.lapse > -0.0098 && feps.z850 > feps.zs)
                            dz8 = dz = PlumeCalc(&feps);

/* 2.l.vi. (2) (b) if recalculated height greater than 2000 m,
                  recalculate height based on surface to 700 mb lapse rate
                  (if resulting height less than 2000, use average) */
                        if (dz > 2000.  || dz <= 0.)
                        {
                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l7 = feps.lapse = (feps.t700 - feps.ts) / (feps.z700 - feps.zs);
                            if (feps.lapse > -0.0098 && feps.z700 > feps.zs)
                            {
                                dz7 = dz = PlumeCalc(&feps);
                                if (dz < 2000)
                                {
                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    feps.lapse = (l8+l7)/2;
                                    if (feps.lapse > -0.0098)
                                        dz = PlumeCalc(&feps);
                                }
                            }

/* 2.l.vi. (2) (c) if recalculated height greater than 4000 m,
                  recalculate height based on surface to 500 mb lapse rate
                  (if resulting height less than 4000, use average) */
                            if (dz > 4000.  || dz <= 0.)
                            {
                                feps.Qo = Qo;  // remember the energy prior to entering decision tree
                                l5 = feps.lapse = (feps.t500 - feps.ts) / (feps.z500 - feps.zs);
                                if (feps.lapse > -0.0098)
                                {
                                    dz5 = dz = PlumeCalc(&feps);
                                    if (dz < 4000)
                                    {
                                        feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                        feps.lapse = (l7+l5)/2;
                                        if (feps.lapse > -0.0098)
                                            dz = PlumeCalc(&feps);
                                    }
                                }

/* 2.l.vi. (2) (d) if superadiabatic from surface up to 500m, recalculate using 850 mb as the base
                  recalculate height based on 850 mb to 700 mb lapse rate
                  (if resulting height less than 2000, use average) */
                                if (dz <= 0.)
                                {
                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l7 = feps.lapse = (feps.t700 - feps.t850) / (feps.z700 - feps.z850);
                                    if (feps.lapse > -0.0098  && feps.z700 > feps.zs)
                                    {
                                        dz7 = dz = PlumeCalc(&feps);
                                        if (dz < 2000)
                                        {
                                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                            feps.lapse = (l8+l7)/2;
                                            if (feps.lapse > -0.0098)
                                                dz = PlumeCalc(&feps);
                                        }
                                    }

/* 2.l.vi. (2) (e) if recalculated height greater than 4000 m,
                  recalculate height based on 850 mb to 500 mb lapse rate
                  (if resulting height less than 4000, use average) */
                                    if (dz > 4000.  || dz <= 0.)
                                    {
                                        feps.Qo = Qo;  // remember the energy prior to entering decision tree
                                        l5 = feps.lapse = (feps.t500 - feps.t850) / (feps.z500 - feps.z850);
                                        if (feps.lapse > -0.0098)
                                        {
                                            dz5 = dz = PlumeCalc(&feps);
                                            if (dz < 4000)
                                            {
                                                feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                                feps.lapse = (l7+l5)/2;
                                                if (feps.lapse > -0.0098)
                                                    dz = PlumeCalc(&feps);
                                            }
                                        }
                                    }    // if (dz > 4000.  || dz < 0.)
                                }    // if (dz < 0.)  [superadiabatic from surface]
                            }    // if (dz > 4000.  || dz < 0.)
                        }    // if (dz > 2000.  || dz < 0.)

/* New approach, rewritten, same idea -- 2019-07-17 */
                        if (feps.ts > -99. && feps.zs > -99. )  /* otherwise missing data */
                        {
                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l8 = feps.lapse = (feps.t850 - feps.ts) / (feps.z850 - feps.zs);
                            if (feps.lapse > -0.0098 && feps.z850 > feps.zs && feps.z850 > 0.)
                                dz8 = PlumeCalc(&feps);

                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l7 = feps.lapse = (feps.t700 - feps.ts) / (feps.z700 - feps.zs);
                            if (feps.lapse > -0.0098 && feps.z700 > feps.zs && feps.z700 > 0.)
                                dz7 = PlumeCalc(&feps);

                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l5 = feps.lapse = (feps.t500 - feps.ts) / (feps.z500 - feps.zs);
                            if (feps.lapse > -0.0098 && feps.z500 > feps.zs && feps.z850 > 0.)
                                dz5 = PlumeCalc(&feps);

                            if (dz8 > 0. && l8 > -0.0098 ) dz = dz8;
                            if (dz7 > 0. && l7 > -0.0098 && dz > 2000.) dz = dz7;
                            if (dz5 > 0. && l5 > -0.0098 && dz > 4000.) dz = dz5;
                        }

                        if (dz < 0. && feps.z850 > 0.)  /* height still missing -- start at 850 mb */
                        {
                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l7 = feps.lapse = (feps.t700 - feps.t850) / (feps.z700 - feps.z850);
                            if (feps.lapse > -0.0098 && feps.z700 > feps.zs && feps.z700 > 0.)
                                dz7 = PlumeCalc(&feps);

                            feps.Qo = Qo;   // remember the energy prior to entering decision tree
                            l5 = feps.lapse = (feps.t500 - feps.t850) / (feps.z500 - feps.z850);
                            if (feps.lapse > -0.0098 && feps.z500 > feps.zs && feps.z500 > 0.)
                                dz5 = PlumeCalc(&feps);

                            if (dz7 > 0. && l7 > -0.0098) dz = dz7;
                            if (dz5 > 0. && l5 > -0.0098 && dz > 4000.) dz = dz5;
                        }
/* End of new approach */

                        if (dz8 > 2000. && dz7 < 2000.  && dz7 > 0.)
                        {
                            dz = (dz8+dz7)/2;
                            feps.lapse = (l8+l7)/2;
                        }

                        else if (dz7 > 4000. && dz5 < 4000. && dz5 > 0.)
                        {
                            dz = (dz7+dz5)/2;
                            feps.lapse = (l7+l5)/2;
                        }

                    }    // if (!strcmp(feps.type, "dry"))... else

/* 2.l.vi. (3) limit height to 250 mb height */
                    if (dz > feps.z250)    dz = feps.z250;

/* 2.l.vi. (4) impose a minimum height */
                    if (dz < feps.dzmin) dz = feps.dzmin;

                    feps.Qo = Qo;  // remember the energy prior to entering decision tree (note that as is, this discounts the energy lost due to black body radiation)

                    feps.Qo = Qo = Qo + feps.Qplume;    // I used to do this in PlumeCalc but the new EmissionsCalc has thrown this off

                    feps.dz = dz;

                } // if ( (fbp.Dj+(detectionUTC-discovery)/24.) > (cmcDj+(double)cmcUTC/24.)  || (discovery == 999.9) ||  FFMCy < 60.)  ...  else

/* 2.l.vii. Find emodel that matches the fuel type */
if (feps.print==-2) printf("2.l.vii.\n");
                if (emissions.nModels==0 || emodels.nModels==0)
                    jModel=0;
                else
                    for (iFuel=0; iFuel<nFuels; iFuel++)
                    if (!strcasecmp(emodels.FuelType[iFuel], fbp.FuelType)) // found a match
                        for (iModel=0;iModel<emissions.nModels;iModel++)
                            if (emissions.emodel[iModel]==emodels.emodel[iFuel] )
                                jModel=iModel;

/* 2.l.viii.  Distribute the emission over time */
if (feps.print==-2) printf("2.l.viii.\n");
                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                {
                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);
                    for (i=0; i< iMax; i++)
                        if (i+itimestep <= ntimesteps)
                        {
                            emissions.f[iSpecies][i+itimestep]=emissions.f[iSpecies][i+itimestep]+Fs[i];
                            emissions.s[iSpecies][i+itimestep]=emissions.s[iSpecies][i+itimestep]+Ss[i];
                            emissions.r[iSpecies][i+itimestep]=emissions.r[iSpecies][i+itimestep]+Rs[i];

                            if (iSpecies==SpeciesCO)
                            {
                                COemissions =  Fs[i]+Ss[i]+Rs[i];
                                totalCOemissions = totalCOemissions + Fs[i]+Ss[i]+Rs[i];
                            }
                        }
                        else
                            if (feps.print==-1) printf("2.l.viii. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", i+itimestep, MAX_HOURS);

                }


//                dy=4.;
//                Mdd = EmissionsOverDistance(Cy, Mdd, dy, jModel, SpeciesCO, &emissions, &feps, &fbp );


/* 2.l.ix. Calculate total emissions and r_smoke */
if (feps.print==-2) printf("2.l.ix.\n");
                totalemissions = totalemissions+10.*fbp.TFC*feps.growth;
//                totalarea = totalarea+feps.growth;
                totalarea = feps.area;
				totalgrowth = totalgrowth+feps.growth;	// 2023-10-25
                GridTotalEmissions[itimestep] = GridTotalEmissions[itimestep] + totalemissions;

                if (feps.dz > 0. && feps.M > 0.)
                    r_smoke = 1000. * (1000.* totalemissions) / feps.M;
                else
                    r_smoke = 0.;


/* replace missing values (-9999) or any negative plume heights with 0. */
                if (feps.dz < 0.)
                {
                    feps.dz = 0.;
                    feps.M = 0.;
                }


            }  // if (strcasecmp(fbp.FuelType, "NF"))
            else
            {
                A = 0.;
                feps.area = 0.;
                feps.growth = 0.;
                totalemissions = 0.;
                totalCOemissions = COemissions = 0.;    // 2023-03-29
				totalarea = 0.;	// 2023-08-07
                totalgrowth = 0.;
                r_smoke = 0.;
                feps.dz = 0;
                feps.M = 0.;
                dH = -999.;
                discovery = 0;
                feps.Qo = 0.;
                feps.Qplume = 0.;
                dzdry = dzwet = dzwetter = 0.;
            }

/* 2.m.  Print results to the screen. */
//if (feps.print==-2) printf("2.m.\n");

//    if (cmcUTC==23) // a trick that prints out just the 24 hour growth (Area(t) - estarea), useful for evaulation
// printf ("%d %d %d %s\n", cmcdate, cmcUTC, region, fbp.FuelType);

/*    if (cmcUTC==24  ||
    (cmcdate==20150309 && cmcUTC==25) ||
    (cmcdate==20151101 && cmcUTC==23) ||
    (cmcdate==20160101 && cmcUTC==00) ) */

//        if (feps.region==1 || feps.region==2)

//    if (feps.region == 13)
//        if (pregion>0)  // from region>0... not sure why but if the first record is missing a region (=X), then it dies  -- 2022-06-14
//            if (strcmp(fbp.FuelType, "NF"))
//                printf("%d %d %s\n", feps.region, pregion, fbp.FuelType);

//    if (cmcUTC==0) // a trick that prints out just the 00 hour growth (Area(t) - estarea), useful for evaluation
//    if (feps.region == 12) // limit to "SEAS"
//    if (feps.region == 13) // limit to "EQAS"
//    if (cmcUTC==24)

        if (pregion>0) { // if (pregion==14) printf("%d '%s' ", pregion, fbp.FuelType); // from region>0... not sure why but if the first record is missing a region (=X), then it dies  -- 2022-06-14
            if (strncmp(fbp.FuelType, "NF", 2))
            {
// lat, lon, rep_date, source, sensor, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, tfc, bfc, hfi, estarea,

               fprintf(outfile, "%.4lf, %.4lf, %s, %s, %s, %.1lf, %.1lf, %.1lf, %.1lf, %.1lf, %s, %lf, %lf, %lf, %lf, %lf, %.0lf, %.2lf,",
                    fbp.LAT, fbp.LON, feps.rep_date, feps.source, feps.sensor, fbp.FFMC, feps.DMC, feps.DC, fbp.WS,
//                    feps.FWI, fbp.FuelType, fbp.ROS, fbp.SFC, fbp.TFC-fbp.SFC, fbp.TFC, feps.bfc, fbp.HFI, feps.estarea);
                    feps.FWI, fbp.FuelType, ROSd, SFCd, CFCd, TFCd, feps.bfc, HFId, feps.estarea);    // print conditions at time of detection for comparison to FRP
               if (!strncasecmp(feps.method, "global", 3) || !strncasecmp(feps.method, "tropomi", 3))
                    fprintf(outfile, " %s, %d, %d, %s, %s, %lf, %lf, %lf, ", feps.glc_region, feps.region, feps.glc, feps.country, feps.giglio, feps.FRP, feps.GSI, fbp.FMC);
                fprintf(outfile, " %s, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, ",
                    feps.cmc_date, feps.temp, feps.rh, fbp.WS, feps.precip, feps.ts, feps.t850, feps.t700, feps.t500, feps.t250, feps.zs, feps.z850, feps.z700, feps.z500, feps.z250);

                fprintf(outfile, " %lf, %lf, %lf, %lf, %d, %lf", feps.area, feps.growth, totalemissions, r_smoke, (int)feps.dz, feps.M);
                fprintf(outfile, ", %lf, %lf", feps.Qo, feps.Qplume);


                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                {
                    fprintf(outfile, ", %.4e", emissions.f[iSpecies][itimestep]);
                    fprintf(outfile, ", %.4e", emissions.s[iSpecies][itimestep]);
                    fprintf(outfile, ", %.4e", emissions.r[iSpecies][itimestep]);
                }

                if (!strncasecmp(feps.method, "tropomi", 3))
                    for (i=0; i<100; i++)
                        fprintf(outfile, ", %.2lf", Cy[i]);

                fprintf(outfile, "\n");
            }} // just track 24 hour area and growth (without the "if (cmcUTC==24)" statement, the brackets have no effect)

// if (pregion<=5) printf("%d %s %lf %lf\n", pregion, regions[pregion], totalCOemissions, totalemissions);

//printf("... %s", feps.rep_date);
/* 2.n. Read next line */
if (feps.print==-2) printf("2.n.\n");
            fgets(buffer, 640, csvfile);

        } while (!feof(csvfile) );

    fclose(csvfile);
    fclose(outfile);


//fopen("Final.csv", "a");
    Final[pregion] = Final[pregion] + totalemissions;
    Final[0] = Final[0] + totalemissions;		// global values
    FinalCO[pregion] = FinalCO[pregion] + totalCOemissions;
    FinalCO[0] = FinalCO[0] + totalCOemissions;	// global values
//	FinalArea[pregion] = FinalArea[pregion] + totalarea;
//	FinalArea[0] = FinalArea[0] + totalarea;	// global values
	FinalArea[pregion] = FinalArea[pregion] + totalgrowth;
	FinalArea[0] = FinalArea[0] + totalgrowth;	// global values


    FinalCO[pregion] = FinalCO[pregion] + totalCOemissions;
    FinalCO[0] = FinalCO[0] + totalCOemissions;

    glc_emissions[glc] = glc_emissions[glc] + totalemissions;   // track emissions per glc type -- 2023-06-27
    glc_region_emissions[glc][pregion] = glc_region_emissions[glc][pregion] + totalemissions;   // 2023-07-04
//    glc_areas[glc] = glc_areas[glc] + totalarea;   // track emissions per glc type -- 2023-06-27
    glc_areas[glc] = glc_areas[glc] + totalgrowth;   // track emissions per glc type -- 2023-10-25
    totalemissions = 0.;
    totalCOemissions = COemissions = 0.;
	totalarea = 0.;
	totalgrowth = 0.;

//    for (i=1;i<5;i++)
//        totalemissions = totalemissions + Final[i];

//    printf("emissions: %lf\n\n", totalemissions);    // print the total emissions


//    for (i = 0;i<;i++)    // Giglio et al (2006) regions
//    for (i=1;i<5;i++)    // North America quadrants
//    for (i = 0; i<15; i++)  // Giglio et al (2006) regions (O=total, 1="BONA", etc.)
//        printf("%s: %lf\n", regions[i], Final[i]);      // print emissions per region

//    for (i = 0; i<15; i++)  // Giglio et al (2006) regions (O=total, 1="BONA", etc.)
//        printf("%s: %lf\n", regions[i], FinalCO[i]);      // print emissions per region

//    printf(" %lf tonnes\n", Final[0]);
//    printf("%d %lf hectares", feps.ddate, totalarea);


// New: write to files
    glc_count_file = fopen("glc_count.csv", "a");
    glc_emissions_file = fopen("glc_emissions.csv", "a");
    glc_areas_file = fopen("glc_areas.csv", "a");

    fprintf(glc_count_file, "%d", feps.ddate);
    fprintf(glc_emissions_file, "%d", feps.ddate);
    fprintf(glc_areas_file, "%d", feps.ddate);
    for (i = 0; i<24; i++)  // Giglio et al (2006) regions
    {
        fprintf(glc_count_file, ", %d", glc_count[i]);      // print emissions per region
        fprintf(glc_emissions_file, ", %lf", glc_emissions[i]);      // print emissions per region
        fprintf(glc_areas_file, ", %lf", glc_areas[i]);      // print emissions per region
    }
    fprintf(glc_count_file, "\n");
    fprintf(glc_emissions_file, "\n");
    fprintf(glc_areas_file, "\n");
    fclose(glc_count_file);
    fclose(glc_emissions_file);
    fclose(glc_areas_file);

// New: write to files -- 2023-07-04
//    glc_region_count_file = fopen("glc_region_count.csv", "a");
//    glc_region_emissions_file = fopen("glc_region_emissions.csv", "a");

	for (j = 0; j < 15; j++)
	{
		sprintf(filename, "glc_%s_count.csv", regions[j]);
		glc_region_count_file = fopen(filename, "a");
		sprintf(filename, "glc_%s_emissions.csv", regions[j]);
		glc_region_emissions_file = fopen(filename, "a");

		fprintf(glc_region_count_file, "%d, %s", feps.ddate, regions[j]);
		fprintf(glc_region_emissions_file, "%d, %s", feps.ddate, regions[j]);
		for (i = 0; i<24; i++)  // Giglio et al (2006) regions
		{

			fprintf(glc_region_count_file, ", %d", glc_region_count[i][j]);      // print emissions per region
			fprintf(glc_region_emissions_file, ", %lf", glc_region_emissions[i][j]);      // print emissions per region
		}
		fprintf(glc_region_count_file, "\n");
		fprintf(glc_region_emissions_file, "\n");
//	}
		fclose(glc_region_count_file);
		fclose(glc_region_emissions_file);
	}

// New: write to files
    Final_file = fopen("Final.csv", "a");
    Final_CO_file = fopen("FinalCO.csv", "a");
    Final_Area_file = fopen("FinalArea.csv", "a");

    fprintf(Final_file, "%d", feps.ddate);
    fprintf(Final_CO_file, "%d", feps.ddate);
    fprintf(Final_Area_file, "%d", feps.ddate);


    for (i = 0; i<15; i++)  // Giglio et al (2006) regions
    {
        fprintf(Final_file, ", %lf", Final[i]);      // print emissions per region
        fprintf(Final_CO_file, ", %lf", FinalCO[i]);      // print CO emissions per region
        fprintf(Final_Area_file, ", %lf", FinalArea[i]);      // print area per region
    }
    fprintf(Final_file, "\n");
    fprintf(Final_CO_file, "\n");
    fprintf(Final_Area_file, "\n");

    fclose(Final_file);
    fclose(Final_CO_file);
    fclose(Final_Area_file);

//    printf("\n");
//printf ("3\n");

// use this for Lac La Loche study results
//        if (strstr(feps.csvfilename, "Loche")!=NULL)
//            for (itimestep=0; itimestep<ntimesteps; itimestep++)
//                printf("%e\n", GridTotalEmissions[itimestep]);    // print the last hour's values of 72 hour forecast
    }  //   if (!strcasecmp(feps.method, "cmc") )  /* CMC Firework method *////////////////////////////////////////////////

/* the following section is included to test PlumeCalcDry(), the piece-wise plume rise calculation, following an ICAO standard atmosphere */



    else if (!strcasecmp(feps.method, "BIGFOOT") ) ////////////////////////////////////////////////////////////////////////
    {

// Using feps.ddate & feps.dtime from the input ini file as the model's start time
        cmcDATEo = feps.ddate;  // YYYYMMDD
        cmcUTCo = feps.dtime; // HHMM -  note this is different form earlier use of cmcUTCo, which is in decimal hours.

        for (itimestep=0; itimestep<ntimesteps; itimestep++)
            GridTotalEmissions[itimestep]=0.;

/* Determine CMC julian date */
            if (cmcDATEo> 0)
            {

                if (cmcDATEo > 9999)
                    iyear = (int)(cmcDATEo/10000);
                else
                    iyear = 0;
                imonth = (int)((cmcDATEo-10000*iyear)/100);
                iday = cmcDATEo-10000*iyear-100*imonth;
                cmcDj = jdays[imonth-1]+iday;
                if (iyear > 0 && iyear%4 == 0 && imonth >2) cmcDj+=1;

            }  // iyear, imonth and iday will be overwritten

/* 1.e. Open CMC GEM forecast CSV file as input.  Read in header line */
if (feps.print==-2) printf("1.e.\n");
        if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL) {
            printf("Error: could not find %s\n\n", feps.csvfilename);

        } else if ( (outfile = fopen(feps.outfilename, "w")) == NULL) {
            printf("Error: could not write to %s\n\n", feps.outfilename);
        }
        else
        {

// read (and ignore) header info
            fgets(buffer, 640, csvfile);

            fbp.LFL = 0.;
            fbp.DFL = 0.;
            fbp.HFL = 0.;
            if (strstr(buffer, "lfl")==NULL && strstr(buffer, "LFL")==NULL)
            {
                fbp.LFL = -1.;
                fbp.DFL = -1.;
                fbp.HFL = -1.;
            }

// print header info
//            if (!strcasecmp(feps.method, "cmc")   || !strcasecmp(feps.method, "cmc3")  ) // seems we're already in (!strcasecmp(feps.method, "cmc")
            {

/* 1.f. Print (to the screen), new header line for the CFFEPS predictions CSV file. */
if (feps.print==-2) printf("1.f.\n");
                fprintf(outfile, "lat, lon, rep_date, source, sensor, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, tfc, bfc, hfi, estarea, ");
                fprintf(outfile, "UTC, temp, rh, ws, precip, TS, T850, T700, T500, T250, ZS, Z850, Z700, Z500, Z250, Area(t), Growth (t), TotalEmissions, r_smoke, Zplume, Mplume, Mw, Qo, Qplume");
                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                {
                    fprintf(outfile, ", %s F", emissions.species[iSpecies]);
                    fprintf(outfile, ", %s S", emissions.species[iSpecies]);
                    fprintf(outfile, ", %s R", emissions.species[iSpecies]);
                }
                fprintf(outfile, "\n");
            }
            status = ReadHotspotHeader(&buffer, &fields);
// read in first line of data
            fgets(buffer, 640, csvfile);
        }  // if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL)... else

/* 2. Begin reading CFFEPS predictions CSV file */
if (feps.print==-2) printf("2.\n");
        do  /* while !feof(csvfile); */
        {

/* 2.b. Read line of data into buffer.  Parse comma separated variables in buffer.
        Make adjustments for possible variation in units (e.g Kelvin instead of Celsius, etc.) */
if (feps.print==-2) printf("2.b.\n");


// read in first line of data
            status = ReadHotspotData(&buffer, &fields, &feps, &fbp, &ua);

// set these back to internal variables
            estarea = feps.estarea;
            precip = feps.precip;
            strcpy(rep_date, feps.rep_date);
            strcpy(cmc_date, feps.cmc_date);

            strcpy(source, "BIGFOOT");
            strcpy(sensor, "NA");

/* 2.d. Determine if the line read in is a new hotspot
        (if the latitude, longitude or date does not equal previous line’s values).
        If new, reset arrays to zero. */
if (feps.print==-2) printf("2.d.\n");

            feps.area=0.; //=estarea;
            totalemissions = 0.;  // zero everything including total emissions
            feps.Mw = 0.;
            feps.Qo = Qo = 0.;    // Qo is used to temporarily store the energy when comparing height predictions using different lapse rates
            feps.Qplume = 0;
            feps.growth=0.;
            feps.Flaming=0.;
            feps.Smoldering=0.;
            feps.Residual=0.;
            fbp.ROS = fbp.SFC = fbp.TFC = fbp.HFI = 0.;

/* elapsed is the time since detectionLST (dhrs) */
            elapsed = 0.;
            timestep = feps.timestep;
            r_smoke = 0.;
            feps.dz = 0;
            feps.M = 0.;

            for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                for (itimestep=0; itimestep<ntimesteps; itimestep++)
                {

                    Qs[itimestep]=0.;
                    Mws[itimestep]=0.;
                    emissions.f[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                    emissions.s[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                    emissions.r[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                    FCs[itimestep]=0.;
                    Fs[itimestep]=0.;       // new fire, set concentrations back to zero
                    Ss[itimestep]=0.;       // new fire, set concentrations back to zero
                    Rs[itimestep]=0.;       // new fire, set concentrations back to zero
                }

/* 2.f. Determine time zone (longitude/15) and parse detection date and time.
        Adjust detection time from UTC to local time such that noon is at solar zenith .
        Adjust LDT to LST (hardcoded to LST?).
        Determine UTC and LST time of the forecast record. */
if (feps.print==-2) printf("2.f.\n");
            timezone = (fbp.LON)/15.;
            feps.LDT = 0.;

/* Determine detection date and time */
            // 2014-04-01 20:01:00 or
            // 2018-06-15 2:47

            feps.ddate = atoi(&rep_date[0])*10000 + atoi(&rep_date[5])*100 + atoi(&rep_date[8]);
            if (atoi(&rep_date[11]) < 10.)
                feps.dtime  = atoi(&rep_date[11])*100 + atoi(&rep_date[13]);    // currently in UTC
            else
                feps.dtime  = atoi(&rep_date[11])*100 + atoi(&rep_date[14]);    // currently in UTC

///// Set the first cmc dates and times to rounded hour (drop minutes) of detection time /////
// e.g. 20160216_00:00
//            cmcdate = feps.ddate;
//            cmcLST = atoi(&rep_date[11]);   // hrs

/* Determine julian date */
            if (fbp.Dj = -1)
            {
                if (feps.ddate > 9999)
                    iyear = (int)(feps.ddate/10000);
                else
                    iyear = 0;
                imonth = (int)((feps.ddate-10000*iyear)/100);
                iday = feps.ddate-10000*iyear-100*imonth;
                fbp.Dj = jdays[imonth-1]+iday;
                if (iyear > 0 && iyear%4 == 0 && imonth >2) fbp.Dj+=1;
            }

/* Adjust LDT value to LST */
            if (feps.LDT == 100 || feps.LDT == 1)
                feps.LDT = 1;
            else
                feps.LDT = 0;

            detectionUTC = (int)(feps.dtime / 100) + (feps.dtime % 100) /60.;
            detectionLST = (int)(feps.dtime / 100) + (feps.dtime % 100) /60. - feps.LDT + (int)timezone;

            if (detectionLST < 0.)
            {
                detectionLST = detectionLST + 24.;
                fbp.Dj = fbp.Dj;    // keep fbp.Dj as the UTC date
            }

/* 2.g. Recalculate the BUI value from DMC and DC values.  Determine whether the BUI effect is being used. */
if (feps.print==-2) printf("2.g.\n");
            fbp.BUI = BUIcalc(feps.DMC, feps.DC);


/* If BUI is zero or less, BUIeff = False */
            if (fbp.BUI <= 0) fbp.BUIEff = -1;

            Do = -1; /* converted so Do value is returned (but not used in CFFEPS) -- 2019-07-29 */
            fbp.FMC = FMCcalc(fbp.LAT, fbp.LON, feps.zs, fbp.Dj, &Do);

            FFMCy = FFMC = fbp.FFMC;     /* we are assuming the value passed in FBP is the noon FFMC */

            current = floor(cmcUTCo/100.) + (cmcUTCo%100)/60.;  // current time in UTC (accumulated from the start time as defined by cmcUTCo)
            hour = mtime(current + (int)timezone);  // mtime turns current time into military time, adjusting for 24 hour offsets

 /* Calculate yesterday's FFMC */
            if (hour < 1200)
            {
                FFMCy = FFMC; FFMCt=0.;    /* first guess */
                FFMCnoon = bdl_ffmc(FFMC, 1200, (int)feps.rh);
                i=0;
                while (fabs(FFMCnoon-FFMCt) > .1 && FFMCy > 50. && FFMCy < 100. && i < 100)
                {
                    FFMCt = bdl_ffmc(FFMCy, 1159, (int)feps.rh);
                    FFMCy = FFMCy + (FFMCnoon-FFMCt);
                    i++;
                }
                if (i==100) FFMCy = (FFMCt+FFMCy)/2;
                if (FFMCy>99.) FFMCy=99.;
            }

            if (FFMCy < 60. && strcasecmp(feps.method, "AB") && strncasecmp(feps.method, "cmc", 3) )    // we're in BIGFOOT, neither of these feps.methods are true
            {
//                fprintf(outfile, "Low FFMC: %lf set to 75.\n", FFMCy);
                FFMCy = 75.;
            }

/* adjust cmcUTC to a time wrt detection time and date, so this is 0 at detection time, negative before and positive after (can exceed 24) */
//            cmcUTC = cmcUTC + 24*(cmcDj - fbp.Dj);    // should use a new variable name like "elapsed"

// start loop around here

            for (itimestep=0.; itimestep<ntimesteps; itimestep++)    // 72 hour forecast
            {
                dt = (double)itimestep/timestep;    // track the model time from the start
                if (itimestep > 0) current = current + timestep;
                elapsed = (24.*cmcDj+current) - (24.*fbp.Dj+detectionUTC);   // time [hrs] from detection to current timestep

/* 2.i. Calculate FFMC from yesterday’s value using the technique described in Lawson et al. (1996).
        This is based on the observed noon FFMC.
        If the time is before noon, yesterday’s FFMC is required.
        This is converged on so that yesterday’s FFMC will predict today’s observed FFMC
        (set to 11:59AM, 23:59 hours later). */
if (feps.print==-2) printf("2.i.\n");

//                if (elapsed < 0. || elapsed >= 1.)  // really need to do one timestep not one hour
                if ( floor(elapsed/timestep) != 0. )
                    feps.growth = 0.;    // and skip all the calculations
                else // 0. < elapsed <= 1.
                {
                    feps.growth = estarea;
                    feps.area = feps.area+feps.growth;

                    hour = mtime(current + (int)timezone);

                    if (!strncmp(feps.diurnal,"BDL",3))  /* BDL diurnal effect -- 2020-06-04 */
                        if (hour < 1200)
                            fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
                        else
                            fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);
                    else if (!strncmp(feps.diurnal,"CVW",3))  /* CVW diurnal effect -- 2020-06-05 */
                    {
                        fbp.FFMC = hourly_ffmc(feps.temp, feps.rh, feps.ws, precip, fbp.FFMC);
                    }
                    else if (!strncmp(feps.diurnal,"EMC",3))  /* EMC diurnal effect -- 2020-06-04 */
                    {
                        fbp.FFMC = eq_ffmc(feps.temp, feps.rh, feps.ws, precip, zero);
                    }   // else fbp.FFMC will be constant

// replaced this -- 2020-06-15
//                   if (hour < 1200)
//                        fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
//                    else
//                        fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);
                    fbp.t = 0.; // not interested in time related output from FBP

                                if (feps.elapsed < 0.) feps.elapsed = 0.;

// fbp.FC_method = 4;   /* was hard coded in, now include this in INI file 2020-02-19 */
                    status = FBPCalc (&fbp);    // FBP at rep_dat
// if (fbp.CFB> 0.) printf("... %lf %lf %lf %lf\n", fbp.ROS, fbp.CFB, fbp.CFL, fbp.TFC);
                    if (status==0)
                    {
                        dz8 = dz7 = dz5 = dz = -999.;
                        l8 = l7 = l5 = -999.;
                        r_smoke = 0.;
/* 2.l.iii. Calculate energy of the fire */
if (feps.print==-2) printf("2.l.iii.\n");
                        feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);

/* 2.l.iv. Calculate the energy released from flaming, smoldering and residual combustion.
            Calculate flaming, smoldering and residual emissions rates in tonnes/h  */
if (feps.print==-2) printf("2.l.iv.\n");
                        EmissionsCalc(&feps, &fbp, &emodels, &allocations);
                            emissions.residence_flaming=0.25;   /* residence time for flaming combustion (hours) -- remains the same */
                            emissions.residence_smoldering = feps.DOB_S/feps.SCR/2.; /* CSR = smoldering combustion rate [cm/hr] (Dan Thompson) */
                            emissions.residence_residual = feps.DOB_S/feps.SCR/2.;   /* split equally between smoldering and residual */

/* 2.l.v. Distribute the energy over time */
if (feps.print==-2) printf("2.l.v.\n");
                        jModel = -1;    /* an admittedly awkward way of creating hourly energy components */
                        iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                        for (i=0; i< iMax; i++)
                            if (i+itimestep <= ntimesteps)
                                Qs[i+itimestep] = Qs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];

/* 2.l.v. Distribute the water released [kg/timestep] over time, required for the subroutine PlumeCalcWet -- 2019-07-17*/
if (feps.print==-2) printf("2.l.v.\n");
                        jModel = -2;    /* an admittedly awkward way of creating hourly water components */
                        iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                        for (i=0; i< iMax; i++)
                            if (i+itimestep <= ntimesteps)
                                Mws[i+itimestep] = Mws[i+itimestep] + Fs[i]+Ss[i]+Rs[i];

/* 2.l.v. Distribute the fuel consumption over time */
if (feps.print==-2) printf("2.l.v.\n");
                        jModel = -3;    /* an admittedly awkward way of creating hourly energy components */
                        iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                        for (i=0; i< iMax; i++)
                            if (i+itimestep <= ntimesteps)
                            {
                                FCs[i+itimestep] = FCs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                                GridTotalFs[i+itimestep] = GridTotalFs[i+itimestep] + Fs[i];
                                GridTotalSs[i+itimestep] = GridTotalSs[i+itimestep] + Ss[i];
                                GridTotalRs[i+itimestep] = GridTotalRs[i+itimestep] + Rs[i];
                            }

/* 2.l.vii. Find emodel that matches the fuel type */
if (feps.print==-2) printf("2.l.vii.\n");
                        if (emissions.nModels==0 || emodels.nModels==0)
                            jModel=0;
                        else
                            for (iFuel=0; iFuel<MAX_CWFIS; iFuel++)
                                if (!strcasecmp(emodels.FuelType[iFuel], fbp.FuelType)) // found a match
                                    for (iModel=0;iModel<emissions.nModels;iModel++)
                                        if (emissions.emodel[iModel]==emodels.emodel[iFuel] )
                                            jModel=iModel;

/* 2.l.viii.  Distribute the emission over time */
if (feps.print==-2) printf("2.l.viii.\n");
                        for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                        {
                            iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);
                            for (i=0; i< iMax; i++)
                                if (i+itimestep <= ntimesteps)
                                {
                                    emissions.f[iSpecies][i+itimestep]=emissions.f[iSpecies][i+itimestep]+Fs[i];
                                    emissions.s[iSpecies][i+itimestep]=emissions.s[iSpecies][i+itimestep]+Ss[i];
                                    emissions.r[iSpecies][i+itimestep]=emissions.r[iSpecies][i+itimestep]+Rs[i];
                                }
                        }

                    }   // if (status==0)
                } // if (elapsed <= 0. && elapsed > 1.)

                feps.Qplume = Qs[itimestep];  // need to check this (only care about the first hour?)
                feps.dz = dz = PlumeCalc(&feps); // the BigFoot routine is based strictly on an input lapse rate
                feps.Qo = feps.Qo + feps.Qplume;    // I used to do this in PlumeCalc but the new EmissionsCalc has thrown this off
                feps.Mw = feps.Mw + Mws[itimestep];       // Cumulative mass of water released [kg] from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/

/* 2.l.ix. Calculate total emissions and r_smoke */
if (feps.print==-2) printf("2.l.ix.\n");

                totalemissions = totalemissions + FCs[itimestep];

                GridTotalEmissions[itimestep] = GridTotalEmissions[itimestep] + totalemissions;
                GridTotalMws[itimestep] = GridTotalMws[itimestep] + feps.Mw;

                if (feps.dz > 0 && feps.M > 0)
                    r_smoke = 1000 * (1000.* totalemissions) / feps.M;
                else
                    r_smoke = 0.;

/* replace missing values (-9999) or any negative plume heights with 0. */
                if (feps.dz < 0.)
                {
                    feps.dz = 0.;
                    feps.M = 0.;
                }

/* 2.m.  Print results to the screen. */
if (feps.print==-2) printf("2.m.\n");


// lat, lon, rep_date, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, tfc, bfc, hfi, estarea,
                fprintf(outfile, "%.4lf, %.4lf, %s, %s, %s, %.1lf, %.1lf, %.1lf, %.1lf, %.1lf, %s, %lf, %lf, %lf, %lf, %.0lf, %.2lf,",
                        fbp.LAT, fbp.LON, rep_date, feps.source, feps.sensor, fbp.FFMC, feps.DMC, feps.DC, fbp.WS,
                        feps.FWI, fbp.FuelType, fbp.ROS, fbp.SFC, fbp.TFC, bfc, fbp.HFI, estarea);


// e.g. 20160216_00:00
                cmcUTC=mtime(current);
                sprintf(cmc_date, "%d_%02d:%02d", cmcDATEo + (int)(current/24.), cmcUTC/100, cmcUTC%100);

                fprintf(outfile, " %s, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf,",
                        cmc_date, feps.temp, feps.rh, fbp.WS, precip, feps.ts, feps.t850, feps.t700, feps.t500, feps.t250, feps.zs, feps.z850, feps.z700, feps.z500, feps.z250);

                fprintf(outfile, " %lf, %lf, %lf, %lf, %d, %lf, %lf", feps.area, feps.growth, totalemissions, r_smoke, (int)feps.dz, feps.M, feps.Mw);
                fprintf(outfile, ", %lf, %lf", feps.Qo, feps.Qplume);

                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                {
                    fprintf(outfile, ", %.4e", emissions.f[iSpecies][itimestep]);
                    fprintf(outfile, ", %.4e", emissions.s[iSpecies][itimestep]);
                    fprintf(outfile, ", %.4e", emissions.r[iSpecies][itimestep]);
                }
                    fprintf(outfile, "\n");

            } // for (itimestep=0.; itimestep<=72; itimestep++)

/* 2.n. Read next line */
if (feps.print==-2) printf("2.n.\n");
            fgets(buffer, 640, csvfile);

        } while (!feof(csvfile));

// use this for La La Loche study results
        if (strstr(feps.csvfilename, "Loche")!=NULL)
        {
            for (itimestep=1; itimestep<ntimesteps; itimestep++)    // need to accumulate these
            {
//                GridTotalMws[itimestep] = GridTotalMws[itimestep]+GridTotalMws[itimestep-1];
                GridTotalFs[itimestep] = GridTotalFs[itimestep]+GridTotalFs[itimestep-1];
                GridTotalSs[itimestep] = GridTotalSs[itimestep]+GridTotalSs[itimestep-1];
                GridTotalRs[itimestep] = GridTotalRs[itimestep]+GridTotalRs[itimestep-1];
            }

            for (itimestep=0; itimestep<ntimesteps; itimestep++)
            {
                printf("%e ", GridTotalEmissions[itimestep]);
//                printf("%e ", GridTotalMws[itimestep]);    // print the last hour's values of 72 hour forecast [tonnes], [kg]
//                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
//                    printf(" %e", emissions.a[0][iSpecies]*GridTotalFs[itimestep]+ emissions.b[0][iSpecies]*GridTotalSs[itimestep]+emissions.c[0][iSpecies]*GridTotalRs[itimestep]); // [kg]
                printf("\n");
            }
        }
    }   //     else if (!strcasecmp(feps.method, "BIGFOOT") )

 ////////////////////////////////////////////////////////////////////////*
/*    else if (!strcasecmp(feps.method, "OVERLAP") )
    {
        plat = plon = cmcdate = -999;
        i = iii = 0;
        A = AA = AAA = 0.;

/* 1.e. Open CMC GEM forecast CSV file as input.  Read in header line * /
        if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL) {
            printf("Error: could not find %s\n\n", feps.csvfilename);
        }

        else
        {

// read (and ignore) header info
            fgets(buffer, 640, csvfile);

            status = ReadHotspotHeader(&buffer, &fields);

// read in first line of data
            fgets(buffer, 640, csvfile);

            do  // while !feof(csvfile);
            {

                status = ReadHotspotData(&buffer, &fields, &feps, &fbp, &ua);

                if (fbp.LAT < 56.3 || fbp.LAT > 56.56 || fbp.LON < -109.95 || fbp.LON > -109.65)
                    strcpy(fbp.FuelType, "NF"); // ignore these records
                else
                    if (plat==fbp.LAT && plon==fbp.LON) //  && cmcdate <= atoi(&cmc_date[0])) // same fire
                        newfire = false; // false -- ignore these records too
                    else    // new fire
                    {

                // e.g. 20160216_00:00
                        cmcdate = atoi(&cmc_date[0]);

                        lats[i] = plat = fbp.LAT;
                        lons[i] = plon = fbp.LON;
                        estareas[i] = feps.estarea;
                        AA = AA + feps.estarea;
                        if (i > 0)
                        for (j=0;j<i;j++)
                        {

                            A = overlap(lats[i], lons[i], estareas[i], lats[j], lons[j], estareas[j]);
                            if (A > 0.)
                            {

                                AAA = AAA + A;  // cumulative overlapping areas
                                iii++;          // count of hotspots with overlapping areas
                            }
                        }
                        i++;
                    }

// read in next line of data
                fgets(buffer, 640, csvfile);

            } while (!feof(csvfile) );

            printf ("%lf in %d hotspots\n%lf ha in %d overlapping areas\n", AA, i, AAA, iii);
        }
    }
*/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    else if (!strcasecmp(feps.method, "ICAO") )
    {
        feps.Qo = 0.;
        feps.Qplume = 8.35E10*feps.area;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m AGL

// from data stored in infile.dat
        feps.dz = PlumeCalc(&feps);
        printf("1. PlumeCalc (infile.dat): %lf\n\n", feps.dz);

        /* ICAO Standard Atmosphere */

        feps.ts = 14.3;
        feps.zs = 111.;
        feps.pressure = 1000.;
        feps.area = 1.0;
        feps.perimeter=0.;
        feps.residencetime=0.;
        feps.radiation = 100;
        feps.alpha=0.;
        feps.lapse = -6.5/1000.;
        feps.growth=-1.;
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m AGL
        feps.dz = PlumeCalc(&feps);
        printf("2. PlumeCalc (lapse rate): %lf\n\n", feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.lapse = (feps.t500-feps.t850)/(feps.z500-feps.z850);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalc(&feps);
        printf("3. PlumeCalc (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.lapse = -999.;
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalc(&feps);
        printf("4. PlumeCalc (%s): %lf\n\n", profilename, feps.dz);

printf("**************************\n");

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("5. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_8752.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("6. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_752.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("7. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_852.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("8. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_872.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
//        feps.Qplume = (1636.5+254.8364)*5102*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("9. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_87642.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
//        feps.Qplume = (1636.5+254.8364)*5102*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 5463 m
        feps.dz = PlumeCalcDry(&feps, &ua);
        printf("10. PlumeCalcDry (%s): %lf\n\n", profilename, feps.dz);

// feps.Qplume = 1842.881*5102*10000.*feps.area;   // using ICAO2 with an isothermal layer under 800 mb, should get the same height using this energy
// feps.Qplume = (1842.881+254.8364)*5102*10000.*feps.area;
// feps.Qplume = 2097.717*5102*10000.*feps.area;

        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm.profile");
        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_8752.profile");
        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_872.profile");
        sprintf(profilename, "D:\\Alberta Smoke Monitoring\\Soundings\\ICAO_2015062718_gpm_87642.profile");
        iLevels = Readupperair(profilename, &feps, &ua);
        feps.Qo = 0.;
        feps.Qplume = 1636.5*5102.*10000.*feps.area;    // using a standard atmosphere, plume height should be 6889? m
        feps.dz = PlumeCalcWet(&feps, &ua);
        printf("11. PlumeCalcWet (%s): %lf\n\n", profilename, feps.dz);
    }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    else if (!strcasecmp(feps.method, "FGM") )
    {

// NOTE: variables ncols, nrows, xllcorner, yllcorner, cellsize are global variables declared in feps_2011.h (was pfas.h)

/////////////////////////////////////////////////////
///   Part 1: Build ignition grid from hotspots   ///
/////////////////////////////////////////////////////
        missing=-999;
        for (i=0; i< MAX_STNS;i++)
        {
            for (j=0; j < MAX_HOURS+1; j++)
            {
                fgmwx.Ts[i][j] = missing;
                fgmwx.rhs[i][j] = missing;
                fgmwx.Ws[i][j] = missing;
                fgmwx.wds[i][j] = missing;
                fgmwx.rains[i][j] = missing;
                fgmwx.FFMCts[i][j] = missing;
            }
            for (j=0; j < fgmwx.ndays; j++)
            {
                fgmwx.DMCs[istn][j] = missing;
                fgmwx.DCs[istn][j] = missing;
            }
            fgmwx.xs[i] = missing;
            fgmwx.ys[i] = missing;
        }

        plat = plon = missing;
        newfire = true;
        istn = -1; // start at -1 so fire record sets this to 0
        fgmwx.ndays = 1;
        fgmwx.greenup=0;
        fgmwx.standing=1;
        fgmwx.cured = 95.0;

        strcpy(profilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\fuels2a.prj"); // Projection file name

//        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\fbp_test.asc"); // a test grid for a single hotspot
        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ascii_fuels.asc");
printf ("Reading %s\n", infilename);
        FBPg = read_int_ASCIIGRID(infilename);
        FBPg0 = FBPg;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_aspect.asc");
printf ("Reading %s\n", infilename);
        aspect = read_int_ASCIIGRID(infilename);
        aspect0 = aspect;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_slope.asc");
printf ("Reading %s\n", infilename);
        slope = read_int_ASCIIGRID(infilename);
        slope0 = slope;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_cbh.asc");
printf ("Reading %s\n", infilename);
        cbh = read_double_ASCIIGRID(infilename);
        cbh0 = cbh;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_cfl.asc");
printf ("Reading %s\n", infilename);
        cfl = read_double_ASCIIGRID(infilename);
        cfl0 = cfl;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_dfl.asc");
printf ("Reading %s\n", infilename);
        dfl = read_double_ASCIIGRID(infilename);
        dfl0 = dfl;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_pc.asc");
printf ("Reading %s\n", infilename);
        pc = read_double_ASCIIGRID(infilename);
        pc0 = pc;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_sd.asc");
printf ("Reading %s\n", infilename);
        sd = read_double_ASCIIGRID(infilename);
        cbh0 = sd;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_sfl.asc");
printf ("Reading %s\n", infilename);
        sfl = read_double_ASCIIGRID(infilename);
        sfl0 = sfl;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ultra\\ascii_sh.asc");
printf ("Reading %s\n", infilename);
        sh = read_double_ASCIIGRID(infilename);
        sh0 = sh;

/* turn off for now  */
        slope0 = NULL;
        aspect0 = NULL;
        cbh0 = NULL;
        cfl0 = NULL;
        dfl0 = NULL;
        pc0  = NULL;
        sd0  = NULL;
        sfl0 = NULL;
        sh0  = NULL;

/* Allocate memory for array */
        ign0 = (double *)calloc(ncols*nrows, sizeof(double));
        ign=ign0;

        for(jrow=0;jrow<nrows;jrow++)
            for(icol=0;icol<ncols;icol++)
            {
//               *FBPg = 2;   // set fuels grid to homogeneous C2
//               FBPg++;

               *ign = (double)NODATA_value;
               ign++;
            }

        FBPg = FBPg0;
        ign = ign0;
        aspect = aspect0;
        slope = slope0;
        cbh = cbh0;
        cfl = cfl0;
        dfl = dfl0;
        pc = pc0;
        sd = sd0;
        sfl = sfl0;
        sh = sh0;

// Using feps.ddate & feps.dtime from the input ini file as the model's start time
        cmcDATEo = feps.ddate;  // YYYYMMDD
        cmcUTCo = feps.dtime/100;   // integer hours

/* Determine CMC julian date */
        if (cmcDATEo> 0)
        {
            if (cmcDATEo > 9999)
                iyear = (int)(cmcDATEo/10000);
            else
                iyear = 0;
            imonth = (int)((cmcDATEo-10000*iyear)/100);
            iday = cmcDATEo-10000*iyear-100*imonth;
            cmcDjo = jdays[imonth-1]+iday;
            if (iyear > 0 && iyear%4 == 0 && imonth >2) cmcDjo+=1;
        }  // iyear, imonth and iday will be overwritten

        istn=0;
        lat0=0;
        lon0=0;

/// Read in gridded surface weather data
        sprintf(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ua\\201806%02d00_72hr_lalocheGrid\\201806%02d00_001_f72_laloche_WSWD.csv", iday, iday);
        if ( (csvfile = fopen(infilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", infilename);
        else
        {
printf ("Reading %s\n", infilename);
            fgets(buffer, 640, csvfile);  // header
            fgets(buffer, 640, csvfile);
            do
            {
                token = strtok(buffer, ",");
                lat = atof(token);  token=strtok(NULL, ",");     // latitude [degrees]
                lon = atof(token);  token=strtok(NULL, ",");    // longitude [degrees]
                if (lat0==0) // first record
                {
                    lat0=lat;
                    lon0=lon;
                }

                if (lat == lat0 && lon == lon0)
                    istn = 0;
                else
                    istn++;

                cmcdate = atoi(&token[0])*10000 + atoi(&token[5])*100 + atoi(&token[8]);
                cmcUTC = atoi(&token[11]); token=strtok(NULL, ",");

                if (cmcdate >= cmcDATEo)
                {
                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo);
                    fgmwx.Ts[istn][cmcUTC] = atof(token);  token=strtok(NULL, ",");     // temperature [oC]
                    fgmwx.rhs[istn][cmcUTC] = 100.* atof(token);  token=strtok(NULL, ",");    // relative humidity [%]
                    fgmwx.Ws[istn][cmcUTC] = atof(token);  token=strtok(NULL, ",");     // wind speed [km/h]
                    fgmwx.wds[istn][cmcUTC] = atof(token);  token=strtok(NULL, ",");    // wind direction [degrees]

// as written, these manditory level values will be the for the last grid point and last time step... good enough for now but needs to be addressed
                    feps.t850 = atof(token);  token=strtok(NULL, ",");  // 850mb temperature [oC]
                    feps.t700 = atof(token);  token=strtok(NULL, ",");  // 700mb temperature [oC]
                    feps.t500 = atof(token);  token=strtok(NULL, ",");  // 500mb temperature [oC]
                    feps.t250 = atof(token);  token=strtok(NULL, ",");  // 250mb temperature [oC]
                    feps.z850 = 10.* atof(token);  token=strtok(NULL, ",");  // 850mb height [m]
                    feps.z700 = 10.* atof(token);  token=strtok(NULL, ",");  // 700mb height [m]
                    feps.z500 = 10.* atof(token);  token=strtok(NULL, ",");  // 500mb height [m]
                    feps.z250 = 10.* atof(token);                            // 250mb height [m]

                    fgmwx.rains[istn][cmcUTC] = feps.precip;
//                    fgmwx.FFMCts[istn][cmcUTC] = eq_ffmc(fgmwx.Ts[istn][cmcUTC],fgmwx.rhs[istn][cmcUTC], fgmwx.Ws[istn][cmcUTC], feps.precip, zero);
                    fgmwx.FFMCts[istn][cmcUTC] = eq_ffmc(fgmwx.Ts[istn][cmcUTC],fgmwx.rhs[istn][cmcUTC], fgmwx.Ws[istn][cmcUTC], zero, zero);
                    fgmwx.DMCs[istn][(int)(cmcUTC/24)] = feps.DMC;
                    fgmwx.DCs[istn][(int)(cmcUTC/24)] = feps.DC;

                    x=0.; y=0.;
                    get_xy(&x, &y, &lat, &lon, profilename);
                    fgmwx.xs[istn] = x;
                    fgmwx.ys[istn] = y;
                }
                fgets(buffer, 640, csvfile);
            } while (!feof(csvfile));
            fclose(csvfile);
        }

/// Read in Upper Air Data ///
        istn=0;
        lat0=0.;
        lon0=0.;

        sprintf(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ua\\201806%02d00_72hr_lalocheGrid\\201806%02d00_72hr_lalochePSnlay.csv", iday, iday);
        if ( (uafile = fopen(infilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", infilename);
        else
        {
printf ("Reading %s\n", infilename);
            fgets(buffer, 640, uafile);  // header
            fgets(buffer, 640, uafile);
            do
            {
                token = strtok(buffer, ",");
                lat = atof(token);  token=strtok(NULL, ",");     // latitude [degrees]
                lon = atof(token);  token=strtok(NULL, ",");    // longitude [degrees]
                if (lat0==0) // first record
                {
                    lat0=lat;
                    lon0=lon;
                }

                if (lat == lat0 && lon == lon0)
                    istn = 0;
                else
                    istn++;

                cmcdate = atoi(&token[0])*10000 + atoi(&token[5])*100 + atoi(&token[8]);
                cmcUTC = atoi(&token[11]);

                if (cmcdate >= cmcDATEo)
                {
                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo);
                    for (i=MAX_LEVELS-1; i>=0; i--)
                    {
                        token=strtok(NULL, ",");
                        uawx.P[istn][cmcUTC][i] = hy[i]*atof(token);     // pressure [mb]
                    }

                }
                fgets(buffer, 640, uafile);
            } while (!feof(uafile));
            fclose(uafile);
        }

        sprintf(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ua\\201806%02d00_72hr_lalocheGrid\\201806%02d00_72hr_lalocheGZnlay.csv", iday, iday);
        if ( (uafile = fopen(infilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", infilename);
        else
        {
printf ("Reading %s\n", infilename);
            fgets(buffer, 640, uafile);  // header
            fgets(buffer, 640, uafile);
            do
            {
                token = strtok(buffer, ",");
                lat = atof(token);  token=strtok(NULL, ",");     // latitude [degrees]
                lon = atof(token);  token=strtok(NULL, ",");    // longitude [degrees]
                if (lat0==0) // first record
                {
                    lat0=lat;
                    lon0=lon;
                }

                if (lat == lat0 && lon == lon0)
                    istn = 0;
                else
                    istn++;

                cmcdate = atoi(&token[0])*10000 + atoi(&token[5])*100 + atoi(&token[8]);
                cmcUTC = atoi(&token[11]);

                if (cmcdate >= cmcDATEo)
                {
                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo);
                    for (i=MAX_LEVELS-1; i>=0; i--)
                    {
                        token=strtok(NULL, ",");
                        uawx.Z[istn][cmcUTC][i] = 10.*atof(token);     // geopotential height [m]
                    }
                }
                fgets(buffer, 640, uafile);
            } while (!feof(uafile));
            fclose(uafile);
        }


        sprintf(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ua\\201806%02d00_72hr_lalocheGrid\\201806%02d00_72hr_lalocheTTnlay.csv", iday, iday);
        if ( (uafile = fopen(infilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", infilename);
        else
        {
printf ("Reading %s\n", infilename);
            fgets(buffer, 640, uafile);  // header
            fgets(buffer, 640, uafile);
            do
            {
                token = strtok(buffer, ",");
                lat = atof(token);  token=strtok(NULL, ",");     // latitude [degrees]
                lon = atof(token);  token=strtok(NULL, ",");    // longitude [degrees]
                if (lat0==0) // first record
                {
                    lat0=lat;
                    lon0=lon;
                }

                if (lat == lat0 && lon == lon0)
                    istn = 0;
                else
                    istn++;

                cmcdate = atoi(&token[0])*10000 + atoi(&token[5])*100 + atoi(&token[8]);
                cmcUTC = atoi(&token[11]);

                if (cmcdate >= cmcDATEo)
                {
                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo);
                    for (i=MAX_LEVELS-1; i>=0; i--)
                    {
                        token=strtok(NULL, ",");
                        uawx.T[istn][cmcUTC][i] = atof(token);     // temperature [oC]
                    }
                }
                fgets(buffer, 640, uafile);
            } while (!feof(uafile));
            fclose(uafile);
        }

        sprintf(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\input\\ua\\201806%02d00_72hr_lalocheGrid\\201806%02d00_72hr_lalocheESnlay.csv", iday, iday);
        if ( (uafile = fopen(infilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", infilename);
        else
        {
printf ("Reading %s\n", infilename);
            fgets(buffer, 640, uafile);  // header
            fgets(buffer, 640, uafile);
            do
            {
                token = strtok(buffer, ",");
                lat = atof(token);  token=strtok(NULL, ",");     // latitude [degrees]
                lon = atof(token);  token=strtok(NULL, ",");    // longitude [degrees]
                if (lat0==0) // first record
                {
                    lat0=lat;
                    lon0=lon;
                }

                if (lat == lat0 && lon == lon0)
                    istn = 0;
                else
                    istn++;

                cmcdate = atoi(&token[0])*10000 + atoi(&token[5])*100 + atoi(&token[8]);
                cmcUTC = atoi(&token[11]);

                if (cmcdate >= cmcDATEo)
                {
                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo);
                    for (i=MAX_LEVELS-1; i>=0; i--)
                    {
                        token=strtok(NULL, ",");
                        uawx.Td[istn][cmcUTC][i] = uawx.T[istn][cmcUTC][i]-atof(token);     // dew-point temperature [oC]
                    }
                }
                fgets(buffer, 640, uafile);
            } while (!feof(uafile));
            fclose(uafile);
        }

/// Done reading upper air ///

/* 1.e. Open CMC GEM forecast CSV file as input.  Read in header line */
if (feps.print==-2) printf("1.e.\n");
        if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL)
            printf("Error: could not find %s\n\n", feps.csvfilename);
        else
        {
printf ("Reading %s\n", feps.csvfilename);

// read (and ignore) header info
            fgets(buffer, 640, csvfile);

            status = ReadHotspotHeader(&buffer, &fields);
// read in first line of data
            fgets(buffer, 640, csvfile);

/* 2. Begin reading CFFEPS predictions CSV file */
if (feps.print==-2) printf("2.\n");
            do  /* while !feof(csvfile); */
            {

/* 2.b. Read line of data into buffer.  Parse comma separated variables in buffer.
        Make adjustments for possible variation in units (e.g Kelvin instead of Celsius, etc.) */
if (feps.print==-2) printf("2.b.\n");

// read in first line of data
                status = ReadHotspotData(&buffer, &fields, &feps, &fbp, &ua);

                if (fbp.LAT > 56.3 && fbp.LAT < 56.56 && fbp.LON > -109.95 && fbp.LON < -109.65) // limit to Lac la Loche study area
                {

// set these back to internal variables
                    estarea = feps.estarea;
                    precip = feps.precip;
                    timezone = feps.timezone;

                    if (fbp.LAT != plat || fbp.LON != plon)
                    {

                        newfire = true;
                        plat = fbp.LAT;
                        plon = fbp.LON;
                        istn++;

                        x=0.; y=0.;
                        lat = fbp.LAT; lon = fbp.LON;
                        get_xy(&x, &y, &lat, &lon, profilename);

                        down = nrows - (int)((y - yllcorner) / cellsize);
                        across = (int)((x - xllcorner) / cellsize);
                        ignpt = ncols*(down-1)+across;

                        r2 = 10000.*estarea/3.1415926/cellsize/cellsize; /* radius^2 in gridcells */

                        detectionUTC = (int)(feps.dtime / 100) + (feps.dtime % 100) /60.;
                        detectionLST = (int)(feps.dtime / 100) + (feps.dtime % 100) /60. - feps.LDT + (int)timezone;

                        if (detectionLST < 0.)
                        {
                            detectionLST = detectionLST + 24.;
                            fbp.Dj = fbp.Dj;    // keep fbp.Dj as the UTC date
                        }

                        Do = -1; /* converted so Do value is returned (but not used in CFFEPS) -- 2019-07-29 */
                        fbp.FMC = FMCcalc(fbp.LAT, fbp.LON, feps.zs, fbp.Dj, &Do);

                        elapsed = (24.*fbp.Dj+detectionUTC) - (24.*cmcDjo+cmcUTCo);   // time [hrs] from model start time to satellite detection

                        ign = ign0;
                        for(jrow=0;jrow<nrows;jrow++)
                            for(icol=0;icol<ncols;icol++)
                            {
                                if ((jrow-down)*(jrow-down) + (icol-across)*(icol-across)<r2) *ign = elapsed;
                                ign++;
                            }
                        ign = ign0;
                    }
                    else
                        newfire = false;

if (feps.print==-2) printf("2.m.\n");

                }

// read in next line of data
                fgets(buffer, 640, csvfile);

            } while (!feof(csvfile) );

            fclose(csvfile);
        }  // if ( (csvfile = fopen(feps.csvfilename, "r")) == NULL)... else

// Starting ignition times -- note when writing this grid (and any others), the output CSV file is not written (memory problems?)
        strcpy(infilename, "ign_start.asc");
        status = write_double_ASCIIGRID(infilename, ign0);

/////////////////////////////////////
///   Part 2: Model fire growth   ///
/////////////////////////////////////
        fgmdirection=1; // forward
        fgmmethod=8; // 8-point fire growth
        fgmmethod=16; // 16-point fire growth
        ua.iLevels=MAX_LEVELS; // ReadHotspotData() overwrites this value

printf("\nConducting %d-point fire growth", fgmmethod);

		fire_propagate(ign0, FBPg0, slope0, aspect0, cbh0, cfl0, dfl0, pc0, sd0, sfl0, sh0, fgmdirection, fgmmethod, &feps, &fbp, &fgmwx);

        printf("Done\n\n");


///////////////////////////////
///   Part 3: Model smoke   ///
///////////////////////////////

/* Allocate memory for array */
        Zg = (double *)calloc(ncols*nrows, sizeof (double));
        Zg0 = Zg;

        for(jrow=0;jrow<nrows;jrow++)
            for(icol=0;icol<ncols;icol++)
            {
               *Zg = (double)NODATA_value;
               Zg++;
            }

        FBPg = FBPg0;
        ign = ign0;
        Zg = Zg0;

        feps.estarea = estarea = 6.25;
        strcpy(feps.source, "FGM");
        strcpy(feps.sensor, "NA");
        timestep = 1.; // one hour timesteps
        ntimesteps = (int)(MAX_HOURS/timestep);

        if ( (outfile = fopen(feps.outfilename, "w")) == NULL)
            printf("Error: could not write to %s\n\n", feps.outfilename);
        else
        {
printf ("Writing %s", feps.outfilename);

if (feps.print==-2) printf("1.f.\n");

            fprintf(outfile, "lat, lon, rep_date, source, sensor, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, tfc, bfc, hfi, estarea, ");
            fprintf(outfile, "UTC, temp, rh, ws, precip, TS, T850, T700, T500, T250, ZS, Z850, Z700, Z500, Z250, Area(t), Growth (t), TotalEmissions, r_smoke, Zplume, Mplume, Mw, Qo, Qplume");
            for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
            {
                fprintf(outfile, ", %s F", emissions.species[iSpecies]);
                fprintf(outfile, ", %s S", emissions.species[iSpecies]);
                fprintf(outfile, ", %s R", emissions.species[iSpecies]);
            }
            fprintf(outfile, "\n");

         for(jrow=0;jrow<nrows;jrow++)
            for(icol=0;icol<ncols;icol++)
            {
                status = FBPType(fbp.FuelType, *FBPg, fgmwx.greenup, fgmwx.standing);       // there is a chance of a hotspot with an assigned ignition time in a non-fuel
                if (*ign>0. && status >=0)
                {

if (feps.print==-2) printf("2.d.\n");
                    feps.area=0.; //=estarea;
                    totalemissions = 0.;  // zero everything including total emissions
                    feps.Qo = Qo = 0.;    // Qo is used to temporarily store the energy when comparing height predictions using different lapse rates
                    feps.Qplume = 0;
                    feps.growth=0.;
                    feps.Flaming=0.;
                    feps.Smoldering=0.;
                    feps.Residual=0.;
                    fbp.ROS = fbp.SFC = fbp.TFC = fbp.HFI = 0.;

/* elapsed is the time since detectionLST (dhrs) */
                    elapsed = 0.;
                    timestep = feps.timestep;
                    r_smoke = 0.;
                    feps.dz = 0;
                    feps.M = 0.;
                    feps.Mw = 0.;

                    for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                        for (itimestep=0; itimestep<ntimesteps; itimestep++)
                        {
                            Qs[itimestep]=0.;
                            Mws[itimestep]=0.;
                            emissions.f[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                            emissions.s[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                            emissions.r[iSpecies][itimestep]=0.;       // new fire, set concentrations back to zero
                            FCs[itimestep]=0.;
                            Fs[itimestep]=0.;       // new fire, set concentrations back to zero
                            Ss[itimestep]=0.;       // new fire, set concentrations back to zero
                            Rs[itimestep]=0.;       // new fire, set concentrations back to zero
                        }

                    y = yllcorner + (nrows - jrow) * cellsize;
                    x = xllcorner + i * cellsize;
                    lat = 0.; lon = 0.;
                    get_xy(&x, &y, &lat, &lon, profilename);
                    fbp.LAT = lat; fbp.LON = lon;

// determine closest upper air grid point
                    r2min=9999999999.;
                    for (iii = 0; iii< MAX_STNS; iii++)
                    {
                        r2 = sqrt(fgmwx.xs[iii]-x)*(fgmwx.xs[iii]-x)+(fgmwx.ys[iii]-y)*(fgmwx.ys[iii]-y);
                        if (r2<r2min && fgmwx.xs[iii]!=missing)
                        {
                            r2min=r2;
                            istn = iii;
                        }
                    }

                    detectionUTC = (cmcUTCo+*ign);  // can exceed 24.
                    while (detectionUTC> 24.)
                        detectionUTC = detectionUTC-24.;
                    fbp.Dj = cmcDjo + (int)(*ign/24.); // I suppose I could have a variable detectionDj

                    detectionLST = (cmcUTCo+*ign) - feps.LDT + (int)timezone;
                    while (detectionLST>24.)
                        detectionLST = detectionLST-24.;

                    if (detectionLST < 0.)
                    {
                        detectionLST = detectionLST + 24.;
                        fbp.Dj = fbp.Dj;    // keep fbp.Dj as the UTC date
                    }

                    sprintf(rep_date, "%d_%02d:%02d", cmcDATEo + (int)(detectionUTC/24.), (int)detectionUTC, mtime(detectionUTC)%100);//  (not fretting over the month changing)


///// The following taking from method="BIGFOOT" /////
                    for (itimestep=0.; itimestep<=ntimesteps; itimestep++)    // 72 hour forecast
                    {
                        dt = (double)itimestep/timestep;    // track the model time from the start
                        current = cmcUTCo + dt; // double of cmcUTM (integer) in hours
                        cmcUTC = (int)current;
                        cmcdate = cmcDATEo + (int)(current/24.); //  (not fretting over the month changing)
                        cmcDj = cmcDjo + (int)(current/24.);
                        while (current >= 24.)
                            current = current-24.;
                        feps.elapsed = elapsed = (24.*cmcDj+current) - (24.*fbp.Dj+detectionUTC);   // time [hrs] from detection to current timestep

                        feps.temp = fgmwx.Ts[istn][cmcUTC];
                        feps.rh = fgmwx.rhs[istn][cmcUTC];
                        precip = fgmwx.rains[istn][cmcUTC];

                        for (iii=0; iii<MAX_LEVELS; iii++)
                        {
                         	ua.P[iii] = uawx.P[istn][cmcUTC][iii];
                            ua.Z[iii] = uawx.Z[istn][cmcUTC][iii];
                            ua.T[iii] = uawx.T[istn][cmcUTC][iii];
                            ua.Td[iii] = uawx.Td[istn][cmcUTC][iii];
                        }

// these values become part of the output csv file (but not used here)
//                    cmcUTC = cmcUTC + 24*(cmcdate-cmcDATEo); ???
// as written, these manditory level are approximated based on hybrid values
                        feps.ts   = ua.T[0];   // 850mb temperature [oC]
                        feps.t850 = ua.T[9];   // 850mb temperature [oC]
                        feps.t700 = ua.T[14];  // 700mb temperature [oC]
                        feps.t500 = ua.T[21];  // 500mb temperature [oC]
                        feps.t250 = ua.T[31];  // 250mb temperature [oC]
                        feps.zs   = ua.Z[0];   // 850mb height [m]
                        feps.z850 = ua.Z[9];   // 850mb height [m]
                        feps.z700 = ua.Z[14];  // 700mb height [m]
                        feps.z500 = ua.Z[21];  // 500mb height [m]
                        feps.z250 = ua.Z[31];  // 250mb height [m]

// interpolate WS, WD, FFMC, DMC, DC
                        ext_status = interp_wx(current, x, y, missing, &fbp, &feps, &fgmwx);


                        if (elapsed > 0)
                        {
                            if (elapsed <= timestep)
                                feps.growth = estarea; // only burn the cell for one time step
                            else
                                feps.growth = 0.;
                            feps.area = feps.area+feps.growth;
/* 2.i. Interpolate weather conditions from fgmwx data collected from hotspots */

if (feps.print==-2) printf("2.i.\n");

                            fbp.t = 0.; // not interested in time related output from FBP

                            if (feps.elapsed < 0.) feps.elapsed = 0.; //  ???

// fbp.FC_method = 4;   /* was hard coded in, now include this in INI file 2020-02-19 */
                            status = FBPCalc (&fbp);    // FBP at rep_dat

                            if (status>=0)
                            {
                                dz8 = dz7 = dz5 = dz = -99.; // NOTE: when PlumeCalc fails, dz = -9999.
                                l8 = l7 = l5 = -999.;
                                r_smoke = 0.;
/* 2.l.iii. Calculate energy of the fire */
if (feps.print==-2) printf("2.l.iii.\n");
                                feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);

/* 2.l.iv. Calculate the energy released from flaming, smoldering and residual combustion.
            Calculate flaming, smoldering and residual emissions rates in tonnes/h  */
if (feps.print==-2) printf("2.l.iv.\n");
                                EmissionsCalc(&feps, &fbp, &emodels, &allocations);
                                    emissions.residence_flaming=0.25;   /* residence time for flaming combustion (hours) -- remains the same */
                                    emissions.residence_smoldering = feps.DOB_S/feps.SCR/2.; /* CSR = smoldering combustion rate [cm/hr] (Dan Thompson) */
                                    emissions.residence_residual = feps.DOB_S/feps.SCR/2.;   /* split equally between smoldering and residual */

if (feps.print==-1) printf("2.l.v. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", itimestep, MAX_HOURS);

/* 2.l.v. Distribute the energy over time */
if (feps.print==-2) printf("2.l.v.\n");
                                jModel = -1;    /* an admittedly awkward way of creating hourly energy components */
                                iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                                for (i=0; i< iMax; i++)
                                    if (i+itimestep <= ntimesteps)
                                        Qs[i+itimestep] = Qs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];

/* 2.l.v. Distribute the water released [kg/timestep] over time, required for the subroutine PlumeCalcWet -- 2019-07-17*/
if (feps.print==-2) printf("2.l.v.\n");
                                jModel = -2;    /* an admittedly awkward way of creating hourly water components */
                                iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                                for (i=0; i< iMax; i++)
                                    if (i+itimestep <= ntimesteps)
                                    {
                                        Mws[i+itimestep] = Mws[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                                        //printf("%d %lf ", i, Mws[i+itimestep]);
                                    }

/* 2.l.v. Distribute the fuel consumption over time */
if (feps.print==-2) printf("2.l.v.\n");
                                jModel = -3;    /* an admittedly awkward way of creating hourly energy components */
                                iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                                for (i=0; i< iMax; i++)
                                    if (i+itimestep <= ntimesteps)
                                        FCs[i+itimestep] = FCs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];

/* 2.l.vii. Find emodel that matches the fuel type */
if (feps.print==-2) printf("2.l.vii.\n");
                                if (emissions.nModels==0 || emodels.nModels==0)
                                    jModel=0;
                                else
                                    for (iFuel=0; iFuel<MAX_CWFIS; iFuel++)
                                        if (!strcasecmp(emodels.FuelType[iFuel], fbp.FuelType)) // found a match
                                            for (iModel=0;iModel<emissions.nModels;iModel++)
                                                if (emissions.emodel[iModel]==emodels.emodel[iFuel] )
                                                jModel=iModel;

/* 2.l.viii.  Distribute the emission over time */
if (feps.print==-2) printf("2.l.viii.\n");
                                for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                                {
                                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);
                                    for (i=0; i< iMax; i++)
                                        if (i+itimestep <= ntimesteps)
                                        {
                                            emissions.f[iSpecies][i+itimestep]=emissions.f[iSpecies][i+itimestep]+Fs[i];
                                            emissions.s[iSpecies][i+itimestep]=emissions.s[iSpecies][i+itimestep]+Ss[i];
                                            emissions.r[iSpecies][i+itimestep]=emissions.r[iSpecies][i+itimestep]+Rs[i];
                                        }
                                }
                            }   // if (status==0)
                        } // if (elapsed > 0.)

                        feps.Qplume = Qs[itimestep];  // need to check this (only care about the first hour?)
                        feps.Mw = feps.Mw + Mws[itimestep];       // Cumulative mass of water released [kg] from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/

/* Hard code in plume rise method for quick testing */
// strcpy(feps.type, "dry");
// strcpy(feps.type, "wet");
// strcpy(feps.type, "wetter");

// taken directly from method="CMC"
                        if (feps.Qplume > 1.) // ignore residual energy amount
                        {
                            if (!strcmp(feps.type, "dry"))
                            {
                                feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                dz = PlumeCalcDry(&feps, &ua); // new piece-wise approach
                            }

                            else if (!strcmp(feps.type, "wet"))
                            {
                                Mw = feps.Mw;
                                feps.Mw = 0.;   // setting feps.Mw to 0 turns off PlumeCalcWetter (returning to PlumeCalcWet)        /* KRA: 2019-09-25 */
                                feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                dz = PlumeCalcWet(&feps, &ua); // new piece-wise approach
                                feps.Mw = Mw;   // rewrite the value to feps.Mw

                            }
                            else if (!strcmp(feps.type, "wetter") ||        /* KRA: 2019-09-25 */
                                     !strcmp(feps.type, "lcl"))             /* KRA: 2020-11-09 */
                            {
                                feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                dz = PlumeCalcWet(&feps, &ua); // new piece-wise approach
                            }
/* 2.l.vi. (2) follow the standard average lapse rate method outlined in Anderson et al., 2011 */
                            else    // follow the standard average lapse rate method outlined in Anderson et al., 2011
                            {

/*** old approach -- a cumbersome sequence of layered if/else conditions.  Rewritten more succinctly below (this section removed) ***/

/* 2.l.vi. (2) (a) calculate height based on surface to 850 mb lapse */

/* 2.l.vi. (2) (b) if recalculated height greater than 2000 m,
                  recalculate height based on surface to 700 mb lapse rate
                  (if resulting height less than 2000, use average) */


/* New approach, rewritten, same idea -- 2019-07-17 */
                                if (feps.ts > -99. && feps.zs > -99. )  /* otherwise missing data */
                                {
                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l8 = feps.lapse = (feps.t850 - feps.ts) / (feps.z850 - feps.zs);
                                    if (feps.lapse > -0.0098 && feps.z850 > feps.zs && feps.z850 > 0.)
                                        dz8 = PlumeCalc(&feps);

                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l7 = feps.lapse = (feps.t700 - feps.ts) / (feps.z700 - feps.zs);
                                    if (feps.lapse > -0.0098 && feps.z700 > feps.zs && feps.z700 > 0.)
                                        dz7 = PlumeCalc(&feps);

                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l5 = feps.lapse = (feps.t500 - feps.ts) / (feps.z500 - feps.zs);
                                    if (feps.lapse > -0.0098 && feps.z500 > feps.zs && feps.z850 > 0.)
                                        dz5 = PlumeCalc(&feps);

                                    if (dz8 > 0. && l8 > -0.0098 ) dz = dz8;
                                    if (dz7 > 0. && l7 > -0.0098 && dz > 2000.) dz = dz7;
                                    if (dz5 > 0. && l5 > -0.0098 && dz > 4000.) dz = dz5;
                                }

                                if (dz < 0. && feps.z850 > 0.)  /* height still missing -- start at 850 mb */
                                {
                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l7 = feps.lapse = (feps.t700 - feps.t850) / (feps.z700 - feps.z850);
                                    if (feps.lapse > -0.0098 && feps.z700 > feps.zs && feps.z700 > 0.)
                                        dz7 = PlumeCalc(&feps);

                                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                                    l5 = feps.lapse = (feps.t500 - feps.t850) / (feps.z500 - feps.z850);
                                    if (feps.lapse > -0.0098 && feps.z500 > feps.zs && feps.z500 > 0.)
                                    dz5 = PlumeCalc(&feps);

                                    if (dz7 > 0. && l7 > -0.0098) dz = dz7;
                                    if (dz5 > 0. && l5 > -0.0098 && dz > 4000.) dz = dz5;
                                }
/* End of new approach */

                                if (dz8 > 2000. && dz7 < 2000.  && dz7 > 0.)
                                {
                                    dz = (dz8+dz7)/2;
                                    feps.lapse = (l8+l7)/2;
                                }

                                else if (dz7 > 4000. && dz5 < 4000. && dz5 > 0.)
                                {
                                    dz = (dz7+dz5)/2;
                                    feps.lapse = (l7+l5)/2;
                                }

                            }    // if (!strcmp(feps.type, "dry"))... else
                        }   //  if (feps.Qplume > 1.)

/* 2.l.vi. (3) limit height to 250 mb height */
                        if (dz > feps.z250)    dz = feps.z250;

/* 2.l.vi. (4) impose a minimum height */
                        if (dz < feps.dzmin) dz = feps.dzmin;

                        feps.Qo = Qo;  // remember the energy prior to entering decision tree (note that as is, this discounts the energy lost due to black body radiation)

                        feps.Qo = Qo = Qo + feps.Qplume;    // I used to do this in PlumeCalc but the new EmissionsCalc has thrown this off

                        feps.dz = dz;

                        if (feps.dz > *Zg) *Zg = dz; // record the maximum plume height to the grid

/* 2.l.ix. Calculate total emissions and r_smoke */
if (feps.print==-2) printf("2.l.ix.\n");

                        totalemissions = totalemissions + FCs[itimestep];

                        GridTotalEmissions[itimestep] = GridTotalEmissions[itimestep] + totalemissions;

                        if (feps.dz > 0 && feps.M > 0)
                            r_smoke = 1000 * (1000.* totalemissions) / feps.M;
                        else
                            r_smoke = 0.;

/* replace missing values (-9999) or any negative plume heights with 0. */
                        if (feps.dz < 0.)
                        {
                            feps.dz = 0.;
                            feps.M = 0.;
                        }

/* 2.m.  Print results to the screen. */
if (feps.print==-2) printf("2.m.\n");

// lat, lon, rep_date, ffmc, dmc, dc, ws, fwi, fuel, ros, sfc, tfc, bfc, hfi, estarea,
                        fprintf(outfile, "%.4lf, %.4lf, %s, %s, %s, %.1lf, %.1lf, %.1lf, %.1lf, %.1lf, %s, %lf, %lf, %lf, %lf, %.0lf, %.2lf,",
                            fbp.LAT, fbp.LON, rep_date, feps.source, feps.sensor, fbp.FFMC, feps.DMC, feps.DC, fbp.WS,
                            feps.FWI, fbp.FuelType, fbp.ROS, fbp.SFC, fbp.TFC, bfc, fbp.HFI, estarea);
//                        fbp.LAT, fbp.LON, rep_date, source, sensor, fbp.FFMC, feps.DMC, feps.DC, fbp.WS,
//                        fwi, fbp.FuelType, fbp.ROS, fbp.SFC, fbp.TFC, bfc, fbp.HFI, estarea);

if (feps.print==-2) printf("2.n.\n");
// e.g. 20160216_00:00
                        cmcUTC=mtime(current);
                        sprintf(cmc_date, "%d_%02d:%02d", cmcdate, (int)current, cmcUTC%100);

                        fprintf(outfile, " %s, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf, %.2lf,",
                                cmc_date, feps.temp, feps.rh, fbp.WS, precip, feps.ts, feps.t850, feps.t700, feps.t500, feps.t250, feps.zs, feps.z850, feps.z700, feps.z500, feps.z250);

                        fprintf(outfile, " %lf, %lf, %lf, %lf, %d, %lf, %lf", feps.area, feps.growth, totalemissions, r_smoke, (int)feps.dz, feps.M, feps.Mw/1000.);
                        fprintf(outfile, ", %lf, %lf", feps.Qo, feps.Qplume);

if (feps.print==-2) printf("2.o.\n");
                        for (iSpecies = 0; iSpecies < emissions.nSpecies; iSpecies++)
                        {
                            fprintf(outfile, ", %.4e", emissions.f[iSpecies][itimestep]);
                            fprintf(outfile, ", %.4e", emissions.s[iSpecies][itimestep]);
                            fprintf(outfile, ", %.4e", emissions.r[iSpecies][itimestep]);
                        }
                        fprintf(outfile, "\n");
                    }   // for (itimestep=0.; itimestep<=72; itimestep++)

if (feps.print==-2) printf("2.p. %d\n", itimestep);
                }  // if (*ign>0)

                FBPg++;
                ign++;
                Zg++;

            }   //   for(jrow=0;jrow<nrows;jrow++)... for(icol=0;icol<ncols;icol++)

            fclose(outfile);
        } //   if ( (outfile = fopen(feps.outfilename, "w")) == NULL).... else
            printf("\n");
// These write_double_ASCIIGRID calls have to happen outside the above loop -- can't figure out why (memory?)
        strcpy(infilename, "ign_end.asc");
        status = write_double_ASCIIGRID(infilename, ign0);
        strcpy(infilename, "Z.asc");
        status = write_double_ASCIIGRID(infilename, Zg0);

//        for (itimestep=0; itimestep<ntimesteps; itimestep++)
//            printf("%e\n", GridTotalEmissions[itimestep]);    // print the last hour's values of 72 hour forecast

/* quick and dirty skill score calculations * /
        Pg = (double *)calloc(ncols*nrows, sizeof (double));
        Pg0 = Pg;
        Og = (double *)calloc(ncols*nrows, sizeof (double));
        Og0 = Og;

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062300\\ign_start.asc");
        ign0 = read_double_ASCIIGRID(infilename);
        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062300\\ign_end.asc");
        Pg0 = read_double_ASCIIGRID(infilename);
        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062400\\ign_start.asc");
        Og0 = read_double_ASCIIGRID(infilename);

        ign=ign0;
        Pg=Pg0;
        Og=Og0;

        x=H=F=M=Z=BIAS=HR=FAR=CSI=0.;
        for(jrow=0;jrow<nrows;jrow++)
            for(icol=0;icol<ncols;icol++)
            {
				file1=0; file2=0;

				if (*ign != NODATA_value && *ign > 0.)
                    x++;
                else
                {
                    if (*Pg != NODATA_value && *Pg > 0.) file1=1;	// area burned by fire 1
                    if (*Og != NODATA_value && *Og > 0.) file2=1;	// ignition point for fire 2 for next day


                    if (file1==1 && file2==1) H++;
                    if (file1==1 && file2==0) F++;
                    if (file1==0 && file2==1) M++;
                    if (file1==0 && file2==0) Z++;
                }

				ign++;
                Pg++;
				Og++;
			}

		if (H+M > 0)
		{
			BIAS = (H+F)/(H+M);
			HR = H/(H+M);
		}
		else
		{
			BIAS = -1;
			HR = -1;
		}

		if (H+F > 0)
			FAR = F/(H+F);
		else
			FAR = -1;

		if (H+F+M > 0)
			CSI = H/(H+F+M);
		else
			CSI = -1;

        printf("\nJune 23\n%lf %lf %lf %lf %lf\n%lf %lf %lf %lf\n", H, F, M, Z, x, BIAS, HR, FAR, CSI);

        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062400\\ign_start.asc");
        ign0 = read_double_ASCIIGRID(infilename);
        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062400\\ign_end.asc");
        Pg0 = read_double_ASCIIGRID(infilename);
        strcpy(infilename, "D:\\projects\\src\\CFFEPS\\Lac_La_Loche\\2018062500\\ign_start.asc");
        Og0 = read_double_ASCIIGRID(infilename);

        ign=ign0;
        Pg=Pg0;
        Og=Og0;

        x=H=F=M=Z=BIAS=HR=FAR=CSI=0.;
        for(jrow=0;jrow<nrows;jrow++)
            for(icol=0;icol<ncols;icol++)
            {
				file1=0; file2=0;

				if (*ign != NODATA_value && *ign > 0.)
                    x++;
                else
                {
                    if (*Pg != NODATA_value && *Pg > 0.) file1=1;	// area burned by fire 1
                    if (*Og != NODATA_value && *Og > 0.) file2=1;	// ignition point for fire 2 for next day


                    if (file1==1 && file2==1) H++;
                    if (file1==1 && file2==0) F++;
                    if (file1==0 && file2==1) M++;
                    if (file1==0 && file2==0) Z++;
                }

				ign++;
                Pg++;
				Og++;
			}

		if (H+M > 0)
		{
			BIAS = (H+F)/(H+M);
			HR = H/(H+M);
		}
		else
		{
			BIAS = -1;
			HR = -1;
		}

		if (H+F > 0)
			FAR = F/(H+F);
		else
			FAR = -1;

		if (H+F+M > 0)
			CSI = H/(H+F+M);
		else
			CSI = -1;

        printf("\nJune 24\n%lf %lf %lf %lf %lf\n%lf %lf %lf %lf\n", H, F, M, Z, x, BIAS, HR, FAR, CSI);  */
    }   ///  FGM

    else if (!strcasecmp(feps.method, "FBP") )
    {
        FBP_C_test(&fbp);
    }

    else if (!strcasecmp(feps.method, "FWI") )  // a routine to allow for a quick calculation of FWI indices
    {

        infile = fopen("Perth.txt","r");
        fscanf(infile, "%d %lf %lf %lf %lf %lf", &imonth, &T, &WS, &Vis, &Td, &Precip);

        FFMC=85.;DMC =40; DC = 300; // starting values

        do
        {
            RH = 100. * e_w_Calc(Td)/e_w_Calc(T);

            FFMC = FFMCcalc(T,  RH,  WS,  Precip,  FFMC);
//                printf("%lf %lf %lf %lf %lf\n", T,  RH,  WS,  Precip,  FFMC);
            DMC = DMCcalc(T,  RH,  Precip,  DMC,  imonth);
            DC = DCcalc(T,  Precip,  DC,  imonth);
            ISI = ISIcalc(FFMC,  WS);
            BUI = BUIcalc(DMC,  DC);
            FWI = FWIcalc(ISI,  BUI);

            printf("%d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf\n", imonth, T, Td, RH, WS, Vis, Precip, FFMC, DMC, DC, ISI, BUI, FWI);
            fscanf(infile, "%d %lf %lf %lf %lf %lf", &imonth, &T, &WS, &Vis, &Td, &Precip);
        } while (!feof(infile));
        fclose(infile);
    }
    else if (!strcasecmp(feps.method, "EMC") )     // used to test the dirunal calculations -- 2023-03-20
    {
        infile = fopen("average.txt","r");
        fscanf(infile, "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", rep_date, &T, &RH, &WS, &FFMC, &DMC, &DC, &GSI, &area, &growth, &totalemissions);
        do
        {
            printf ( "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf ", rep_date, T, RH, WS, FFMC, DMC, DC, GSI, area, growth, totalemissions);

            strcpy(fbp.FuelType, "O1B");
            fbp.FMC = FMCcalc(fbp.LAT, fbp.LON, feps.zs, fbp.Dj, &Do);
            fbp.FFMC = eq_ffmc(T, RH, WS, zero, zero);
            fbp.FFMC = FFMC;
            fbp.DMC = DMC;
            fbp.DC = DC;
            fbp.WS = WS;
            fbp.C = 100.*(1.-GSI);
            fbp.BUI = BUIcalc(DMC,  DC);
            FWI = FWIcalc(ISI,  BUI);

            fbp.t = detectionLST;

            status = FBPCalc (&fbp);    // FBP at detectionLST
            printf("%lf %lf %lf %lf %lf ", fbp.FFMC, fbp.ROS, fbp.CFB, fbp.HFI, fbp.TFC);

            if (growth == 0) t1 = 0.; // a new fire
            else (t1 = t2);

            t2 = t1+3;
            A=0.; P=0.; D=0;    // line 775ish
            PROCalc(fbp.FuelType, fbp.Accel,
                    fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
                    &t1, &t2, &A, &P, &D);
                    feps.growth = A;   //
            printf("%lf\n", A);


            fscanf(infile, "%s %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", rep_date, &T, &RH, &WS, &FFMC, &DMC, &DC, &GSI, &area, &growth, &totalemissions);
        } while (!feof(infile));
        fclose(infile);




    }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    else    // Tabular & Alberta Plume Rise Study -- the old code (removed)

    {
/*******************************************************************************************************************/
        feps.header = 1;// 2023-08-28
feps.print = 0		;
//        if (strcasecmp(feps.method, "AB") && feps.print>=0 && feps.header>=0)
        if (strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR") && feps.header>=0)
        {
            printf ("-------------------------------------------------------------------------------\n");
            if (feps.print > 0)
            {
                printf ("Elpsd Qfire     Qw        Qf        Qr        Qs        Qinc      Qplume    Qt        Qbb       dz\n");

            }
            else if (feps.print < 0)
            {
                printf ("Elpsd Hour  Area  Flamg  Smold  Resid    PM  PM10 PM2.5    CO   CO2   CH4  NMHC\n");
			if (feps.LDT)
                printf ("Time  LDT    ha   kg/m^2 kg/m^2 kg/m^2    t     t     t     t     t     t     t\n");
			else
                printf ("Time  LST    ha   kg/m^2 kg/m^2 kg/m^2    t     t     t     t     t     t     t\n");
            }
            else
            {
                printf (" Elps Hour FFMC HROS   HFI  FROS  FFI  BROS  BFI CFB Dist Growth    Area Plume\n");
			if (feps.LDT)
                printf (" Time LDT      m/min  kW/m m/min kW/m m/min kW/m   %%   km     ha      ha     m\n");
			else
                printf (" Time LST      m/min  kW/m m/min kW/m m/min kW/m   %%   km     ha      ha     m\n");
            }
            printf ("-------------------------------------------------------------------------------\n");
        }

    csvfile= NULL;
    iMax=24;

    iii = 0;
    Ft = St = Rt = 0.;
    timezone  = feps.timezone;
    feps.Qo = 0.;
    if (feps.maxarea <=0) feps.maxarea = 9999999999.;

    totalemissions = 0.;  // zero everything including total emissions
    feps.Qo = Qo = 0.;    // Qo is used to temporarily store the energy when comparing height predictions using different lapse rates
    feps.Qplume = 0.;
    feps.Mw = 0.;   // Mass of water released from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/

    for (itimestep=0; itimestep<ntimesteps; itimestep++)
    {
        Qs[itimestep]=0.;
        Mws[itimestep]=0.;
    }

	/* Determine julian date */
    if (fbp.Dj = -1)
    {
        if (feps.ddate > 9999)
            iyear = (int)(feps.ddate/10000);
        else
            iyear = 0;
        imonth = (int)((feps.ddate-10000*iyear)/100);
        iday = feps.ddate-10000*iyear-100*imonth;
        fbp.Dj = jdays[imonth-1]+iday;
        if (iyear > 0 && iyear%4 == 0 && imonth >2) fbp.Dj+=1;
    }
/* recalculate BUI value from DMC and DC */
    fbp.BUI = BUIcalc(feps.DMC, feps.DC);

/* Determine whether BUI effect is being used.  If BUI is zero or less, BUIeff = False */
    if (fbp.BUI <= 0) fbp.BUIEff = 0;

/* Confirm time is a LDT value */
    if (feps.LDT == 100 || feps.LDT == 1)
        feps.LDT = 1;
    else
        feps.LDT = 0;

/* detectionLST is the time of the fire detection (associated with the fire size) [decimal hours] */
      if (csvfile != NULL)	// CSV file reports detectionLST time in UTC -- need to adjust this to LST
	  {
            detectionLST = (int)(feps.dtime / 100) + (feps.dtime % 100) /60. - feps.LDT + timezone;
            detectionUTC = (int)(feps.dtime / 100) + (feps.dtime % 100) /60.;
            if (detectionLST < 0.)
			{
				detectionLST = detectionLST + 24.;
				fbp.Dj = fbp.Dj-1;	// fbp.Dj is now the UTC date
			}
	  }
	  else	// otherwise detectionLST time is in local time (LST)  [decimal hours] */
	  {
            detectionLST = (int)(feps.dtime / 100) + (feps.dtime % 100) /60. - feps.LDT ;
            detectionUTC = (int)(feps.dtime / 100) + (feps.dtime % 100) /60.  - feps.LDT - timezone;
            if (detectionUTC < 0.)
			{
				detectionUTC = detectionUTC + 24.;
				if (profile_date > 0)
					fbp.Dj = fbp.Dj-1;	// fbp.Dj is now the UTC date
			}
	  }
      detectionsize = feps.area;

/* elapsed is the time since detectionLST  [decimal hours]  */
    elapsed = 0.;firsttime = 0;  // 0 = TRUE
    timestep = feps.timestep;

/* if the detectionLST time is before noon LST assume FFMC is forecasted.
   It is therefore necessary to converge on yesterday's value, FFMCy,
   which is what BDL uses */
    FFMCy = FFMC = fbp.FFMC; 	/* we are assuming the value passed in FBP is the noon FFMC */

 /* Calculate yesterday's FFMC */
//    if (detectionLST < 12.)
    {
        FFMCy = FFMC; FFMCt=0.;	/* first guess */
        FFMCnoon = bdl_ffmc(FFMC, 1200, (int)feps.rh);
        while (fabs(FFMCnoon-FFMCt) > .1 && FFMCy > 50. && FFMCy < 100.)
        {
            FFMCt = bdl_ffmc(FFMCy, 1159, (int)feps.rh);
            FFMCy = FFMCy + (FFMCnoon-FFMCt);
        }
        if (FFMCy>99.) FFMCy=99.;
    }
    if (FFMCy < 60. && strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR"))
    {
        printf("Low FFMC: %lf set to 75.\n", FFMCy);
        FFMCy = 75.;
    }
    hour = mtime(detectionLST);

    if (hour < 1200)
        fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
    else
        fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);
    fbp.t = detectionLST;

 	status = FBPCalc (&fbp);

/* discovery is the time from discoveryLST to detectionLST [decimal hours] */
    t1=0.; t2=0.; A=feps.area; P=0.; D=0.;      // A(t)
    PROCalc(fbp.FuelType, fbp.Accel,
                fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
                &t1, &t2, &A, &P, &D);
    dist = D;                               // used to recalculate discovery

/* recalculate discovery based upon forward spread distance using elliptical growth */
/* (necessary because of diurnal variation of ROS) */
    t1=0.; t2=0.; A=0.; P=0.; D=dist;
    PROCalc(fbp.FuelType, fbp.Accel,
            fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
            &t1, &t2, &A, &P, &D);
    discovery = t2;

    nextfdist = nextbdist = 0.;
    if (fbp.ROS > 0) nextfdist = dist * fbp.FROS/fbp.ROS; /* flank discovery distance */
    if (fbp.ROS > 0) nextbdist = dist * fbp.BROS/fbp.ROS; /* back discovery distance */

/* for the simple fire line, we reject all the previous calculations
   and simply assume the fire starts from a continuous line
   (all the discovery time calculations becomes irrelevant) */
    if (!strncmp(feps.shape, "line", 3))
    {
        t1=0.; t2=0.; discovery = 0.;
        P = diameter = 2*sqrt(10000.*feps.area/pi)/1000.;
        D = dist = 0;
        A = area = feps.area; // use the reported fire size
    }

    iLevels=0;
    if (strlen(feps.profilename)>0)
    {
        strcpy(profilename, feps.profilename);
//        if (!strcasecmp(feps.method, "AB")) sprintf(profilename, "Soundings//%d//%s", iyear, feps.profilename);
        iLevels = Readupperair(profilename, &feps, &ua);
    }

/* observationLST is the time of plume observation (dhrs) */
        if (feps.obs < 0) feps.obs = feps.dtime;    // if missing, set observation time to detection time

        observationLST = (int)(feps.obs / 100) + (feps.obs % 100) /60. - feps.LDT;
        if (observationLST < 0.) observationLST = observationLST + 24.;

        current = detectionLST;     // current time of day in FGM
        hour = mtime(current);

        if (current < 6.) current = 6.; // set current time to 6:00 AM -- not too sure what's going to happen

// printf("Start: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n", hour, elapsed, fbp.FFMC, fbp.ROS, D, nextdist, nextfdist, nextbdist, A, discoveryLST);
        if (feps.print > 0) printf("Start: %d %lf %lf %lf %lf %lf\n", hour, D, fbp.ROS, nextfdist, nextbdist, A);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

        do  /*** Reverse fire-growth modelling to determine ingitionLST, discovery (time) and discoverysize ***/
        {

            current = current-feps.timestep;
            hour = mtime(current);

            if (hour < 1200)
                fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh);
            else
                fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);

            fbp.t = discovery-feps.timestep;
            if (fbp.t<0) fbp.t=0.;

            status = FBPCalc (&fbp);

/* this will lead to discoverydist used as the starting point for the forward FGM */
            nextdist = dist - fbp.ROS*feps.timestep*60./1000.;

            if (fbp.ROS > 0) nextfdist -= (dist-nextdist) * fbp.FROS/fbp.ROS; /* flank distance covered for next time step */
            if (fbp.ROS > 0) nextbdist -= (dist-nextdist) * fbp.BROS/fbp.ROS; /* back distance covered for next time step */

            if (nextdist <= 0.)
            {
                t1=0.; t2=0.; P=0.; D=0.;   // use last A
                PROCalc(fbp.FuelType, fbp.Accel,
                    fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
                    &t1, &t2, &A, &P, &D);
                current = current + feps.timestep - t2;
                timestep = t2;  // in this case we adjust the first time step to a partial time step in the forward fire growth modelling
                hour = mtime(current);
                A=0; P=0.; D=0;
                dist = nextdist = nextfdist = nextbdist = 0.;
            }
            else
            {
                t1=0.; t2=0.; A=0; P=0.; D = nextdist;  // use D
                PROCalc(fbp.FuelType, fbp.Accel,
                        fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
                        &t1, &t2, &A, &P, &D);
                timestep = feps.timestep;  // establish time step as the feps value (expect for the above condition, which will likely happen on the last iteration)
				a = 1000.*(nextdist + nextbdist)/2; b = 1000.*nextfdist;	// major and minor axes in metres
				A = pi*a*b /10000.;						// total area [ha]
				P = 2.*pi*sqrt(0.5*(a*a+b*b))/1000.; 	// perimeter [km]
                dist = nextdist;
            }

            if (feps.print > 0) printf("Reverse: %d %lf %lf %lf %lf %lf %lf %lf %lf\n", hour, elapsed, fbp.ROS, D, nextdist, nextfdist, nextbdist, A, discoveryLST);
// printf("Reverse: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n", hour, elapsed, fbp.FFMC, fbp.HROSt, D, nextdist, nextfdist, nextbdist, A, discoveryLST);
        } while (current > 6. && A > 0);    // the ignition time or 6:00 AM

        hour = mtime(current);
        discoverysize = A;      // the size at time of ignition (zero) or 6:00 AM
        discoverydist = dist;      // the forward spread distance at time of ignition (zero) or 6:00 AM
        discoveryLST = current;    // time of ignition (LST) or 6:00 AM
        discovery=detectionLST-discoveryLST; // time from ignition or 6:00 AM to detection time
        discoveryLST1=discoveryLST;


/* it is possible reverse fire growth does not reach the observation time,
   as a result, the fire is assumed to burn atleast one time step */
        if (discoveryLST > observationLST - feps.timestep)
        {
//            printf("\tWARNING: plume observation time %d precedes discovery time %d based on detection time %d\n and size %lf\n\n",
//            mtime(observationLST), mtime(discoveryLST), mtime(detectionLST), feps.area);
            discovery = feps.timestep;
            discoverysize = 0.;
            discoverydist = 0.;
            discoveryLST = current = observationLST-discovery;
//            iLevels = 0; /* cancel these runs */
        }

        firsttime = 0;  // 0 = TRUE
        feps.Qo = 0.;
        itimestep=0;
        elapsed = 0.;
        current = discoveryLST;

        Qs8=0.; Qs7=0.; Qs5=0.; Q87=0.; Q85=0.; Qfire=0.;
        Qd=Qw=Qr=0.;

        areas = areas0; *areas = discoverysize; // this is an array of fire sizes from discoveryLST
        ts = ts0; *ts = discovery;
        dist = discoverydist;
        feps.growth = 0.;

// printf("\nDiscovery: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n\n", hour, elapsed, fbp.FFMC, fbp.HROSt, D, nextdist, nextfdist, nextbdist, A, discoveryLST);

//        if (strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR") && feps.print == 0) // && current >= observationLST)
        {
            hour = mtime(current);
            printf ("%5.2lf %04i %4.1lf %4.1lf%6.0lf %5.1lf%5.0lf %5.1lf%5.0lf %3.0lf %4.1lf %6.1lf %7.1lf %6d\n",
                    elapsed, hour+100*feps.LDT, fbp.FFMC, fbp.HROSt, fbp.HFI, fbp.FROSt,
                    fbp.FFI, fbp.BROSt, fbp.BFI, 100.* fbp.CFB,
                    dist, dailygrowth, *areas, (int)feps.dz); //                            dist, feps.growth, feps.area, (int)feps.dz); */
        }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

        hour = mtime(current+timestep-feps.timestep);   // 900

        do /*** forward fire-growth modelling ***/////////////////////////////////////////////
        {
            feps.lapse =  lapse;  /* refresh the lapse rate each time step */

            if (hour < 1200)
                fbp.FFMC = bdl_ffmc(FFMCy, hour, (int)feps.rh); // 900
            else
                fbp.FFMC = bdl_ffmc(FFMC, hour, (int)feps.rh);

            fbp.t = discovery+elapsed; // 0

            status = FBPCalc (&fbp);    // calculate ROS(elapsed), etc
            {
                t1=0.; t2=0.; A=0; P=0.; D=dist;  // use D to recalculate A(elapsed)
                PROCalc(fbp.FuelType, fbp.Accel,
                        fbp.CFB, fbp.ROS, fbp.FROS, fbp.BROS,
                        &t1, &t2, &A, &P, &D);

                nextdist = dist + fbp.ROS*timestep*60./1000.;
            }

            if (fbp.ROS > 0) nextfdist += (nextdist - dist) * fbp.FROS/fbp.ROS; /* flank distance covered for next time step */
            if (fbp.ROS > 0) nextbdist += (nextdist - dist) * fbp.BROS/fbp.ROS; /* back distance covered for next time step */

/* new equations to avoid effects of changing wind speed on LB and thus area growth eqns 4.22, 4.23 in Schaum's Mathematical Handbook */
            a = 1000.*(nextdist + nextbdist)/2; b = 1000.*nextfdist;	// major and minor axes in metres
            A = pi*a*b /10000.;						// total area [ha]
            P = 2.*pi*sqrt(0.5*(a*a+b*b))/1000.; 	// perimeter [km]


// printf("Forward: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n", hour, elapsed, fbp.FFMC, fbp.HROSt, D, nextdist, nextfdist, nextbdist, A, discoveryLST);

            dist = nextdist;
            elapsed += timestep;
            itimestep++;

            current = discoveryLST+elapsed;
            hour = mtime(current);
            timestep = feps.timestep;   // restore the feps default timestep (after the first iteration to capture any partial first timestep)

            if (A > feps.maxarea && feps.maxarea>0.) A = feps.maxarea;    // limit growth to the maximum size in the fire reports [2017-05-11]

            dist = nextdist;            // forward spread distance [km]
            feps.perimeter = P;         // perimeter [km]
            feps.growth = A - *areas;   // growth per time step

            areas++; ts++;
            *areas = A; *ts = elapsed;

            if (discoveryLST<= 6.)
                feps.area = dailygrowth = A - discoverysize;  // plume rise uses feps.area so we are limiting it to the daily growth
            else
                feps.area = dailygrowth = A;  // use total daily area growth (reset from anything else)

            if (timestep>0)
                hourlygrowth = feps.growth/timestep;
            else
                hourlygrowth = -999;

/* Plume rise */

// printf("Plume rise: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n", hour, elapsed, fbp.FFMC, fbp.HROSt, D, nextdist, nextfdist, nextbdist, A, discoveryLST);

/* 2.l.v. Distribute the water released [kg/timestep] over time, required for the subroutine PlumeCalcWet -- 2019-07-17*/
//if (feps.print==-2) printf("2.l.v.\n");
                    jModel = -2;    /* an admittedly awkward way of creating hourly water components */
                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                    for (i=0; i< iMax; i++)
                        if (i+itimestep <= ntimesteps)
                        {
                            Mws[i+itimestep] = Mws[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                        }
                        else
                            if (feps.print==-1) printf("2.l.v. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", itimestep, MAX_HOURS);


/* 2.l.v. Distribute the energy over time * /
//if (feps.print==-2) printf("2.l.v.\n");
                    jModel = -1;    /* an admittedly awkward way of creating hourly energy components */
                    iMax = EmissionsOverTime(Fs, Ss, Rs, jModel, iSpecies, &emissions, &feps);

                    for (i=0; i< iMax; i++)
                        if (itimestep < ntimesteps)
                        {
                            Qs[i+itimestep] = Qs[i+itimestep] + Fs[i]+Ss[i]+Rs[i];
                        }
                        else
                            if (feps.print==-1) printf("2.l.v. Warning: i+cmcUTC-cmcUTCo (%d) . MAX_HOURS (%d)\n", itimestep, MAX_HOURS);
/* 2.l.vi. From the area growth and energy released, calculate the plume rise */
//if (feps.print==-2) printf("2.l.vi.\n");

                    feps.Qplume = Qs[itimestep];  // need to check this
                    feps.Qo = Qo;   // remember the energy prior to entering decision tree
                    feps.Mw = feps.Mw + Mws[itimestep];       // Cumulative mass of water released from fuel moisture and due to combustion, required for the subroutine PlumeCalcWet -- 2019-07-17*/

            Qo = feps.Qo;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            Qfire = Qfire+feps.Qfire;
            EmissionsCalc(&feps, &fbp, &emodels, &allocations);
            dH = -999.;   //  = BriggsCalc(&feps, &fbp);

            if (feps.area < 0) feps.area = 0.;  // I have seen some negative values due to steps in BDL's FFMC calculations
			dzdry = dzwet = dzwetter = -999.;

            feps.Qo = Qd;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            dzdry = PlumeCalcDry(&feps, &ua);
            Qd = feps.Qo;

            feps.Qo = Qw;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            dzwetter = PlumeCalcWet(&feps, &ua);
            Qw = feps.Qo;

            feps.Mw=0.;
            feps.Qo = Qr;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            dzwet = PlumeCalcWet(&feps, &ua);
            Qr = feps.Qo;

            feps.Qo = Qs8;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            feps.lapse = Les8 = (feps.t850 - feps.ts)/(feps.z850 - feps.zs);
            dzs8 = PlumeCalc(&feps);
            Qs8 = feps.Qo;

            feps.Qo = Qs7;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            feps.lapse = Les7 = (feps.t700 - feps.ts)/(feps.z700 - feps.zs);
            dzs7 = PlumeCalc(&feps);
            Qs7 = feps.Qo;

            feps.Qo = Qs5;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);

            feps.lapse = Les5 = (feps.t500 - feps.ts)/(feps.z500 - feps.zs);
            dzs5 = PlumeCalc(&feps);
            Qs5 = feps.Qo;

            feps.Qo = Q87;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            feps.lapse = Le87 = (feps.t700 - feps.t850)/(feps.z700 - feps.z850);
            dz87 = PlumeCalc(&feps);
            Q87 = feps.Qo;

            feps.Qo = Q85;
            feps.Qplume = EnergyCalc(&feps, &fbp, &emodels);
            feps.lapse = Le85 = (feps.t500 - feps.t850)/(feps.z500 - feps.z850);
            dz85 = PlumeCalc(&feps);
            Q85 = feps.Qo;

// printf("%lf, %lf, %lf, %lf, %lf, %lf\n", Les8, Les7, Les5, dzs8, dzs7, dzs5);

            dzb8 = dz87;
            if ((dzb8<0. || dzb8> 4000.) && dz85>0.) dzb8 = dz85;

            dzbs = dzs8;
            if ((dzbs<0. || dzbs> 2000.) && dzs7>0.) dzbs = dzs7;
            if ((dzbs<0. || dzbs> 4000.) && dzs5>0.) dzbs = dzs5;

            if (feps.print > 0) printf(" %lf %.4e\n\n", dzbs, feps.Qo);

            feps.dz = dzbs;
            if (!strcmp(feps.type, "dry")) feps.dz = dzdry;
            if (!strcmp(feps.type, "wet")) feps.dz = dzwet;
            if (!strcmp(feps.type, "wetter")) feps.dz = dzwetter;
// printf("%lf %lf %lf %lf %lf %lf\n", feps.t850, feps.t700, feps.t500, feps.z850, feps.z700, feps.z500);
            if (strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR") && feps.print == 0) // && current >= observationLST)
            {

                    hour = mtime(current);
                    printf ("%5.2lf %04i %4.1lf %4.1lf%6.0lf %5.1lf%5.0lf %5.1lf%5.0lf %3.0lf %4.1lf %6.1lf %7.1lf %6d\n",
                            elapsed, hour+100*feps.LDT, fbp.FFMC, fbp.HROSt, fbp.HFI, fbp.FROSt,
                            fbp.FFI, fbp.BROSt, fbp.BFI, 100.* fbp.CFB,
                            dist, dailygrowth, *areas, (int)feps.dz);
            }


        }
        while ( ((!strcasecmp(feps.method, "AB") || !strcasecmp(feps.method, "MISR")) && (current +.001 < observationLST) && A < feps.maxarea )
              || (strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR") && (current + timestep <= 24.)) );

// printf("Done: %d %lf %lf HROS = %lf D = %lf %lf %lf %lf A = %lf %lf\n", hour, elapsed, fbp.FFMC, fbp.HROSt, D, nextdist, nextfdist, nextbdist, A, discoveryLST);

        if (strcasecmp(feps.method, "AB") && strcasecmp(feps.method, "MISR") && feps.print>=0 && feps.header>=0)
        {
            printf ("-------------------------------------------------------------------------------\n");
            if (!strncmp(feps.shape, "line", 3) )
                printf("   Fire growth is based on forward spread from a line equal to the perimeter length\n");
            else
                printf("   Fire growth is based on elliptical growth from the initial fire size\n");
            if (discoveryLST1 > observationLST)
                printf("   WARNING: plume observation time %d precedes discovery time %d based on detection time %d\n and size %lf\n\n",
                       mtime(observationLST)+100*feps.LDT, mtime(discoveryLST1)+100*feps.LDT, mtime(detectionLST)+100*feps.LDT, detectionsize);
				printf("   Plume rise follows %s lapse-rate technique\n", feps.type);
        }

        if (!strcasecmp(feps.method, "AB") && Qfire > 0 && iLevels>0)
        {
            printf ("%s, ", feps.ID);
            printf ("%d, ", feps.ddate);
            printf ("%s, ", fbp.FuelType);
            printf("%lf, ", feps.area);           // area [ha]
            printf("%lf, ", feps.growth);         // area [ha]
            printf("%lf, ", fbp.ROS);             // Rate of Spread [m/min]
            printf("%lf, ", fbp.CFB);             // Crown Fraction Burned
            printf("%lf, ", fbp.HFI);             // Head Fire Intensity [kW/m]
            printf("%lf, ", fbp.SFC);             // Surface Fuel Consumption [kg/m^2]
            printf("%lf, ", fbp.TFC);             // Total Fuel Consumption [kg/m^2]
            printf("%.4e, ", Qo);

            printf("%lf, %lf, ", Les8*1000., dzs8);
            printf("%lf, %lf, ", Les7*1000., dzs7);
            printf("%lf, %lf, ", Les5*1000., dzs5);
            printf("%lf, %lf, ", Le87*1000., dz87);
            printf("%lf, %lf, ", Le85*1000., dz85);
            printf("%lf, %lf, ", dzbs, dzb8);
            printf("%lf\n", dzdry);
//            printf("%lf\n", dzwet);
//            printf("%lf\n", dzwetter);
        }   /*     if (!strcasecmp(feps.method, "AB") )  */

        if (!strcasecmp(feps.method, "MISR") && Qfire > 0 && iLevels>0)
        {
            printf ("%s, ", feps.ID);
            printf ("%d, ", feps.ddate);
//          printf ("%lf, ", current);
            printf ("%s, ", fbp.FuelType);
//          printf("%lf, ", feps.area);           // area [ha]
//          printf("%lf, ", feps.growth);         // area [ha]
            printf("%lf, ", dailygrowth);           // area [ha]
            printf("%lf, ", hourlygrowth);         // area [ha]
            printf("%lf, ", fbp.ROS);             // Rate of Spread [m/min]
            printf("%lf, ", fbp.CFB);             // Crown Fraction Burned
            printf("%lf, ", fbp.HFI);             // Head Fire Intensity [kW/m]
            printf("%lf, ", fbp.SFC);             // Surface Fuel Consumption [kg/m^2]
            printf("%lf, ", fbp.TFC);             // Total Fuel Consumption [kg/m^2]
//          printf("%.4e, ", Qo);
            printf("%.4e, ", Qfire);            // total energy released by fire
//          printf("%lf, ", feps.Qplume/Qfire);            // total energy released by fire
            printf("%lf, %lf, ", Les8*1000., dzs8);
            printf("%lf, %lf, ", Les7*1000., dzs7);
            printf("%lf, %lf, ", Les5*1000., dzs5);
            printf("%lf, %lf, ", Le87*1000., dz87);
            printf("%lf, %lf, ", Le85*1000., dz85);
            printf("%lf, %lf, ", dzbs, dzb8);
            printf("%lf, ", dzdry);
            printf("%lf, %lf, %lf\n", feps.FRP, feps.Primary, feps.Secondary);
        }   /*     if (!strcasecmp(feps.method, "MISR") )  */
/*******************************************************************************************************************/
    }// 	else    // the old code



    return 0;
}

