#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>

#include "diurnal.h"
#include "thermo.h"
#include "fbp_2021.h"
#include "fwi84.h"
#include "feps_2011.h"
//#include "fop93.h"


int FBPCalc();
void fire_propagate();
double burn_cell();
double idw();


/*----------------------------*/
int FBPType(char *FBPstr, int FBPg, int greenup, int standing)
{
int i, status, PC;

/*
# fuel codes have been adopted from Burn-P3 ------
# 1 <- C-1
# 2 <- C-2
# 3 <- C-3
# 4 <- C-4
# 5 <- C-5
# 7 <- C-7
# 11 <- D-1
# 12 <- D-2
# 13 <- D-1/D-2
# 31 <- O-1
# 101 <- NonFuel
# 102 <- Water
# 106 <- Urban
# 425 <- M-1 C25%
# 525 <- M-2 C25%
# 635 <- M-1/M-2 C35%
# 650 <- M-1/M-2 C50%
# 665 <- M-1/M-2 C65%
*/
    strcpy(FBPstr, "NF");
    PC=0.;

    if (FBPg == 1) strcpy(FBPstr, "C1");
    if (FBPg == 2) strcpy(FBPstr, "C2");
    if (FBPg == 3) strcpy(FBPstr, "C3");
    if (FBPg == 4) strcpy(FBPstr, "C4");
    if (FBPg == 5) strcpy(FBPstr, "C5");
    if (FBPg == 7) strcpy(FBPstr, "C7");
    if (FBPg == 11) strcpy(FBPstr, "D1");
    if (FBPg == 12) strcpy(FBPstr, "D2");
    if (FBPg == 13) strcpy(FBPstr, "D1");
    if (FBPg == 31) strcpy(FBPstr, "O1a");
    if (FBPg == 101) strcpy(FBPstr, "NF");
    if (FBPg == 102) strcpy(FBPstr, "WA");
    if (FBPg == 106) strcpy(FBPstr, "NF");
    if (FBPg == 425) { strcpy(FBPstr, "M1"); PC = 25; }
    if (FBPg == 525) { strcpy(FBPstr, "M2"); PC = 25; }
    if (FBPg == 635) { strcpy(FBPstr, "M1"); PC = 35; }
    if (FBPg == 650) { strcpy(FBPstr, "M1"); PC = 50; }
    if (FBPg == 665) { strcpy(FBPstr, "M1"); PC = 65; }

/* Adjust fuels for greenup and standing */
   if (greenup >=0)
   {
      if (greenup == 1 && !strcasecmp(FBPstr, "M1")) strcpy(FBPstr, "M2");
      if (greenup == 0 && !strcasecmp(FBPstr, "M2")) strcpy(FBPstr, "M1");
      if (greenup == 1 && !strcasecmp(FBPstr, "M3")) strcpy(FBPstr, "M4");
      if (greenup == 0 && !strcasecmp(FBPstr, "M4")) strcpy(FBPstr, "M3");

   }

   if (standing >=0)
   {
      if (standing == 1 && !strcasecmp(FBPstr, "O1a")) strcpy(FBPstr, "O1b");
      if (standing == 0 && !strcasecmp(FBPstr, "O1b")) strcpy(FBPstr, "O1a");
   }

/* New addition */
   if (!strcasecmp(FBPstr, "WA") || !strcasecmp(FBPstr, "NF")) status = -1;
   else status = 0;

   if (PC > 0) status = PC;  // okay this is cluggy but good enough for now

   return (status);
}

/*----------------------------*/
double idw(double values[MAX_STNS], double xs[MAX_STNS], double ys[MAX_STNS],
     int istations, double missing_value, double x, double y, double power)
{
double value, distance, sumvalues, sumdistances, min;
int i, iWxStn;

   value = missing_value;
   sumvalues = 0.;
   sumdistances = 0.;
   min = 99999999.;
   if (istations > MAX_STNS) istations = MAX_STNS;

   if (istations==1)
	   value = values[0];
   else
		for (i=0;i<istations;i++)
		{
			if (values[i] != missing_value &&
	            xs[i] != missing_value &&
		        ys[i] != missing_value)
			{
                if (power <=0)    // nearest neighbour
                {
                    distance = sqrt((xs[i]-x)*(xs[i]-x) + (ys[i]-y)*(ys[i]-y));
                    if (distance < min)
                    {
                        value = values[i];
                        min= distance;
                    }
                }
                else
                {
//                    distance = pow((xs[i]-x)*(xs[i]-x) + (ys[i]-y)*(ys[i]-y), power/2.);
                    distance = pow((xs[i]-x)*(xs[i]-x) + (ys[i]-y)*(ys[i]-y), power);  // corrected 2021-08-24

                    if (distance > 0)
                    {
                        sumvalues = sumvalues + values[i]/distance;
                        sumdistances = sumdistances + 1/distance;
                    }
                    else
                        value = values[i];
                }
			}
		}

   if (value == missing_value)
      if (sumdistances == 0)
         value = missing_value;
      else
         value = sumvalues/sumdistances;

   return(value);
}

/*----------------------------*/
int interp_wx(double hrs, double x, double y, double missing_value, FBP *fbp, FEPS *feps, FGMWX *fgmwx)
/* hrs = time from midnight on day of ignition... hrs = 0 is the start of the time series array
   ext_status set to 0 if DMC drops below extinction level */

// NOTE: this code was changed 2018-08-06 to ext_stats = 0, errors may have been introduced.
{
int ext_status, day, iWxStn;
double DMCex = 20.,
       idw_pow=2., // nearest neighbour
       PI = 3.1415926535,
       u, v,
       values[MAX_STNS];

    ext_status = 1;    // fail (still burning)


    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
        values[iWxStn] = fgmwx->FFMCts[iWxStn][(int)hrs];
    fbp->FFMC = idw(values, fgmwx->xs, fgmwx->ys, MAX_STNS, missing_value, x, y, idw_pow);

//    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
//        printf("%d %d %lf %lf\n", iWxStn, (int)hrs, fgmwx->FFMCts[iWxStn][(int)hrs], fbp->FFMC);

    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
        if (fgmwx->Ws[iWxStn][(int)hrs] != missing_value && fgmwx->wds[iWxStn][(int)hrs] != missing_value)
            values[iWxStn] = -fgmwx->Ws[iWxStn][(int)hrs]*sin(PI*fgmwx->wds[iWxStn][(int)hrs]/180.);
        else
            values[iWxStn] = missing_value;
    u = idw(values, fgmwx->xs, fgmwx->ys, MAX_STNS, missing_value, x, y, idw_pow);

    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
        if (fgmwx->Ws[iWxStn][(int)hrs] != missing_value && fgmwx->wds[iWxStn][(int)hrs] != missing_value)
            values[iWxStn] = -fgmwx->Ws[iWxStn][(int)hrs]*cos(PI*fgmwx->wds[iWxStn][(int)hrs]/180.);
        else
            values[iWxStn] = missing_value;
    v = idw(values, fgmwx->xs, fgmwx->ys, MAX_STNS, missing_value, x, y, idw_pow);

    if (u != missing_value && v != missing_value)
    {
        fbp->WS = sqrt(u*u+v*v);
        fbp->WD = 180./PI * atan2(u, v)+180.;/* a little gymnastics to reorient circle */
    }
    else
    {
        fbp->WS = missing_value;
        fbp->WD = missing_value;
    }

    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
    {
        values[iWxStn] = fgmwx->DMCs[iWxStn][0];
    }
    feps->DMC = idw(values, fgmwx->xs, fgmwx->ys, MAX_STNS, missing_value, x, y, idw_pow);

    if (feps->DMC < DMCex) ext_status=0; /* fire is extinct */

    for (iWxStn=0; iWxStn< MAX_STNS; iWxStn++)
        values[iWxStn] = fgmwx->DCs[iWxStn][0];
    feps->DC = idw(values, fgmwx->xs, fgmwx->ys, MAX_STNS, missing_value, x, y, idw_pow);

    if (feps->DMC != missing_value && feps->DC != missing_value)
        fbp->BUI = BUIcalc(feps->DMC, feps->DC);
    else
        fbp->BUI = missing_value;

      return(ext_status);
}
/*----------------------------*/
void fire_propagate(double * ign0, int * FBPg0, int * slope0, int * aspect0, double * cbh0, double * cfl0, double * dfl0, double * pc0, double * sd0, double * sfl0, double * sh0, int direction, int method, FEPS *feps, FBP *fbp, FGMWX *fgmwx)
{
char Fuel0[FUEL_SIZE], Fuel1[FUEL_SIZE], Fuel2[FUEL_SIZE], Fuel3[FUEL_SIZE];
int i, j, k, icol, jrow, array_size, OldChanged, NewChanged, step, net_change; //, iFuel;
int *OldStack, *OldStack0, *NewStack, *NewStack0;
int *changed, *changed0, current;
int in[16], out[16], idir;    /* In and out directions for spread propogation */
double *ign, // *ign0,
       x, y, missing_value,
       xs[MAX_STNS], ys[MAX_STNS], elevs[MAX_STNS], values[MAX_STNS],
       t, T, rh, W, wd, rain, wxmax, u, v, DMC, DC, r2,
       FFMCign;

double newign, HFImin, // WSign, WDign,
       igntime,
       diagonal, hrs,
       hrs0, hrs1, hrs2, hrs3,
       GS0, GS1, GS2, GS3,
       Asp0, Asp1, Asp2, Asp3,
       CBH0, CBH1, CBH2, CBH3,
       CFL0, CFL1, CFL2, CFL3,
       DFL0, DFL1, DFL2, DFL3,
       PC0, PC1, PC2, PC3,
       SD0, SD1, SD2, SD3,
       SFL0, SFL1, SFL2, SFL3,
       SH0, SH1, SH2, SH3;

int *FBPg, *slope, *aspect;
double *cbh, *cfl, *dfl, *pc, *sd, *sfl, *sh;

// int status, ext_status, day,
int    iFuel0, iFuel1, iFuel2, iFuel3;

// trying to move away from global values
   array_size = nrows*ncols;
   missing_value = missing;

/* forward or reverse spread calculations w.r.t. time */
   if (direction >= 0)
      direction = 1;  /* default is forward growth */
   else
      direction = -1; /* backwards growth */
/* NOTE: when direction=-1, *ign and hrs < 0 */

   if (method <=8)
       method=8;
   else
       method=16;

/* 1. Define gird cell relations to propagation spread directions: 8 and 16 pt models from _top_ left */

                    in[11]=-2*ncols-1;                      in[12]= -2*ncols+1;
in[10]=  -ncols-2;  in[3]=   -ncols-1;  in[4]= -ncols;      in[5]=    -ncols+1; in[13]=   -ncols+2;
                    in[2]=         -1;  /* ignition pt */   in[6]=           1;
in[9]=    ncols-2;  in[1]=    ncols-1;    in[0]=  ncols;    in[7]=     ncols+1; in[14]=   ncols+2;
                    in[8]=  2*ncols-1;                      in[15]=  2*ncols+1;


                    out[15]=-2*ncols-1;                     out[8]= -2*ncols+1;
out[14]=  -ncols-2; out[7]=   -ncols-1; out[0]= -ncols;     out[1]=   -ncols+1; out[9]=   -ncols+2;
                    out[6]=         -1; /* ignition pt */   out[2]=          1;
out[13]=   ncols-2; out[5]=    ncols-1; out[4]=  ncols;     out[3]=  ncols+1;   out[10]=   ncols+2;
                    out[12]= 2*ncols-1;                     out[11]= 2*ncols+1;

/* grid is actually oriented from lower left corner */

                    in[8]=  2*ncols-1;                      in[15]=  2*ncols+1;
in[9]=    ncols-2;  in[1]=    ncols-1;    in[0]=  ncols;    in[7]=     ncols+1; in[14]=   ncols+2;
                    in[2]=         -1;  /* ignition pt */   in[6]=           1;
in[10]=  -ncols-2;  in[3]=   -ncols-1;  in[4]= -ncols;      in[5]=    -ncols+1; in[13]=   -ncols+2;
                    in[11]=-2*ncols-1;                      in[12]= -2*ncols+1;

                    out[12]= 2*ncols-1;                     out[11]= 2*ncols+1;
out[13]=   ncols-2; out[5]=    ncols-1; out[4]=  ncols;     out[3]=  ncols+1;   out[10]=   ncols+2;
                    out[6]=         -1; /* ignition pt */   out[2]=          1;
out[14]=  -ncols-2; out[7]=   -ncols-1; out[0]= -ncols;     out[1]=   -ncols+1; out[9]=   -ncols+2;
                    out[15]=-2*ncols-1;                     out[8]= -2*ncols+1;

    igntime = floor(feps->dtime/100) + (feps->dtime%100)/60.; // don't use this

/* 2. Create 2 stacks (OldStack and NewStack) to track locations of cells that have changed */
    OldStack =(int *)calloc(ncols*nrows, sizeof (int));
    OldStack0 = OldStack;

    ign = ign0;
    OldChanged = 0;
    for(jrow=0;jrow<nrows;jrow++)
        for(icol=0;icol<ncols;icol++)
        {
              if (*ign > 0)  // why was this *ign == 0 ?
            {
                *OldStack = ncols*jrow+icol;
                OldStack++;
                OldChanged++;
            }
            ign++;
        }

    NewStack =(int *)calloc(ncols*nrows, sizeof (int));
    NewStack0 = NewStack;
    ign = ign0;

/* 3. Build a grid to track steps when cells have last changed */
    changed =(int *)calloc(ncols*nrows, sizeof (int));
    changed0 = changed;
    for(jrow=0;jrow<nrows;jrow++)
        for(icol=0;icol<ncols;icol++)
        {
              if (*ign > 0)  // why was this *ign == 0 ?
                *changed = 1;
            else
                *changed = 0;

            changed++; ign++;
        }

    step = 0;
    net_change=OldChanged;

/* 4. While cells continue to change */
    while (OldChanged > 0)
    {
        NewChanged = 0;
        printf (".");

        OldStack = OldStack0; /* start at top of stack */
        NewStack = NewStack0; /* start at top of stack */

/* 4.a. Step through the old stack of changed cells */
        for (k=0; k < OldChanged; k++)
        {
            current = *OldStack;
            ign = ign0 + current;
            changed = changed0 + current;
            FBPg = FBPg0 + current;

            if (cbh0 != NULL) cbh = cbh0 + current;
            if (cfl0 != NULL) cfl = cfl0 + current;
            if (dfl0 != NULL) dfl = dfl0 + current;
            if (pc0 != NULL)  pc = pc0 + current;
            if (sd0 != NULL)  sd = sd0 + current;
            if (sfl0 != NULL) sfl = sfl0 + current;
            if (sh0 != NULL)  sh = sh0 + current;

// default values
            GS0=GS1=GS2=GS3= 0.;        // flat
            Asp0=Asp1=Asp2=Asp3= 0.;    // north facing
            CBH0=CBH1=CBH2=CBH3= -1.;   // missing
            CFL0=CFL1=CFL2=CFL3= -1.;   // missing
            DFL0=DFL1=DFL2=DFL3= -1.;   // missing
            PC0=PC1=PC2=PC3= -1.;       // missing
            SD0=SD1=SD2=SD3= -1.;       // missing
            SFL0=SFL1=SFL2=SFL3= -1.;   // missing
            SH0=SH1=SH2=SH3= -1.;       // missing

            if (slope0 != NULL && aspect0 != NULL)
            {
                slope = slope0 + current;
                aspect = aspect0 + current;
            }

            OldStack++;

/* 4.a.i. Unless the edge of the grid has been reached or time has expire */
    /* For the sake of spead, the calculations stop one square in from the edge */
            if ((current % ncols > 0) &&
                (current % ncols < ncols - 1) &&
                (current / ncols > 0) &&
                (current / ncols < nrows - 1) &&
    /* calculate only for cells that have changed */
                (*changed >= step) &&
    /* stop calculation when exceeded time limit */
                (direction*(*ign) < fgmwx->ndays*24.))
            {

// was
                /* Determine x-y location of cell */
//                i = current / nrows;
//                j = current % ncols;
// now          *OldStack = ncols*j+i;

                icol = current % ncols;
                jrow = current / ncols;

                x = cellsize*(icol+.5)+xllcorner;
                y = cellsize*(nrows-jrow+.5)+yllcorner;

                fbp->C = fgmwx->cured;

/* 4.a.ii Determine Fuel types of origin */
                iFuel0 = FBPType(Fuel0, *FBPg, fgmwx->greenup, fgmwx->standing);

                for (idir=0;idir<method;idir++)
                    if (current + out[idir] >= 0 && current + out[idir] < array_size) /* make sure we're within the grid */
                    {

/* 4.a.ii.1. Determine Fuel types of surrounding targets */
                        iFuel1 = FBPType(Fuel1, *(FBPg + out[idir]), fgmwx->greenup, fgmwx->standing);

                /* Determine intermediate fuels for 16 point growth */
                        if (idir>=8)
                        {
                            iFuel2 = FBPType(Fuel2, *(FBPg + out[idir-7]), fgmwx->greenup, fgmwx->standing);// was -9
                            iFuel3 = FBPType(Fuel3, *(FBPg + out[idir-8]), fgmwx->greenup, fgmwx->standing);// was -8
                        }

                     diagonal = 1.;
                     if (idir%2 == 1.)
                        diagonal = sqrt(2.);
                     if (idir >= 8)
                        diagonal = sqrt(5)/2;

                     fbp->theta = 45.*idir;
                     if (idir >= 8) // fbp->theta = 45.*(idir-8) + 22.5;... close but not quite
                     {
                         fbp->theta = 45.*(idir-8)+atan(.5)*180./3.1415926;
                         if (idir%2 == 1.)
                             fbp->theta = 45.*(idir-9)+atan(2.)*180./3.1415926;
                     }


                     if (iFuel0 >= 0 && iFuel1 >= 0
                         && ( direction * (*(ign + out[idir])) >= 0 || // a quick fix for now to continue capturing burned area
                              *(ign + out[idir])==NODATA_value ) )

                     {

                        if (slope0 != NULL && aspect0 != NULL)
                        {
                            GS0 = (double)(*slope);
                            Asp0 = (double)(*aspect);

                            GS1 = (double)(*(slope + out[idir]));
                            Asp1 = (double)(*(aspect + out[idir]));

                            if (idir >= 8)
                            {
                                GS2 = (double)(*(slope + out[idir-7]));
                                Asp2 = (double)(*(aspect + out[idir-7]));

                                GS3 = (double)(*(slope + out[idir-8]));
                                Asp3 = (double)(*(aspect + out[idir-8]));
                            }
                        } /* if (slope0 != NULL && aspect0 != NULL) */

// cbh0, cfl0, dfl0, pc0, sd0, sfl0, sh0,

                        if (cbh0 != NULL)
                        {
                            CBH0 = (double)(*cbh);
                            CBH1 = (double)(*(cbh + out[idir]));

                            if (idir >= 8)
                            {
                                CBH2 = (double)(*(cbh + out[idir-7]));
                                CBH3 = (double)(*(cbh + out[idir-8]));
                            }
                        } /* if (cbh0 != NULL) */

                        if (cfl0 != NULL)
                        {
                            CFL0 = (double)(*cfl);
                            CFL1 = (double)(*(cfl + out[idir]));

                            if (idir >= 8)
                            {
                                CFL2 = (double)(*(cfl + out[idir-7]));
                                CFL3 = (double)(*(cfl + out[idir-8]));
                            }
                        } /* if (cbh0 != NULL) */

                        if (dfl0 != NULL)
                        {
                            DFL0 = (double)(*dfl);
                            DFL1 = (double)(*(dfl + out[idir]));

                            if (idir >= 8)
                            {
                                DFL2 = (double)(*(dfl + out[idir-7]));
                                DFL3 = (double)(*(dfl + out[idir-8]));
                            }
                        } /* if (cbh0 != NULL) */

                        if (pc0 != NULL)
                        {
                            PC0 = (double)(*pc);
                            PC1 = (double)(*(pc + out[idir]));

                            if (idir >= 8)
                            {
                                PC2 = (double)(*(pc + out[idir-7]));
                                PC3 = (double)(*(pc + out[idir-8]));
                            }
                        } /* if (cbh0 != NULL) */

                        if (sd0 != NULL)
                        {
                            SD0 = (double)(*sd);
                            SD1 = (double)(*(sd + out[idir]));

                            if (idir >= 8)
                            {
                                SD2 = (double)(*(sd + out[idir-7]));
                                SD3 = (double)(*(sd + out[idir-8]));
                            }
                        } /* if (sd0 != NULL) */

                        if (sfl0 != NULL)
                        {
                            SFL0 = (double)(*sfl);
                            SFL1 = (double)(*(sfl + out[idir]));

                            if (idir >= 8)
                            {
                                SFL2 = (double)(*(sfl + out[idir-7]));
                                SFL3 = (double)(*(sfl + out[idir-8]));
                            }
                        } /* if (sfl0 != NULL) */

                        if (sh0 != NULL)
                        {
                            SH0 = (double)(*sh);
                            SH1 = (double)(*(sh + out[idir]));

                            if (idir >= 8)
                            {
                                SH2 = (double)(*(sh + out[idir-7]));
                                SH3 = (double)(*(sh + out[idir-8]));
                            }
                        } /* if (sh0 != NULL) */

/* 4.a.ii.2. Calculate time to burn out of origin cell into target cells */
                /* burn out of origin cell */
                        hrs = hrs0 = hrs1 = hrs2 = 0.;

                        hrs0 = burn_cell (Fuel0, GS0, Asp0, CBH0, CFL0, DFL0, PC0, SD0, SFL0, SH0, diagonal*cellsize/2., direction, *ign, x, y, missing_value, feps, fbp, fgmwx);
                        if (direction*hrs0 > 0.)
                        {
                /* burn into target cell */
                            hrs1 =  burn_cell (Fuel1, GS1, Asp1, CBH1, CFL1, DFL1, PC1, SD1, SFL1, SH1, diagonal*cellsize/2., direction, *ign+hrs0, x, y, missing_value, feps, fbp, fgmwx);
                            if (direction*hrs1 > 0.)
                            {
                                hrs = hrs0 + hrs1;

                                if (idir >= 8 )    /* 16 point model */
                                {
                /* burn intermediate cells */
                                    hrs2 = burn_cell (Fuel2, GS2, Asp2, CBH2, CFL2, DFL2, PC2, SD2, SFL2, SH2, diagonal*cellsize/2., direction, *ign+hrs, x, y, missing_value, feps, fbp, fgmwx);
                                    hrs3 = burn_cell (Fuel3, GS3, Asp3, CBH3, CFL3, DFL3, PC3, SD3, SFL3, SH3, diagonal*cellsize/2., direction, *ign+hrs, x, y, missing_value, feps, fbp, fgmwx);

                                    if (direction*hrs2 > 0. && direction*hrs3 > 0.)
                                        hrs = hrs + hrs2+hrs3;
                                    else
                                        hrs = 0;
                                }
                            }
                        }

/* 4.a.ii.3. If ignition time in target cell is more recent than the previously calculated time, consider it a new ignition and add it to NewStack and count of changed values */
                        newign = *ign+hrs;
// printf("%d %lf %s %s %s %s %lf %lf %lf %lf %lf\n", idir, newign, Fuel0, Fuel1, Fuel2, Fuel3, diagonal*cellsize/2., hrs0, hrs1, hrs2, hrs3);
                        if ((direction*newign < direction*(*(ign + out[idir])) ||
                            *(ign + out[idir]) == NODATA_value) &&
                            direction*newign < fgmwx->ndays*24. && // used to have (fgm.ndays+1)*24
                            direction*hrs > 0)
                        {

                           *(ign + out[idir]) = newign;
// printf("%d %d %lf %lf\n", current, current + out[idir], newign, *(ign+out[idir]) );
                           *NewStack = current + out[idir];/* Note the changed cell */
                           NewStack++;
                           NewChanged++;
                        }
                     } /* if (iFuel >= 0) */
                } /* if (current + direction*out[idir] >=0 && ... */
            }   /* if ... && (ign < fgm.ndays*24) */
        }  /* for (k=0; k < OldChanged; k++) */


/* 4.b. Transfer newly changed locations to old stack */
        OldChanged = NewChanged;
        OldStack = OldStack0;
        NewStack = NewStack0;
        for (i=0;i<NewChanged; i++)
        {
            *OldStack = *NewStack;
            OldStack++;
            NewStack++;
        }
    } /* while (net_change > 0) */

//   free(changed);
    free(OldStack);
    free(NewStack);
}

/*----------------------------*/
/* this routine calculates the time to burn a target distance */
double burn_cell(char * Fuel, double GS, double Asp, double CBH, double CFL, double DFL, double PC, double SD, double SFL, double SH, double target, int direction, double ign, double x, double y, double missing_value, FEPS *feps, FBP *fbp, FGMWX *fgmwx)
{
double distance, delta, hr, hrs, m, d, r1, r2, t2, timestep;
int status, ext_status, jday;

    distance=0;
    status=0;    // success
    hrs=0;    /* represents the time spent in this subroutine crossing the cell */
    timestep=60.*feps->timestep;   // convert hours to minutes

    strcpy(fbp->FuelType, Fuel);

    fbp->GS = GS;
    fbp->Aspect = Asp;
    if (direction < 0)
        fbp->Aspect = fbp->Aspect + 180.;
    if (fbp->Aspect > 360.)
        fbp->Aspect = fbp->Aspect - 360.;

    if (direction >= 0.)
        fbp->t = ign;
    else
        fbp->t = fgmwx->ndays*24 + ign;

    ext_status = interp_wx(ign, x, y, missing_value, fbp, feps, fgmwx);    // 0 = extinct, >0 still burning

    if (direction < 0)
        fbp->WD = fbp->WD + 180.;
    if (fbp->WD > 360.)
        fbp->WD = fbp->WD - 360.;

    status = FBPCalc (fbp); // was (&fbp)
    r2 = fbp->TROSt;

    fbp->CBH = CBH;
    fbp->CFL = CFL;
    fbp->DFL = DFL;
    fbp->PC = PC;
    fbp->SD = SD;
    fbp->SFL =SFL;
    fbp->SH = SH;

    while ((distance < target) && (status >= 0) && (ext_status > 0))
    {

        r1 = r2;

/* fbp->t is time since ignition (needed for accelleration) */
        fbp->t = fbp->t + direction*timestep/60.;     // NOTE: fgm->timestep is NOT adjusted for direction  // NOTE: not sure if feps->timestep is appropriate
        if  (fbp->t < 0 || fbp->t > fgmwx->ndays*24.)
            status=-2;

        if (ext_status > 0 && status >= 0)
        {
/* reverse the wind direction for calculations */
            if (direction < 0)
                fbp->WD = fbp->WD + 180.;
            if (fbp->WD > 360.)
                fbp->WD = fbp->WD - 360.;

            status = FBPCalc (fbp);
            r2 = fbp->TROSt; /* [m/min] */

            if (status>=0)
            {
                delta = (r1 + r2)/2. * timestep; /* distance covered in one timestep [min] */

                if (distance + delta < target)
                {
                    distance = distance + delta;
                    hrs = hrs + direction * timestep/60.; /* negative if reverse direction  [hours] */
                }
                else
                {
                    m = direction*(r2 - r1)/timestep;    /* [m/s2] */
                    d = target - distance; /* remaining distance */

                    if (m == 0.)
                        hrs = hrs + direction * d/r1 /60.;
                    else
                        hrs = hrs + direction * d/r1 /60.;  // keeping it simple
//                      hrs = hrs + direction * (-r1 + sqrt(r1*r1 + direction*2*m*d))/m /60.;    // quadradic - old approach
//                      hrs = hrs + (-2*TROS2 + sqrt(4*TROS2*TROS2 + 2*m*d))/m /60.;    // quadradic - older approach

                    distance = target;
                }
            } /* if (status>=0) */
        } /* if (ext_status >= 0 && status >= 0) */
        else
            hrs=0.;

// printf("... %d %d %lf %lf %lf %lf\n", ext_status, status, r1, r2, distance, target);
    } /* while ((distance < target) && (status >= 0) && (ext_status > 0)) */
// printf("\n");
    return (ext_status*hrs);

}
