/*  Code originally written by Scott Powell,
    Updated by Kerry Anderson */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <malloc.h>
#include <math.h>
//#include "pfas.h"
#include "feps_2011.h"
#include "fbp_2019.h"

//unsigned long ntohl(unsigned long netlong);

FILE *infile, *outfile, *binfile, *hdrfile;


/*************************************************
 *         Flip bytes in an integer               *
 *************************************************/

 /* This function is called when the magic number in a regex doesn't match, in
 order to flip its bytes to see if we are dealing with a pattern that was
 compiled on a host of different endianness. If so, this function is used to
 flip other byte values.

 Arguments:
   value        the number to flip
   n            the number of bytes to flip (assumed to be 2 or 4)

 Returns:       the flipped value
 */

// static long int
long int byteflip(long int value, int n)
{
	if (n == 2)
		return ((value & 0x00ff) << 8) |
			   ((value & 0xff00) >> 8);

	return ((value & 0x000000ff) << 24) |
		   ((value & 0x0000ff00) <<  8) |
	       ((value & 0x00ff0000) >>  8) |
	       ((value & 0xff000000) >> 24);
}

unsigned long FlipBytes(unsigned long i)
{
unsigned long byte0, byte1, byte2, byte3;

   byte0 = i << 24;
   byte1 = i << 8;
   byte2 = i >> 8;
   byte3 = i >> 24;

   return(byte0 + byte1 + byte2 + byte3);
}

/*-------------------------------------------------*/
double *read_ltg_fcst(char * infilename)
{
double *array;
double xpole, ypole;
int i, j;

char buffer[64];
char variable[24];

    xllcorner=0.;
    yllcorner=0.;
    cellsize=0.;
    NODATA_value=-999.;

    printf("Reading %s...\n", infilename);
	if ( (infile = fopen(infilename, "r")) != NULL)
	{
		while( !feof( infile ) )
		{
			fscanf(infile,"%s", buffer );

            if (!strcmp(buffer, "VARIABLE"))
				fscanf( infile," %s %s", buffer, variable);

            if (!strcmp(buffer, "NI"))
				fscanf( infile," %s %d", buffer, &nrows);
            if (!strcmp(buffer, "NJ"))
				fscanf( infile," %s %d", buffer, &ncols);
            if (!strcmp(buffer, "XPOLE"))
				fscanf( infile," %s %lf", buffer, &xpole);
            if (!strcmp(buffer, "YPOLE"))
				fscanf( infile," %s %lf", buffer, &ypole);
            if (!strcmp(buffer, "MESHPS"))
				fscanf( infile," %s %lf", buffer, &cellsize);

            if (!strcmp(buffer, "START_DATA") && !strcmp(variable, "LF"))
            {
//                printf("3. %s '%s' %d %d\n", buffer, variable, nrows, ncols);

                xllcorner = -1. * cellsize * xpole;
                yllcorner = -1. * cellsize * ypole;
//                printf("%lf %lf %lf %lf %lf\n\n", xpole, ypole, cellsize, xllcorner, yllcorner);

/* Allocate memory for array */
                array = (double *)calloc(ncols*nrows, sizeof(double));

/* Read the data into the array structure */
                for(j=0;j<nrows;j++)
                    for(i=0;i<ncols;i++)
                    {
                        fscanf (infile,"%lf", array);
//                        printf("%lf\n", *array);
                        array++;
                    }
                array -= ncols*nrows;
                fclose(infile);
                return(array);
            }
        }
    }
    else
    {
        printf("%s not found\n", infilename);
        return(NULL);
    }
}

/*-------------------------------------------------*/
double *read_wgrib(char * infilename)
{
double *array;
int i, j;

// printf("%s\n", infilename);

   if ((infile = fopen(infilename, "r")) != NULL)
   {
      fscanf(infile,"%d %d", &ncols, &nrows);


/* Allocate memory for array */
      array = (double *)calloc(ncols*nrows, sizeof(double));

/* Read the data into the array structure */
      for(j=0;j<nrows;j++)
         for(i=0;i<ncols;i++)
         {
            fscanf (infile,"%lf", array);
			array++;
         }
      array -= ncols*nrows;
      fclose(infile);
      return(array);
   }
   else
   {
	  printf("%s not found\n", infilename);
      return(NULL);
   }
}

/*-------------------------------------------------*/
int *read_int_ASCIIGRID(char * infilename)
{
int *array;
int i, j, temp;
double dtemp;
char value[24];

   if ((infile = fopen(infilename, "r")) != NULL)
   {
      fscanf(infile,"%s %d ", value, &ncols);
      fscanf(infile,"%s %d ", value, &nrows);
      fscanf(infile,"%s %lf ",value, &xllcorner);
      fscanf(infile,"%s %lf ",value, &yllcorner);
      fscanf(infile,"%s %lf ",value, &cellsize);
      fscanf(infile,"%s %d ", value, &temp);    // temporary NODATA_value (to be written over by universal NODATA_value)
//      fscanf(infile,"%s %d ", value, &NODATA_value);

/* Allocate memory for array */
      array = (int *)calloc(ncols*nrows, sizeof(int));

/* Read the data into the array structure */
      for(j=0;j<nrows;j++)
         for(i=0;i<ncols;i++)
         {
            fscanf (infile,"%lf", & dtemp);
            *array = (int)dtemp;
//            fscanf (infile,"%d", array);
            if (*array==temp) *array = NODATA_value;
			array++;
         }
      array -= ncols*nrows;
      fclose(infile);
      return(array);
   }
   else
   {
	  printf("%s not found\n", infilename);
      return(NULL);
   }
}
/*-------------------------------------------------*/
int write_int_ASCIIGRID (char * outfilename, int * array)
{
int i,j;

   if ((outfile = fopen(outfilename, "w")) != NULL)
   {
      fprintf(outfile,"ncols        %d\n",  ncols);
      fprintf(outfile,"nrows        %d\n",  nrows);
      fprintf(outfile,"xllcorner    %lf\n", xllcorner);
      fprintf(outfile,"yllcorner    %lf\n", yllcorner);
      fprintf(outfile,"cellsize     %lf\n", cellsize);
      fprintf(outfile,"NODATA_value %d\n",  NODATA_value);

      for(j=0;j<nrows;j++)
      {
         for(i=0;i<ncols;i++)
            fprintf (outfile,"%d ", *array++);
         fprintf(outfile, "\n");
      }

      fclose(outfile);
      return(0);
   }
   else
   {
	  printf("Error writing %s\n", outfilename);
      return(-1);
   }
}
/*-------------------------------------------------*/
int write_as_int_ASCIIGRID (char * outfilename, double * array)
{
int i,j;

   if ((outfile = fopen(outfilename, "w")) != NULL)
   {
      fprintf(outfile,"ncols        %d\n",  ncols);
      fprintf(outfile,"nrows        %d\n",  nrows);
      fprintf(outfile,"xllcorner    %lf\n", xllcorner);
      fprintf(outfile,"yllcorner    %lf\n", yllcorner);
      fprintf(outfile,"cellsize     %lf\n", cellsize);
      fprintf(outfile,"NODATA_value %d\n",  NODATA_value);

      for(j=0;j<nrows;j++)
      {
         for(i=0;i<ncols;i++)
            fprintf (outfile,"%d ", (int)*array++);
         fprintf(outfile, "\n");
      }

      fclose(outfile);
      return(0);
   }
   else
   {
	  printf("Error writing %s\n", outfilename);
      return(-1);
   }
}
/*-------------------------------------------------*/
double *read_double_ASCIIGRID(char * infilename)
{
double *array;
int i, j, temp;
char value[24];

   if ((infile = fopen(infilename, "r")) != NULL)
   {
      fscanf(infile,"%s %d ", value, &ncols);
      fscanf(infile,"%s %d ", value, &nrows);
      fscanf(infile,"%s %lf ",value, &xllcorner);
      fscanf(infile,"%s %lf ",value, &yllcorner);
      fscanf(infile,"%s %lf ",value, &cellsize);
      fscanf(infile,"%s %d ", value, &temp);    // temporary NODATA_value (to be written over by universal NODATA_value)
//      fscanf(infile,"%s %d ", value, &NODATA_value);

/* Allocate memory for array */
      array = (double *)calloc(ncols*nrows, sizeof(double));

/* Read the data into the array structure */
      for(j=0;j<nrows;j++)
         for(i=0;i<ncols;i++)
         {
            fscanf (infile,"%lf", array);
            if (*array==temp) *array = NODATA_value;
			array++;
         }
      array -= ncols*nrows;
      fclose(infile);
      return(array);
   }
   else
   {
	  printf("Error reading %s\n", infilename);
      return(NULL);
   }
}
/*-------------------------------------------------*/
int write_double_ASCIIGRID (char * outfilename, double * array)
{
int i,j;
   if ((outfile = fopen(outfilename, "w")) != NULL)
   {
      fprintf(outfile,"ncols        %d\n",  ncols);
      fprintf(outfile,"nrows        %d\n",  nrows);
      fprintf(outfile,"xllcorner    %lf\n", xllcorner);
      fprintf(outfile,"yllcorner    %lf\n", yllcorner);
      fprintf(outfile,"cellsize     %lf\n", cellsize);
      fprintf(outfile,"NODATA_value %d\n",  NODATA_value);

      for(j=0;j<nrows;j++)
      {
         for(i=0;i<ncols;i++)
            fprintf (outfile,"%lf ", *array++);
         fprintf(outfile, "\n");
      }
      fclose(outfile);
      return(0);
   }
   else
   {
	  printf("Error writing %s\n", outfilename);
      return(-1);
   }
}
/*-------------------------------------------------*/
double *read_double_BINGRID(char *binfilename, char *hdrfilename, double x, double y, double size)
{
double *array, val;
int top, bottom, left, right, position, gridsize;
int i, j, result, status, temp;
char value[24], byteorder[24], a[4];
unsigned long old, nnew;

   x = 367014.;
   y = 6083405;
   size = 100000.;
	strcpy(a, "x");

printf("Opening %s...\n", hdrfilename);
   if ((hdrfile = fopen(hdrfilename, "r")) != NULL)
   {
      fscanf(hdrfile,"%s %d ", value, &ncols);
      fscanf(hdrfile,"%s %d ", value, &nrows);
      fscanf(hdrfile,"%s %lf ",value, &xllcorner);
      fscanf(hdrfile,"%s %lf ",value, &yllcorner);
      fscanf(hdrfile,"%s %lf ",value, &cellsize);
      fscanf(hdrfile,"%s %d ", value, &temp);    // temporary NODATA_value (to be written over by universal NODATA_value)
//      fscanf(infile,"%s %d ", value, &NODATA_value);
      fscanf(hdrfile,"%s %s ", value, &byteorder);

/* Adjust x and y to fit the grid */
	  x = x - (int)(x - xllcorner)%(int)cellsize;
	  y = y - (int)(y - yllcorner)%(int)cellsize;

	  top = (int)ceil((y - yllcorner + size/2)/cellsize);
      bottom  = (int)floor((y - yllcorner - size/2)/cellsize);
	  left = (int)floor((x - xllcorner - size/2)/cellsize);
	  right = (int)ceil((x - xllcorner + size/2)/cellsize);

      gridsize = (int)((top-bottom)*(right-left));
	  printf("%lf %d\n", size, gridsize);

/* Allocate memory for array */
      array = (double *)calloc((top-bottom)*(right-left), sizeof(double));

	  printf("Opening %s...\n", binfilename);
	  if ((binfile = fopen(binfilename, "rb")) != NULL)
	  {

/* Read the data into the array structure */
         for (j=top;j>bottom;j--)
			for (i=left;i<right;i++)
			{
			   *array=-999.;
 		       position = (nrows-j)*ncols+i;
               result = fseek(binfile, 4*position, SEEK_SET);

			   if (result==0)
			   {
			      status = fread(&val, sizeof(double), 1, binfile);

//				  printf("%lf ", val);
				  if (!strcmp(byteorder, "LSBFIRST"))
				  {
                     old = (long)val;
					 nnew = FlipBytes(old);
//					 nnew = byteflip (old, 4);

/* flips the bit order within each byte from 0 lsb to 0 msb * /
					 nnew = 0;
                     for (k = 0; k<=3; k++)
					 {
                        nnew |= (old & 01);      /* copy a bit * /
                        if (k < 3)
						{
                           old = old >> 1;
                           nnew = nnew << 1;
						}
					 } */

					 val=(double)nnew;
//					 val=(double)old;
				  }
				  printf("%lf\n", val);
  				  *array=val;
                 if (*array==temp) *array = NODATA_value;

			   }
			   else
			  	  printf("Error reading %d of %s\n", position, binfilename);

   		       array++;
			}

		ncols = right-left;
		nrows = top-bottom;
		xllcorner = x - size/2.;
		yllcorner = y - size/2.;

         array -= ncols*nrows;
         fclose(infile);
         return(array);

		 nrows=right-left;
		 ncols=top-bottom;
		 xllcorner=x-nrows/2*cellsize;
		 yllcorner=y-ncols/2*cellsize;
      }
      else
	  {
	     printf("Error reading %s\n", binfilename);
         return(NULL);
	  }

   }
   else
   {
	  printf("Error reading %s\n", hdrfilename);
      return(NULL);
   }
}
/*-------------------------------------------------*/
int *read_trim_int_ASCIIGRID(char * infilename, double new_size, double center_x, double center_y)
{
int *array, *array0;
int *new_array, *new_array0;
int i, j, i_ul, j_ul, temp;
int new_ncols, new_nrows;
double new_xllcorner, new_yllcorner, new_xurcorner, new_yurcorner, dtemp;
char value[24];

   if ((infile = fopen(infilename, "r")) != NULL)
   {
      fscanf(infile,"%s %d ", value, &ncols);
      fscanf(infile,"%s %d ", value, &nrows);
      fscanf(infile,"%s %lf ",value, &xllcorner);
      fscanf(infile,"%s %lf ",value, &yllcorner);
      fscanf(infile,"%s %lf ",value, &cellsize);
      fscanf(infile,"%s %d ", value, &temp);    // temporary NODATA_value (to be written over by universal NODATA_value)
//      fscanf(infile,"%s %d ", value, &NODATA_value);

/* Allocate memory for array */
      array0 = (int *)calloc(ncols*nrows, sizeof(int));
	  array=array0;

/* Read the data into the array structure */
      for(j=0;j<nrows;j++)
         for(i=0;i<ncols;i++)
         {
            fscanf (infile,"%lf", &dtemp);
            *array = (int)dtemp;
//            fscanf (infile,"%d", array);
            if (*array==temp) *array = NODATA_value;
			array++;
         }
      fclose(infile);

	  if (center_x > xllcorner + cellsize*ncols) center_x=xllcorner + cellsize*ncols;
	  if (center_y > yllcorner + cellsize*nrows) center_y=yllcorner + cellsize*nrows;
	  if (center_x < xllcorner) center_x=xllcorner;
	  if (center_y < yllcorner) center_y=yllcorner;

	  new_xllcorner = center_x - new_size/2.;
	     if (new_xllcorner < xllcorner) new_xllcorner = xllcorner;
      new_yllcorner = center_y - new_size/2.;
	     if (new_yllcorner < yllcorner) new_yllcorner = yllcorner;

	  new_xurcorner = center_x + new_size/2.;
	     if (new_xurcorner > xllcorner + cellsize*ncols) new_xurcorner = xllcorner + cellsize*ncols;
      new_yurcorner = center_y + new_size/2.;
	     if (new_yurcorner > yllcorner + cellsize*nrows) new_yurcorner = yllcorner + cellsize*nrows;

	  new_ncols = (int)((new_xurcorner-new_xllcorner)/cellsize);
      new_nrows = (int)((new_yurcorner-new_yllcorner)/cellsize);

	  i_ul = (int)((new_xllcorner-xllcorner)/cellsize);
      j_ul = nrows - (int)((new_yllcorner-yllcorner)/cellsize) - new_nrows;

/* Allocate memory for array */
      new_array0 = (int *)calloc(new_ncols*new_nrows, sizeof(int));
   	  new_array=new_array0;

      for(j=0;j<new_nrows;j++)
         for(i=0;i<new_ncols;i++)
           *(new_array0+i+j*new_ncols) =  *(array0 + i+i_ul + (j+j_ul)*ncols);

      ncols = new_ncols;
      nrows = new_nrows;
      xllcorner = new_xllcorner;
      yllcorner = new_yllcorner;

      return(new_array0);

   }
   else
   {
	  printf("%s not found\n", infilename);
      return(NULL);
   }
}
/*-------------------------------------------------*/
double *read_trim_double_ASCIIGRID(char * infilename, double new_size, double center_x, double center_y)
{
double *array, *array0;
double *new_array, *new_array0;
int i, j, i_ul, j_ul, temp;
int new_ncols, new_nrows;
double new_xllcorner, new_yllcorner, new_xurcorner, new_yurcorner;
char value[24];

   if ((infile = fopen(infilename, "r")) != NULL)
   {
      fscanf(infile,"%s %d ", value, &ncols);
      fscanf(infile,"%s %d ", value, &nrows);
      fscanf(infile,"%s %lf ",value, &xllcorner);
      fscanf(infile,"%s %lf ",value, &yllcorner);
      fscanf(infile,"%s %lf ",value, &cellsize);
      fscanf(infile,"%s %d ", value, &temp);    // temporary NODATA_value (to be written over by universal NODATA_value)
//      fscanf(infile,"%s %d ", value, &NODATA_value);

/* Allocate memory for array */
      array0 = (double *)calloc(ncols*nrows, sizeof(double));
	  array=array0;

/* Read the data into the array structure */
      for(j=0;j<nrows;j++)
         for(i=0;i<ncols;i++)
         {
            fscanf (infile,"%lf", array);
            if (*array==temp) *array = NODATA_value;
			array++;
         }
      fclose(infile);

	  if (center_x > xllcorner + cellsize*ncols) center_x=xllcorner + cellsize*ncols;
	  if (center_y > yllcorner + cellsize*nrows) center_y=yllcorner + cellsize*nrows;
	  if (center_x < xllcorner) center_x=xllcorner;
	  if (center_y < yllcorner) center_y=yllcorner;

	  new_xllcorner = center_x - new_size/2.;
	     if (new_xllcorner < xllcorner) new_xllcorner = xllcorner;
      new_yllcorner = center_y - new_size/2.;
	     if (new_yllcorner < yllcorner) new_yllcorner = yllcorner;

	  new_xurcorner = center_x + new_size/2.;
	     if (new_xurcorner > xllcorner + cellsize*ncols) new_xurcorner = xllcorner + cellsize*ncols;
      new_yurcorner = center_y + new_size/2.;
	     if (new_yurcorner > yllcorner + cellsize*nrows) new_yurcorner = yllcorner + cellsize*nrows;

	  new_ncols = (int)((new_xurcorner-new_xllcorner)/cellsize);
      new_nrows = (int)((new_yurcorner-new_yllcorner)/cellsize);

	  i_ul = (int)((new_xllcorner-xllcorner)/cellsize);
      j_ul = nrows - (int)((new_yllcorner-yllcorner)/cellsize) - new_nrows;

/* Allocate memory for array */
      new_array0 = (double *)calloc(new_ncols*new_nrows, sizeof(double));
   	  new_array=new_array0;

      for(j=0;j<new_nrows;j++)
         for(i=0;i<new_ncols;i++)
           *(new_array0+i+j*new_ncols) =  *(array0 + i+i_ul + (j+j_ul)*ncols);

      ncols = new_ncols;
      nrows = new_nrows;
      xllcorner = new_xllcorner;
      yllcorner = new_yllcorner;

      return(new_array0);

   }
   else
   {
	  printf("%s not found\n", infilename);
      return(NULL);
   }
}
/*-------------------------------------------------*/
int write_trim_int_ASCIIGRID (char * outfilename, int * array0, double new_size, double center_x, double center_y)
{
int i,j, i_ul, j_ul;
int new_ncols, new_nrows;
double new_xllcorner, new_yllcorner;

   new_ncols = (int)(new_size/cellsize);
   new_nrows = (int)(new_size/cellsize);
   new_xllcorner = center_x - new_size/2.;
   new_yllcorner = center_y - new_size/2.;
   i_ul = (new_xllcorner-xllcorner)/cellsize;
   j_ul = nrows-(new_yllcorner-yllcorner)/cellsize-new_nrows;

   if ((outfile = fopen(outfilename, "w")) != NULL)
   {
      fprintf(outfile,"ncols        %d\n",  new_ncols);
      fprintf(outfile,"nrows        %d\n",  new_nrows);
      fprintf(outfile,"xllcorner    %lf\n", new_xllcorner);
      fprintf(outfile,"yllcorner    %lf\n", new_yllcorner);
      fprintf(outfile,"cellsize     %lf\n", cellsize);
      fprintf(outfile,"NODATA_value %d\n",  NODATA_value);

      for(j=0;j<new_nrows;j++)
      {
         for(i=0;i<new_ncols;i++)

            fprintf (outfile,"%d ", *(array0 + i+i_ul + (j+j_ul)*ncols ));
         fprintf(outfile, "\n");
      }

      fclose(outfile);
      return(0);
   }
   else
   {
	  printf("Error writing %s\n", outfilename);
      return(-1);
   }
}
/*-------------------------------------------------*/
