#include <math.h>
#include <stdio.h>
//#include "pfas.h"
#include "feps_2011.h"
//#include "Forecast.h"


double idw(double values[MAX_STNS], double xs[MAX_STNS], double ys[MAX_STNS],
     int istations, double missing_value, double x, double y, double power)
{
double value, distance, sumvalues, sumdistances;
int i;

   value = missing_value;
   sumvalues = 0.;
   sumdistances = 0.;

   if (istations==1)
	   value = values[0];
   else
		for (i=0;i<istations;i++)
		{
			if (values[i] != missing_value &&
	            xs[i] != missing_value &&
		        ys[i] != missing_value)
			{
				distance = pow((xs[i]-x)*(xs[i]-x) + (ys[i]-y)*(ys[i]-y),
					         power/2);

//				printf("%lf %lf %lf %lf\n",  values[i], xs[i], ys[i], distance);

				if (distance > 0)
				{
					sumvalues = sumvalues + values[i]/distance;
					sumdistances = sumdistances + 1/distance;
				}
				else
				{
					value = values[i];
//					return(value);
				}
			}
		}

   if (value == missing_value)
      if (sumdistances == 0)
         value = missing_value;
      else
         value = sumvalues/sumdistances;

   return(value);
}

/* idw2 conducts the same inverse distance weighted interpolation scheme as idw
   but adds an additional weighting term per station: ws[].

   Typically, the weighting term would equal something like the number of years of data.

*/

//double idw2(double values[MAX_WX_STNS], double xs[MAX_WX_STNS], double ys[MAX_WX_STNS], double ws[MAX_WX_STNS],
double idw2(double values[MAX_STNS], double xs[MAX_STNS], double ys[MAX_STNS], double ws[MAX_STNS],
     int istations, double missing_value, double x, double y, double power)
{
double value, distance, sumvalues, sumdistances;
int i;

   value = missing_value;
   sumvalues = 0.;
   sumdistances = 0.;

   if (istations==1)
	   value = values[0];
   else
		for (i=0;i<istations;i++)
		{
			if (values[i] != missing_value &&
				xs[i] != missing_value &&
				ys[i] != missing_value)
			{
				distance = pow((xs[i]-x)*(xs[i]-x) + (ys[i]-y)*(ys[i]-y),
					         power/2);

				if (distance > 0)
				{
					sumvalues = sumvalues + ws[i]*values[i]/distance;
					sumdistances = sumdistances + ws[i]/distance;
				}
				else
				{
					value = values[i];
//					return(value);
				}
			}
		}

   if (value == missing_value) /* zero or more than one station */
      if (sumdistances == 0)
         value = missing_value;
      else
         value = sumvalues/sumdistances;

   return(value);
}

//double idwll(double values[MAX_WX_STNS], double lats[MAX_WX_STNS], double lngs[MAX_WX_STNS],
double idwll(double values[MAX_STNS], double lats[MAX_STNS], double lngs[MAX_STNS],
     int istations, double missing_value, double lat, double lng, double power)
{
double value, distance, sumvalues, sumdistances, c;
int i;

   value = missing_value;
   sumvalues = 0.;
   sumdistances = 0.;

   c=cos(3.1415926*lat/180.);

   for (i=0;i<istations;i++)
   {

      if (values[i] != missing_value &&
          lats[i] != missing_value &&
          lngs[i] != missing_value)
      {
         distance = 111.* sqrt((lats[i]-lat)*(lats[i]-lat) + c*c*(lngs[i]-lng)*(lngs[i]-lng));

	     if (distance > 0)
		 {
            sumvalues = sumvalues + values[i]/pow(distance,power);
            sumdistances = sumdistances + 1/pow(distance,power);
		 }
         else
		 {
            value = values[i];
//	        return(value);
		 }

      }
   }

   if (value == missing_value)
      if (sumdistances == 0)
         value = missing_value;
      else
         value = sumvalues/sumdistances;


   return(value);
}

double idwll2(double values[MAX_DCS], double lats[MAX_DCS], double lngs[MAX_DCS],
     int istations, double missing_value, double lat, double lng, double power)
{
double value, distance, sumvalues, sumdistances, c;
int i;

   value = missing_value;
   sumvalues = 0.;
   sumdistances = 0.;

   if (power==0) printf("Error: power value equals zero\n");

   c=cos(3.1415926*lat/180.);

   for (i=0;i<istations;i++)
   {

      if (values[i] != missing_value &&
          lats[i] != missing_value &&
          lngs[i] != missing_value)
      {
         distance = 111.* sqrt((lats[i]-lat)*(lats[i]-lat) + c*c*(lngs[i]-lng)*(lngs[i]-lng));

	     if (distance > 0)
		 {
            sumvalues = sumvalues + values[i]/pow(distance,power);
            sumdistances = sumdistances + 1/pow(distance,power);
		 }
         else
		 {
            value = values[i];
//	        return(value);
		 }

      }
   }

   if (value == missing_value)
      if (sumdistances == 0)
         value = missing_value;
      else
         value = sumvalues/sumdistances;


   return(value);
}

