#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include "pfas.h"
#include "feps_2011.h"
//#include "malloc.h"

FILE *infile;

int ReadEmissionsData(char *infilename, EMISSIONS *emissions)
{
char buffer[120], species[SPECIES_SIZE], *args[10], *token, *fields[10];
int i, iColumn, iModel, jModel, nModels, iSpecies, jSpecies, nSpecies, maxfields, iFlaming, iSmoldering, iResidual;

    for (i=0;i<MAX_SPECIES;i++)
    {
        emissions->species[i] = (char *)calloc(SPECIES_SIZE, sizeof(double));
        strncpy(emissions->species[i], "missing", SPECIES_SIZE);

/* these are the original FEPS values, smoldering and residual now replace with depth of burn (at 1 cm per hour) */
        emissions->residence_flaming=0.25;   /* residence time for flaming combustion (hours) */
        emissions->residence_smoldering=1.75; /* residence time for smoldering combustion (hours)  - follows flaming */
        emissions->residence_residual=4.;    /* residence time for residual combustion (hours)  - follows smoldering */
    }
    
    for (i=0;i<MAX_EMODELS; i++)
        emissions->emodel[i] = 0;    

    nSpecies=0; // this tracks each new species
    nModels=0;  // this tracks unique models

    
    maxfields=0;
    iSpecies = iFlaming = iSmoldering = iResidual = iModel = -999;  // these track the column number of fields within the csv (-999 = absent)
    
    for (i=0;i<10;i++)
    {
        args[i] = (char *)calloc(SPECIES_SIZE, sizeof(char));
        fields[i] =(char *)calloc(12, sizeof (char));    
    }

      if ( (infile = fopen(infilename, "r")) != NULL)
    {
        fgets(buffer, 120, infile);// header
        iColumn=0;

// read in the header
        token = strtok(buffer, ",");
        while (token != NULL && iColumn < 12)
        {
            strcpy(fields[iColumn], token);
            token = strtok(NULL, ",");

// keep track of the fields wrt columns (crude allowance for case)
            if (strstr(fields[iColumn], "SPE")) iSpecies = iColumn;
            if (strstr(fields[iColumn], "FLA")) iFlaming = iColumn;
            if (strstr(fields[iColumn], "SMO")) iSmoldering= iColumn;
            if (strstr(fields[iColumn], "RES")) iResidual = iColumn;
            if (strstr(fields[iColumn], "MOD")) iModel = iColumn;

            if (strstr(fields[iColumn], "Spe")) iSpecies = iColumn;
            if (strstr(fields[iColumn], "Fla")) iFlaming = iColumn;
            if (strstr(fields[iColumn], "Smo")) iSmoldering= iColumn;
            if (strstr(fields[iColumn], "Res")) iResidual = iColumn;
            if (strstr(fields[iColumn], "Mod")) iModel = iColumn;

            if (strstr(fields[iColumn], "spe")) iSpecies = iColumn;
            if (strstr(fields[iColumn], "fla")) iFlaming = iColumn;
            if (strstr(fields[iColumn], "smo")) iSmoldering= iColumn;
            if (strstr(fields[iColumn], "res")) iResidual = iColumn;
            if (strstr(fields[iColumn], "mod")) iModel = iColumn;
            
            iColumn++;
        }

        maxfields=iColumn;

// read in the rest of the file

        fgets(buffer, 120, infile);// first line of data

         while( !feof( infile ) && nModels < MAX_EMODELS && nSpecies < MAX_SPECIES)
        {
            iColumn=0; 
            
            token = strtok(buffer, ",");

            while (token != NULL&& iColumn < maxfields)
            {
                strcpy(args[iColumn], token);
                token = strtok(NULL, ",");
                iColumn++;
            }
            
// Find jModel 
            if (iModel==-999)   // there is no column for emissions model
            {
                jModel=0;
                nModels=0;
            }
            else
            {
                jModel = -1;
                for (i=0; i<nModels; i++)
                    if (emissions->emodel[i] == atoi(args[iModel])) jModel = i;
            
                if (jModel<0)  // new model
                {
                    jModel=i;
                    emissions->emodel[jModel] = atoi(args[iModel]);  // add to stack of models within the list
                    nModels++;
                }
            }

// Find species
            jSpecies = -1;
            for (i=0; i<nSpecies; i++)
                if (!strcmp(emissions->species[i], args[iSpecies])) jSpecies = i;

            if (jSpecies<0)
            {
                jSpecies=i;
                strcpy(emissions->species[jSpecies], args[iSpecies]);  // add to stack of species within the list
                nSpecies++;
            }
            
            if (iFlaming >= 0)    emissions->a[jModel][jSpecies] = atof(args[iFlaming]);         else emissions->a[jModel][jSpecies] = 0; 
            if (iSmoldering >= 0) emissions->b[jModel][jSpecies] = atof(args[iSmoldering]);      else emissions->b[jModel][jSpecies] = 0; 
            if (iResidual >= 0)   emissions->c[jModel][jSpecies] = atof(args[iResidual]);        else emissions->c[jModel][jSpecies] = 0;

            fgets(buffer, 120, infile);
        }
        fclose(infile);

        emissions->nModels=nModels;
        emissions->nSpecies=nSpecies;
        
        return (nSpecies);
    }
    else
        return(-1);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////

int ReadEmissionsModels(char *infilename, EMODELS *emodels)
{
    char buffer[120];
    char *token, *args[10], *fields[10];
    int i, j, k, found, iColumn, maxfields, iCWFIS, iemodel, irho2, irho4, irho6, irho8;

    for (i=0;i<MAX_CWFIS;i++)
        emodels->FuelType[i] = (char *)calloc(4, sizeof(char));
    
    j=0; emodels->nModels = 0;
    
    maxfields=0;
    iCWFIS = iemodel = irho2 = irho4 = irho6 = irho8 = -999;  // these track the column number of fields within the csv (-999 = absent)
    
    for (i=0;i<10;i++)
    {
        args[i] = (char *)calloc(12, sizeof(char));  
        fields[i] =(char *)calloc(12, sizeof (char));        
    }
    
    if ( (infile = fopen(infilename, "r")) != NULL)
    {
        fgets(buffer, 120, infile);// header
        iColumn=0;
        
// read in the header
        token = strtok(buffer, ",");
        while (token != NULL && iColumn < 12)
        {
            strcpy(fields[iColumn], token);
            token = strtok(NULL, ",");

// keep track of the fields wrt columns (crude allowance for case)
            if (strstr(fields[iColumn], "CWFIS")) iCWFIS = iColumn;
            if (strstr(fields[iColumn], "emodel")) iemodel = iColumn;
            if (strstr(fields[iColumn], "rho2")) irho2= iColumn;
            if (strstr(fields[iColumn], "rho4")) irho4 = iColumn;
            if (strstr(fields[iColumn], "rho6")) irho6 = iColumn;
            if (strstr(fields[iColumn], "rho8")) irho8 = iColumn;            
            iColumn++;
        }

        maxfields=iColumn;

        fgets(buffer, 120, infile);// first line of data

        while( !feof( infile ) && j < MAX_CWFIS)
        {
            iColumn=0; 
            
            token = strtok(buffer, ",");

            while (token != NULL && iColumn < maxfields)
            {
                strcpy(args[iColumn], token);
                token = strtok(NULL, ",");
                iColumn++;
            }
            
            emodels->rho2[j] = emodels->rho4[j] = emodels->rho6[j] = emodels->rho8[j] = -999.;
            
            if (iCWFIS >= 0) strncpy(emodels->FuelType[j], args[iCWFIS], 3);
            if (iemodel >= 0) emodels->emodel[j] = atoi(args[iemodel]);
            if (irho2 >= 0) emodels->rho2[j] = atof(args[irho2]);
            if (irho4 >= 0) emodels->rho4[j] = atof(args[irho4]);
            if (irho6 >= 0) emodels->rho6[j] = atof(args[irho6]);
            if (irho8 >= 0) emodels->rho8[j] = atof(args[irho8]);
            
            fgets(buffer, 120, infile);
            j++;
        }
        fclose(infile);
        return(j);
    }
    else
        return(-1);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////

int ReadAllocationTable(char *infilename, ALLOCATIONS *allocations)     /* KRA: 2019-09-25 */
{
    
    char buffer[120];
    char *token, *args[10], *fields[10];
    int i, j, k, found, iColumn, maxfields, iFuel, iFlaming, iSmoldering, iResidual;
    
    
/* Defaults */
    allocations->SFC_L[0] = 0.9;
    allocations->SFC_L[1] = 0.1;
    allocations->SFC_L[2] = 0.0;
    allocations->SFC_F[0] = 0.1;
    allocations->SFC_F[1] = 0.7;
    allocations->SFC_F[2] = 0.2;
    allocations->SFC_H[0] = 0.0;
    allocations->SFC_H[1] = 0.2;
    allocations->SFC_H[2] = 0.8;

    allocations->CFC_F[0] = 0.94;
    allocations->CFC_F[1] = 0.06;
    allocations->CFC_F[2] = 0.0;
    allocations->CFC_B[0] = 0.6; // 0.4 left unburned
    allocations->CFC_B[1] = 0.0;
    allocations->CFC_B[2] = 0.0;
    allocations->CFC_S[0] = 0.25; // 0.5 left unburned
    allocations->CFC_S[1] = 0.25;
    allocations->CFC_S[2] = 0.0;

    allocations->FFC[0] = 1.0;
    allocations->FFC[1] = 0.0;
    allocations->FFC[2] = 0.0;
 
    allocations->WFC[0] = 0.4;
    allocations->WFC[1] = 0.3;
    allocations->WFC[2] = 0.3;

    allocations->GFC[0] = 0.95;
    allocations->GFC[1] = 0.05;
    allocations->GFC[2] = 0.0;
    
// printf("Reading Allocation Table: %s\n\n", infilename);

    j=0; // allocation->nModels = 0;
    
    maxfields=0;
    iFuel = iFlaming = iSmoldering = iResidual = -999;  // these track the column number of fields within the csv (-999 = absent)
    
    for (i=0;i<10;i++)
    {
        args[i] = (char *)calloc(12, sizeof(char));  
        fields[i] =(char *)calloc(12, sizeof (char));        
    }
    
    if ( (infile = fopen(infilename, "r")) != NULL)
    {
        fgets(buffer, 120, infile);// header
        iColumn=0;
        
// read in the header
        token = strtok(buffer, ",");
        while (token != NULL && iColumn < 12)
        {
            strcpy(fields[iColumn], token);
            token = strtok(NULL, ",");

// keep track of the fields wrt columns (no allowance for case)
            if (strstr(fields[iColumn], "Fuel")) iFuel = iColumn;
            if (strstr(fields[iColumn], "Flaming")) iFlaming = iColumn;
            if (strstr(fields[iColumn], "Smoldering")) iSmoldering= iColumn;
            if (strstr(fields[iColumn], "Residual")) iResidual = iColumn;
            iColumn++;
        }

        maxfields=iColumn;

        fgets(buffer, 120, infile);// first line of data
        
        while( !feof( infile ) && j < MAX_ALLOCATIONS)
        {
            iColumn=0; 
            
            token = strtok(buffer, ",");

            while (token != NULL && iColumn < maxfields)
            {
                strcpy(args[iColumn], token);
                token = strtok(NULL, ",");
                iColumn++;
            }
            
            if (iFuel >= 0)
            {
                if (!strncasecmp(args[iFuel], "SFC_L", 5) ||
                    !strncasecmp(args[iFuel], "litter", 3) )
                {
                    allocations->SFC_L[0] = atof(args[iFlaming]);
                    allocations->SFC_L[1] = atof(args[iSmoldering]);
                    allocations->SFC_L[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "SFC_F", 5) ||
                    !strncasecmp(args[iFuel], "fermented", 3)  ||
                    !strncasecmp(args[iFuel], "fibric", 3)  ||
                    !strncasecmp(args[iFuel], "fragmented", 3) )
                {
                    allocations->SFC_F[0] = atof(args[iFlaming]);
                    allocations->SFC_F[1] = atof(args[iSmoldering]);
                    allocations->SFC_F[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "SFC_H", 5) ||
                    !strncasecmp(args[iFuel], "humic", 3) )
                {
                    allocations->SFC_H[0] = atof(args[iFlaming]);
                    allocations->SFC_H[1] = atof(args[iSmoldering]);
                    allocations->SFC_H[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "CFC_F", 5) ||
                    !strncasecmp(args[iFuel], "foliage", 3) )
                {
                    allocations->CFC_F[0] = atof(args[iFlaming]);
                    allocations->CFC_F[1] = atof(args[iSmoldering]);
                    allocations->CFC_F[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "CFC_B", 5) ||
                    !strncasecmp(args[iFuel], "branch", 3) )
                {
                    allocations->CFC_B[0] = atof(args[iFlaming]);
                    allocations->CFC_B[1] = atof(args[iSmoldering]);
                    allocations->CFC_B[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "CFC_S", 5) ||
                    !strncasecmp(args[iFuel], "snag", 3) )
                {
                    allocations->CFC_S[0] = atof(args[iFlaming]);
                    allocations->CFC_S[1] = atof(args[iSmoldering]);
                    allocations->CFC_S[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "FFC", 3) ||
                    !strncasecmp(args[iFuel], "fine", 3) )
                {
                    allocations->FFC[0] = atof(args[iFlaming]);
                    allocations->FFC[1] = atof(args[iSmoldering]);
                    allocations->FFC[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "WFC", 3) ||
                    !strncasecmp(args[iFuel], "woody", 3) )
                {
                    allocations->WFC[0] = atof(args[iFlaming]);
                    allocations->WFC[1] = atof(args[iSmoldering]);
                    allocations->WFC[2] = atof(args[iResidual]);
                }
                if (!strncasecmp(args[iFuel], "GFC", 3) ||
                    !strncasecmp(args[iFuel], "grass", 3) )
                {
                    allocations->GFC[0] = atof(args[iFlaming]);
                    allocations->GFC[1] = atof(args[iSmoldering]);
                    allocations->GFC[2] = atof(args[iResidual]);
                }
            }

            fgets(buffer, 120, infile);
            j++;
        }

        fclose(infile);
        
        return(j);
    }
    else
        return(-1);
}

int ReadDiurnal(char *infilename, FEPS *feps)       /* KRA: 2019-09-25 */
{
    char buffer[120];
    char *token;
    int i, itime, status;
    double w, total, min;

/* this is an artifical diurnal trend for fire growth */
static double weight[] =
                  { 0.015648395,
                    0.012470576,
                    0.010137905,
                    0.008443153,
                    0.004159725,
                    0.003641906,
                    0.004455177,
                    0.005716244,
                    0.017972534,
                    0.032938579,
                    0.043543306,
                    0.055667556,
                    0.065778522,
                    0.078207403,
                    0.090168462,
                    0.104189127,
                    0.101294339, // adjusted to make the total = 1
                    0.091901405,
                    0.072798822,
                    0.057406612,
                    0.044004006,
                    0.033647664,
                    0.025825035,
                    0.019983547 };
					
					/* this is an artifical diurnal trend for fire growth */

static double agweight[] =
                  { 0.0,
					0.0,
					0.0,
					0.0,
					0.0,
					0.0,
					0.0,
					0.0,
					0.0,
					0.041666667,
					0.041666667,
					0.083333333,
					0.083333333,
					0.125,
					0.166666667,
					0.125,
					0.083333333,
					0.083333333,
					0.083333333,
					0.041666667,
					0.041666667,
					0.0,
					0.0,
					0.0 };	// added 2023-05-04

    if ( (infile = fopen(infilename, "r")) != NULL)
    {
        fgets(buffer, 120, infile);// header
        total=0.;
        min=0.; // handle negative weights?
        
        fgets(buffer, 120, infile);// first line of data
        while( !feof( infile ) )
        {
           token = strtok(buffer, ",");
           itime = atoi(token);
           token = strtok(NULL, ",");
           w = atof(token);
           if (w < min) min=w;  // only happens if w is negative

           while(itime>=24) itime=itime-24;
           while(itime<0) itime=itime+24;
            
            feps->weight[itime] = w;
            fgets(buffer, 120, infile);// first line of data
        }
        
        for (i=0; i<24; i++)
        {
            feps->weight[i] = feps->weight[i] -  min; // min is an offset to manage negative values
            total = total + feps->weight[i];
        }
        
        for (i=0; i<24; i++)
        {
            feps->weight[i] = feps->weight[i]/total;
        }
        status = 0;
        fclose(infile);
        
    }
    else
    {
        for (i=0; i<24; i++)
        {
            feps->weight[i] = weight[i];
        }
        status = -1;
    }

    for (i=0; i<24; i++)	// added 2023-05-04
    {
        feps->agweight[i] = agweight[i];
    }
}


int reademissionsdata_(char *infilename, EMISSIONS *emissions, int *status)
{
//    int *status;
    *status = ReadEmissionsData(infilename, emissions);
    return(*status);
}


int reademissionsmodels_(char *infilename, EMODELS *emodels, int *status)
{
//    int *status;
    *status = ReadEmissionsModels(infilename, emodels);
    return(*status);
}

